unit Pendula;

// C Code from billp245 obtained from :
//
//  https://www.physicsforums.com/threads/double-pendulum-in-c-trace-motion-real-time-gif-output.818983/
//
// There were no copyright messages available from there.
//
//
// Converted to Delphi by Blue Hell

{

   COPYRIGHT 2020 .. 2022 Blue Hell / Jan Punter (for the modifications only)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 2 as
  published by the Free Software Foundation;

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  For all listed email addresses :

    _dot. to be substituted by a dot      '.'
    2@t2  to be substituted by an at sign '@'


  Blue Hell is a trade mark owned by

    Jan Punter
    https://bluehell.nl/
    jan2@t2mail_dot_bluehell_dot_nl
}


interface

uses

  System.Math,

  KnobsUtils, KnobsConversions;

type

  TPendulum = class
  strict private
  const
    INCREMENT = 0.0175;
  private
    type TPendulumState = record
      FSpeed : TSignal;
      FGMod  : TSignal;
      FNoise : TSignal;
      FL1    : TSignal;
      FL2    : TSignal;
      FM1    : TSignal;
      FM2    : TSignal;
      FTh1   : TSignal;
      FTh2   : TSignal;
      FT1d   : TSignal;
      FT2d   : TSignal;
    end;
    PPendulumState = ^ TPendulumState;
  private
    FPolyphony : Integer;
    FState     : array of TPendulumState;
  private
    function    GetL1   ( aVoice: Integer): TSignal;
    procedure   SetL1   ( aVoice: Integer; aValue: TSignal);
    function    GetL2   ( aVoice: Integer): TSignal;
    procedure   SetL2   ( aVoice: Integer; aValue: TSignal);
    function    GetM1   ( aVoice: Integer): TSignal;
    procedure   SetM1   ( aVoice: Integer; aValue: TSignal);
    function    GetM2   ( aVoice: Integer): TSignal;
    procedure   SetM2   ( aVoice: Integer; aValue: TSignal);
    function    GetSpeed( aVoice: Integer): TSignal;
    procedure   SetSpeed( aVoice: Integer; aValue: TSignal);
    function    GetGMod ( aVoice: Integer): TSignal;
    procedure   SetGMod ( aVoice: Integer; aValue: TSignal);
    function    GetNoise( aVoice: Integer): TSignal;
    procedure   SetNoise( aVoice: Integer; aValue: TSignal);
  private
    function    Jitter( aVoice: Integer): TSignal;
    function    th1_xder( aVoice: Integer; t1d: TSignal): TSignal;
    function    th1_yder( aVoice: Integer; t2d: TSignal): TSignal;
    function    th2_xder( aVoice: Integer; t1d, th1, t2d, th2, l1, l2, m1, m2: TSignal): TSignal;
    function    th2_yder( aVoice: Integer; t1d, th1, t2d, th2, l1, l2, m1, m2: TSignal): TSignal;
  public
    constructor Create( aPolyphony: Integer; l1, l2, m1, m2: TSignal);
    procedure   Reset( aVoice: Integer);
    procedure   Step( aVoice: Integer; out x1, y1, x2, y2, ph1, ph2: TSignal; fetch_only: Boolean);
  public
    property    Polyphony               : Integer read FPolyphony;
    property    L1   [ aVoice: Integer] : TSignal read GetL1      write SetL1;
    property    L2   [ aVoice: Integer] : TSignal read GetL2      write SetL2;
    property    M1   [ aVoice: Integer] : TSignal read GetM1      write SetM1;
    property    M2   [ aVoice: Integer] : TSignal read GetM2      write SetM2;
    property    Speed[ aVoice: Integer] : TSignal read GetSpeed   write SetSpeed;
    property    GMod [ aVoice: Integer] : TSignal read GetGMod    write SetGMod;
    property    Noise[ aVoice: Integer] : TSignal read GetNoise   write SetNoise;
  end;


implementation


{ ========
  TPendulum = class
  private
    type TPendulumState = record
      FSpeed : TSignal;
      FGMod  : TSignal;
      FNoise : TSignal;
      FL1    : TSignal;
      FL2    : TSignal;
      FM1    : TSignal;
      FM2    : TSignal;
      FTh1   : TSignal;
      FTh2   : TSignal;
      FT1d   : TSignal;
      FT2d   : TSignal;
    end;
    PPendulumState = ^ TPendulumState;
  private
    FPolyphony : Integer;
    FState     : array of TPendulumState;
  public
    property    Polyphony               : Integer read FPolyphony;
    property    L1   [ aVoice: Integer] : TSignal read GetL1      write SetL1;
    property    L2   [ aVoice: Integer] : TSignal read GetL2      write SetL2;
    property    M1   [ aVoice: Integer] : TSignal read GetM1      write SetM1;
    property    M2   [ aVoice: Integer] : TSignal read GetM2      write SetM2;
    property    Speed[ aVoice: Integer] : TSignal read GetSpeed   write SetSpeed;
    property    GMod [ aVoice: Integer] : TSignal read GetGMod    write SetGMod;
    property    Noise[ aVoice: Integer] : TSignal read GetNoise   write SetNoise;
  private
}

    function    TPendulum.GetL1( aVoice: Integer): TSignal;
    begin
      Result := FState[ aVoice].FL1;
    end;


    procedure   TPendulum.SetL1( aVoice: Integer; aValue: TSignal);
    begin
      FState[ aVoice].FL1 := aValue;
    end;


    function    TPendulum.GetL2( aVoice: Integer): TSignal;
    begin
      Result := FState[ aVoice].FL2;
    end;


    procedure   TPendulum.SetL2( aVoice: Integer; aValue: TSignal);
    begin
      FState[ aVoice].FL2 := aValue;
    end;


    function    TPendulum.GetM1( aVoice: Integer): TSignal;
    begin
      Result := FState[ aVoice].FM1;
    end;


    procedure   TPendulum.SetM1( aVoice: Integer; aValue: TSignal);
    begin
      FState[ aVoice].FM1 := aValue;
    end;


    function    TPendulum.GetM2( aVoice: Integer): TSignal;
    begin
      Result := FState[ aVoice].FM2;
    end;


    procedure   TPendulum.SetM2( aVoice: Integer; aValue: TSignal);
    begin
      FState[ aVoice].FM2 := aValue;
    end;


    function    TPendulum.GetSpeed( aVoice: Integer): TSignal;
    begin
      Result := FState[ aVoice].FSpeed;
    end;


    procedure   TPendulum.SetSpeed( aVoice: Integer; aValue: TSignal);
    begin
      FState[ aVoice].FSpeed := aValue;
    end;


    function    TPendulum.GetGMod( aVoice: Integer): TSignal;
    begin
      Result := FState[ aVoice].FGMod;
    end;


    procedure   TPendulum.SetGMod( aVoice: Integer; aValue: TSignal);
    begin
      FState[ aVoice].FGMod := aValue;
    end;


    function    TPendulum.GetNoise( aVoice: Integer): TSignal;
    begin
      Result := FState[ aVoice].FNoise;
    end;


    procedure   TPendulum.SetNoise( aVoice: Integer; aValue: TSignal);
    begin
      FState[ aVoice].FNoise := aValue;
    end;


//  private

    function    TPendulum.Jitter( aVoice: Integer): TSignal;
    const
      FIVEPERC = 5.0 / 100.0;     // +/- 5% so 10% in total
    begin
      Result := 1.0 + FIVEPERC * Noise[ aVoice] * ( Random - 2.0 * Random);
    end;


    function    TPendulum.th1_xder( aVoice: Integer; t1d: TSignal): TSignal;
    begin
      Result := t1d * INCREMENT * Speed[ aVoice] * Jitter( aVoice);
    end;


    function    TPendulum.th1_yder( aVoice: Integer; t2d: TSignal): TSignal;
    begin
      Result := t2d * INCREMENT * Speed[ aVoice] * Jitter( aVoice);
    end;


    function    TPendulum.th2_xder( aVoice: Integer; t1d, th1, t2d, th2, l1, l2, m1, m2: TSignal): TSignal;
    begin
      Result :=
        INCREMENT * Speed[ aVoice] * Jitter( aVoice) *
        (
          (( STANDARD_GRAVITY * GMod[ aVoice]) / l1) * ( ( m2 / ( m1 + m2)) * Sin( th2) * Cos( th1 - th2) - Sin( th1))
          - ( m2 / ( m1 + m2)) * Sin( th1 - th2) * (( l2 / l1) * t2d * t2d + t1d * t1d * Cos( th1 - th2))
        )
        / ( 1 - (( m2 / ( m1 + m2)) * Cos( th1 - th2) * Cos( th1 - th2)));
    end;


    function    TPendulum.th2_yder( aVoice: Integer; t1d, th1, t2d, th2, l1, l2, m1, m2: TSignal): TSignal;
    begin
      Result :=
        INCREMENT * Speed[ aVoice] * Jitter( aVoice) *
        (
          (( STANDARD_GRAVITY * GMod[ aVoice]) / l2) * ( Sin( th1) * Cos( th1 - th2) - Sin( th1))
          + Sin( th1 - th2) * (( l1 / l2) * t1d * t1d + ( m2 / ( m1 + m2)) * t2d * t2d * Cos( th1 - th2))
        )
        / ( 1 - (( m2 / ( m1 + m2)) * Cos( th1 - th2) * Cos( th1 - th2)));
    end;


//  public

    constructor TPendulum.Create( aPolyphony: Integer; l1, l2, m1, m2: TSignal);
    var
      zz    : Integer;
      State : PPendulumState;
    begin
      FPolyphony := aPolyphony;
      SetLength( FState, Polyphony);

      for zz := 0 to Polyphony - 1
      do begin
        State := @ FState[ zz];
        State^.FL1 := l1;
        State^.FL2 := l2;
        State^.FM1 := m1;
        State^.FM2 := m2;
        Reset( zz);
      end;
    end;


    procedure   TPendulum.Reset( aVoice: Integer);
    var
      State : PPendulumState;
    begin
      State := @ FState[ aVoice];
      State^.FSpeed := 1.0;
      State^.FGMod  := 1.0;
      State^.FTh1   := 90.0 * PI / 180.0;
      State^.FTh2   :=  0.0 * PI / 180.0;
      State^.FT1d   :=  0.0;
      State^.FT2d   :=  0.0;
    end;


    procedure   TPendulum.Step( aVoice: Integer; out x1, y1, x2, y2, ph1, ph2: TSignal; fetch_only: Boolean);
    var
      State          : PPendulumState;
      k1, k2, k3, k4 : TSignal;
      q1, q2, q3, q4 : TSignal;
      r1, r2, r3, r4 : TSignal;
      s1, s2, s3, s4 : TSignal;
    begin
      State := @ FState[ aVoice];

      if not fetch_only
      then begin
        k1 := th1_xder( aVoice, State^.FT1d);
        q1 := th1_yder( aVoice, State^.FT2d);
        r1 := th2_xder( aVoice, State^.FT1d, State^.FTh1, State^.FT2d, State^.FTh2, State^.FL1, State^.FL2, State^.FM1, State^.FM2);
        s1 := th2_yder( aVoice, State^.FT1d, State^.FTh1, State^.FT2d, State^.FTh2, State^.FL1, State^.FL2, State^.FM1, State^.FM2);

        k2 := th1_xder( aVoice,  State^.FT1d + ( r1 * 0.5));
        q2 := th1_yder( aVoice,  State^.FT2d + ( s1 * 0.5));
        r2 := th2_xder( aVoice, ( State^.FT1d + ( r1 * 0.5)), ( State^.FTh1 + ( k1 * 0.5)), ( State^.FT2d + ( s1 * 0.5)), ( State^.FTh2 + ( q1 * 0.5)), State^.FL1, State^.FL2, State^.FM1, State^.FM2);
        s2 := th2_yder( aVoice, ( State^.FT1d + ( r1 * 0.5)), ( State^.FTh1 + ( k1 * 0.5)), ( State^.FT2d + ( s1 * 0.5)), ( State^.FTh2 + ( q1 * 0.5)), State^.FL1, State^.FL2, State^.FM1, State^.FM2);

        k3 := th1_xder( aVoice,  State^.FT1d + ( r2 * 0.5));
        q3 := th1_yder( aVoice,  State^.FT2d + ( s2 * 0.5));
        r3 := th2_xder( aVoice, ( State^.FT1d + ( r2 * 0.5)), ( State^.FTh1 + ( k2 * 0.5)), ( State^.FT2d + ( s2 * 0.5)), ( State^.FTh2 + ( q2 * 0.5)), State^.FL1, State^.FL2, State^.FM1, State^.FM2);
        s3 := th2_yder( aVoice, ( State^.FT1d + ( r2 * 0.5)), ( State^.FTh1 + ( k2 * 0.5)), ( State^.FT2d + ( s2 * 0.5)), ( State^.FTh2 + ( q2 * 0.5)), State^.FL1, State^.FL2, State^.FM1, State^.FM2);

        k4 := th1_xder( aVoice,  State^.FT1d + r3);
        q4 := th1_yder( aVoice,  State^.FT2d + s3);
        r4 := th2_xder( aVoice, ( State^.FT1d + r3), ( State^.FTh1 + k3), ( State^.FT2d + s3), ( State^.FTh2 + q3), State^.FL1, State^.FL2, State^.FM1, State^.FM2);
        s4 := th2_yder( aVoice, ( State^.FT1d + r3), ( State^.FTh1 + k3), ( State^.FT2d + s3), ( State^.FTh2 + q3), State^.FL1, State^.FL2, State^.FM1, State^.FM2);

        State^.FT1d := State^.FT1d + ( r1 + 2.0 * r2 + 2.0 * r3 + r4) / 6.0;
        State^.FT2d := State^.FT2d + ( s1 + 2.0 * s2 + 2.0 * s3 + s4) / 6.0;
        State^.FTh1 := State^.FTh1 + ( k1 + 2.0 * k2 + 2.0 * k3 + k4) / 6.0;
        State^.FTh2 := State^.FTh2 + ( q1 + 2.0 * q2 + 2.0 * q3 + q4) / 6.0;
      end;

      State^.FTh1 := MathFloatMod( State^.FTh1, TWO_PI);
      State^.FTh2 := MathFloatMod( State^.FTh2, TWO_PI);

      x1  :=      State^.FL1 * Sin( State^.FTh1);
      y1  :=    - State^.FL1 * Cos( State^.FTh1);
      x2  := x1 + State^.FL2 * Sin( State^.FTh2);
      y2  := y1 - State^.FL2 * Cos( State^.FTh2);
      ph1 := ( State^.FTh1 - PI) * PI_REC;
      ph2 := ( State^.FTh2 - PI) * PI_REC;
    end;


end.
