unit FrmDllMain;

interface

uses

  Winapi.Windows, Winapi.Messages, WinApi.ShellApi, Winapi.CommCtrl,

  System.SysUtils, System.Variants, System.Classes, System.Math, System.UITypes, System.IniFiles, System.Types,
  System.StrUtils,

  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, Vcl.StdCtrls, Vcl.ExtCtrls, Vcl.ComCtrls,
  Vcl.Buttons, Vcl.Menus, Vcl.Imaging.jpeg,

  KnobsConversions, KnobsUtils, Globals, wave_dlg, KnobParser, module_defs, module_scales, module_implementations,
  knobs2013, Rationals, wave_tools, synth_info, ctypes, fftw_, Talkie, grains, FormantControl, Speech, SpeechLib_TLB,
  ESpeakInterface, HrastUnit, sieves, AudioCaching,

  FrmStore, FrmMidiCC, FrmRandomizer, FrmLastEdits, FrmSplash, FrmFolderSelector,

  Vst3Base;


const

  MIN_DB        = -100.0;             // Minimum dB value to be shown on Main VU meters
  MAX_DB        =   20.0;             // Maximum dB value to be shown on Main VU meters
  DB_DECAY_RATE =    1.0;             // Decay rate for peak indicators on VU meters (seconds)


type

  TColorSetter = procedure( aColor: TColor) of object;
  TColorGetter = function: TColor           of object;

  TColorId = (
    cidEditorBackground    ,
    cidModule              ,
    cidConnectorBorder     ,
    cidSelector            ,
    cidSelectorBorder      ,
    cidSelectorBorderSingle,
    cidEditFont            ,
    cidDisplay             ,
    cidDisplayBorder       ,
    cidKnobMIDI            ,
    cidKnobFocus           ,
    cidKnobMIDIFocus       ,
    cidViewer              ,
    cidViewerBorder        ,
    cidViewerLine          ,
    cidViewerFill          ,
    cidScopeLine           ,
    cidIndicatorBarPeak    ,
    cidIndicatorBarValey   ,
    cidAudioRate           ,
    cidControlRate         ,
    cidControlRateLogic    ,
    cidLogic               ,
    cidUnconnectedWire     ,
    cidWireHghlight        ,
    cidGroupIO             ,
    cidGroupOSC            ,
    cidGroupLFO            ,
    cidGroupEnv            ,
    cidGroupFilter         ,
    cidGroupSwitch         ,
    cidGroupMix            ,
    cidGroupControl        ,
    cidGroupNote           ,
    cidGroupMath           ,
    cidGroupLogic          ,
    cidGroupSeq            ,
    cidGroupGenerator      ,
    cidGroupFX             ,
    cidGroupDelay          ,
    cidGroupUtility
  );

  TColorMap = record
    Id     : TColorId;
    Name   : string;
    Setter : TColorSetter;
    Getter : TColorGetter;
  end;

  TColorMaps = array of TColorMap;

  TPageControl = class( Vcl.ComCtrls.TPageControl)
  // Some stuff to fix the border width for TPageControls
  private
    procedure   TCMAdjustRect(var Msg: TMessage); message TCM_ADJUSTRECT;
  end;


  TEdit = class( Vcl.StdCtrls.TEdit)
  // Fix VK_DELETE issues for TEdits
  // Fix TextHint color
  private
    procedure    CNKeyDown( var aMessage: TWMKeyDown); message CN_KEYDOWN;
    function     GetTextHintColor: TColor;
    procedure    SetTextHintColor( aColor: TColor);
    procedure    FixWndProc( var aMessage: TMessage);
  public
    property     TextHintColor : TColor read GetTextHintColor write SetTextHintColor;
  end;


  TFormDllMain = class(TForm)
    PanelMain: TPanel;
    PageControlMain: TPageControl;
    TabSheetEditor: TTabSheet;
    PanelEditorTop: TPanel;
    PanelEditorTopRight: TPanel;
    PanelTemplates: TPanel;
    PanelRunControls: TPanel;
    ModuleSelector: TKnobsModuleSelector;
    PatchEditor: TKnobsWirePanel;
    TabSheetSettings: TTabSheet;
    PanelSettings: TPanel;
    ScrollBoxSettings: TScrollBox;
    GroupBoxEditorSettings: TGroupBox;
    LabelLightsRate: TLabel;
    LabelMaxFileHistory: TLabel;
    LabelNewPNT: TLabel;
    LabelSynthName: TLabel;
    LabelProgramVersion: TLabel;
    LabelPatchVersion: TLabel;
    CheckBoxReloadLastPatch: TCheckBox;
    CheckBoxAutoRun: TCheckBox;
    KnobsSmallKnob_LightsRate: TKnobsSmallKnob;
    KnobsDisplayLightsRate: TKnobsDisplay;
    KnobsDisplayHistoryCount: TKnobsDisplay;
    KnobsNoKnob_HistoryCount: TKnobsNoKnob;
    CheckBoxSearchPatch: TCheckBox;
    CheckBoxSearchSelector: TCheckBox;
    CheckBoxWarnOnStructMismatch: TCheckBox;
    CheckBoxAllowAutoLoad: TCheckBox;
    EditNewPatchNameTemplate: TEdit;
    EditSynthName: TEdit;
    BitBtnAcknowledgements: TBitBtn;
    BitBtnDocument: TBitBtn;
    GroupBoxLooks: TGroupBox;
    LabelwheelSensitivity: TLabel;
    LabelMarginLeft: TLabel;
    LabelMarginTop: TLabel;
    LabelEditorMargins: TLabel;
    CheckBoxCurvedLines: TCheckBox;
    RadioGroupControlMode: TRadioGroup;
    CheckBoxUseWheelOnKnobs: TCheckBox;
    knob_WheelSensitivity: TKnobsSmallKnob;
    KnobsDisplayWheelSensitivity: TKnobsDisplay;
    CheckBoxUseUnFocus: TCheckBox;
    CheckBoxUseConfigHints: TCheckBox;
    CheckBoxUseModuleTitleHints: TCheckBox;
    CheckBoxUsePatchHints: TCheckBox;
    CheckBoxUseNoteNames: TCheckBox;
    KnobsDisplayOffsetLeft: TKnobsDisplay;
    KnobsDisplayOffsetTop: TKnobsDisplay;
    KnobsSmallKnob_LeftOffset: TKnobsNoKnob;
    KnobsSmallKnob_TopOffset: TKnobsNoKnob;
    RadioGroupControlDecimation: TRadioGroup;
    GroupBoxTextWriter: TGroupBox;
    LabelTextWriterPrefix: TLabel;
    EditTextWriterPrefix: TEdit;
    CheckBoxPrependPrefix: TCheckBox;
    CheckBoxUseTextWriter: TCheckBox;
    TabSheetDebug: TTabSheet;
    MemoDebug: TMemo;
    PanelDebugControl: TPanel;
    LabelExceptions: TLabel;
    LabelLightsTime: TLabel;
    Labelexcepts: TLabel;
    LabelTextLightsTime: TLabel;
    LabelLockSkips: TLabel;
    LabelTextLockSkips: TLabel;
    LabelRTExceptions: TLabel;
    LabelRTExcepts: TLabel;
    BitBtnClear: TBitBtn;
    TimerStatsUpdate: TTimer;
    SaveDialogPatch: TSaveDialog;
    OpenDialogPatch: TOpenDialog;
    OpenDialogWave: TOpenDialog;
    PopupMenuConnector: TPopupMenu;
    DisconnectConnector: TMenuItem;
    BreakWire: TMenuItem;
    DeleteConnector: TMenuItem;
    N5: TMenuItem;
    Redwires1: TMenuItem;
    Bluewires1: TMenuItem;
    Yellowwires1: TMenuItem;
    Makegreen1: TMenuItem;
    Customwirecolor1: TMenuItem;
    Defaultwirecolor1: TMenuItem;
    PopupMenuModule: TPopupMenu;
    MenuItemModuleColor: TMenuItem;
    MenuItemModuleDefaultcolor: TMenuItem;
    N11: TMenuItem;
    MenuItemModuleHelp: TMenuItem;
    MenuItemModuleHistory: TMenuItem;
    N12: TMenuItem;
    MenuItemModuleCopy: TMenuItem;
    MenuItemModuleCut: TMenuItem;
    MenuItemModuleDelete: TMenuItem;
    MenuItemModulePasteParamsStrict: TMenuItem;
    N14: TMenuItem;
    MenuItemModuleSaveAsPreset: TMenuItem;
    MenuItemModuleLoadFromPreset: TMenuItem;
    N20: TMenuItem;
    MenuItemModuleRandomize: TMenuItem;
    MenuItemModuleExludeFromRandomization: TMenuItem;
    PopupMenuKnob: TPopupMenu;
    KnobMIDICC: TMenuItem;
    AssigntoCC: TMenuItem;
    UnassignCC: TMenuItem;
    N7: TMenuItem;
    SetKnobDefaultvalue: TMenuItem;
    LockKnob: TMenuItem;
    MenuItemKnobHistory: TMenuItem;
    N21: TMenuItem;
    MenuItemKnobRandomize: TMenuItem;
    MenuItemKnobExcludeFromRandomization: TMenuItem;
    TimerLightsUpdate: TTimer;
    PopupMenuSelector: TPopupMenu;
    dynamics1: TMenuItem;
    SaveDialogWave: TSaveDialog;
    ColorDialogModule: TColorDialog;
    FontDialog: TFontDialog;
    SaveDialogPreset: TSaveDialog;
    OpenDialogPreset: TOpenDialog;
    PopupMenuEditor: TPopupMenu;
    MenuItemInsertmodule: TMenuItem;
    N17: TMenuItem;
    cut1: TMenuItem;
    copy1: TMenuItem;
    paste1: TMenuItem;
    pastevalues1: TMenuItem;
    delete1: TMenuItem;
    N18: TMenuItem;
    MenuItemEditorHistory: TMenuItem;
    selectall1: TMenuItem;
    invertselection1: TMenuItem;
    selectnone1: TMenuItem;
    PopupMenuDataMaker: TPopupMenu;
    PopupMenuDataMakerSine: TMenuItem;
    PopupMenuDataMakerCosine: TMenuItem;
    PopupMenuDataMakerArcTan: TMenuItem;
    PopupMenuDataMakerBell: TMenuItem;
    PopupMenuDataMakerTri: TMenuItem;
    PopupMenuDataMakerSaw: TMenuItem;
    PopupMenuDataMakerSquare3: TMenuItem;
    PopupMenuDataMakerSquare10: TMenuItem;
    PopupMenuDataMakerSquare33: TMenuItem;
    PopupMenuDataMakerSquare50: TMenuItem;
    PopupMenuDataMakerSquare67: TMenuItem;
    PopupMenuDataMakerSquare90: TMenuItem;
    PopupMenuDataMakerSquare97: TMenuItem;
    PopupMenuDataMakerNoise: TMenuItem;
    N9: TMenuItem;
    PopupMenuDataMakerClear: TMenuItem;
    PopupMenuDataMakerSwapLeftRight: TMenuItem;
    PopupMenuDataMakerSwapTopBottom: TMenuItem;
    PopupMenuDataMakerRotate180: TMenuItem;
    PopupMenuDataMakerMakeloopable: TMenuItem;
    KMakefullscale1: TMenuItem;
    N16: TMenuItem;
    PopupMenuDataMakerSavetofile: TMenuItem;
    PopupMenuDataMakerLoadfromfile: TMenuItem;
    N22: TMenuItem;
    MenuItemDataMakerRandomize: TMenuItem;
    MenuItemDataMakerExcludefromrandomization: TMenuItem;
    SaveDialogGraphs: TSaveDialog;
    OpenDialogGraphs: TOpenDialog;
    PopupMenuXYControl: TPopupMenu;
    MenuItemXYControlRandomize: TMenuItem;
    MenuItemXYControlExcludefromrandomization: TMenuItem;
    TimerLiveMorphVisuals: TTimer;
    SaveDialogGridControl: TSaveDialog;
    OpenDialogGridControl: TOpenDialog;
    PopupMenuGridControl: TPopupMenu;
    MenuItemGridControlSaveToFile: TMenuItem;
    MenuItemGridControlLoadFromFile: TMenuItem;
    PopupMenuTextControl: TPopupMenu;
    TextMenuEdit: TMenuItem;
    TextMenuClear: TMenuItem;
    PopupMenuControl: TPopupMenu;
    MenuItemShowRunControls: TMenuItem;
    MenuItemShowMorphControls: TMenuItem;
    MenuItemShowMutationControls: TMenuItem;
    MenuItemShowTuningControls: TMenuItem;
    FileOpenDialogSelectFolders: TFileOpenDialog;
    LabelOverlayOpacity: TLabel;
    KnobsSmallKnobModuleOpacity: TKnobsSmallKnob;
    KnobsDisplayModuleOpacity: TKnobsDisplay;
    BitBtnCheckForUpdates: TBitBtn;
    PanelMenu: TPanel;
    BitBtnShowEditor: TBitBtn;
    BitBtnShowSettings: TBitBtn;
    BitBtnShowDebug: TBitBtn;
    BitBtnShowFileMenu: TBitBtn;
    BitBtnShowEditMenu: TBitBtn;
    BitBtnShowActionMenu: TBitBtn;
    BitBtnShowHelp: TBitBtn;
    BitBtnFolderSetup: TBitBtn;
    PopupMenuFile: TPopupMenu;
    PopupMenuFileNew: TMenuItem;
    PopupMEnuFileN1: TMenuItem;
    PopupMenuFileOpenPatch: TMenuItem;
    PopupMenuFileN2: TMenuItem;
    PopupMenuFileSavepatch: TMenuItem;
    EditSearch: TEdit;
    UpDownWireThickness1: TUpDown;
    LabelWires: TLabel;
    KnobsSelectorReset: TKnobsSelector;
    PopupMenuAction: TPopupMenu;
    PopupMenuEdit: TPopupMenu;
    PopupMenuFileReopenPatch: TMenuItem;
    PopupMenuFileLoadlatestpatch: TMenuItem;
    PopupMenuFileLoadpatchtemplate1: TMenuItem;
    PopupMenuFileLoadpatchtemplate2: TMenuItem;
    PopupMEnuFileLoadpatchpreset: TMenuItem;
    PopupMenuFileImportmodules: TMenuItem;
    PopupMenuFileSavepatchas: TMenuItem;
    PopupMenuFileSavepatchastemplate1: TMenuItem;
    PopupMenuFileSavepatchastemplate2: TMenuItem;
    PopupMEnuFileSavepatchastemplatenew: TMenuItem;
    PopupMEnuFileSavepatchpreset: TMenuItem;
    PopupMEnuFileExportselectedmodules: TMenuItem;
    PopupMenuEditCut: TMenuItem;
    PopupMenuEditCopy: TMenuItem;
    PopupMenuEditPaste: TMenuItem;
    PopupMenuEditPastevalues: TMenuItem;
    Delete2: TMenuItem;
    PopupMenuEditN1: TMenuItem;
    PopupMenuEditFindmodules: TMenuItem;
    PopupMenuEditFindunconnectedmodules: TMenuItem;
    PopupMenuEditN2: TMenuItem;
    PopupMenuEditSelectall: TMenuItem;
    PopupMenuEditInvertselection: TMenuItem;
    PopupMenuEditSelectnone: TMenuItem;
    PopupMenuEditN3: TMenuItem;
    PopupMenuEditUndo: TMenuItem;
    PopupMenuEditRedo: TMenuItem;
    PopupMenuActionReset: TMenuItem;
    PopupMenuActionRecompile: TMenuItem;
    PopupMenuActionTogglelights: TMenuItem;
    PopupMenuActionPanic: TMenuItem;
    PopupMenuActionWigglewires: TMenuItem;
    PopupMenuActionTogglewires: TMenuItem;
    EditPark: TEdit;
    UpDownPolyphony: TUpDown;
    LabelPolyphony: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    function ModuleSelectorGetGlyph(const aSender: TObject; const aBitmapsFolder, aName: string): TBitmap;
    procedure ModuleSelectorModuleButtonClick(const aSender: TObject; aModuleclass: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint;
      var Handled: Boolean);
    procedure knob_iolevel_UnFocus(const aSender: TObject);
    procedure UpDownWireThickness1Click(Sender: TObject; Button: TUDBtnType);
    procedure TimerStatsUpdateTimer(Sender: TObject);
    procedure TimerLightsUpdateTimer(Sender: TObject);
    procedure TimerLiveMorphVisualsTimer(Sender: TObject);
    procedure BitBtnAcknowledgementsClick(Sender: TObject);
    procedure BitBtnDocumentClick(Sender: TObject);
    procedure BitBtnCheckForUpdatesClick(Sender: TObject);
    procedure KnobsNoKnob_HistoryCountValueChanged(const aSender: TObject; const aPath, aControlType: string;
      aValue: Double; IsFinal, IsAutomation: Boolean);
    procedure KnobsSmallKnob_LightsRateValueChanged(const aSender: TObject; const aPath, aControlType: string;
      aValue: Double; IsFinal, IsAutomation: Boolean);
    procedure BitBtnShowEditorClick(Sender: TObject);
    procedure BitBtnShowSettingsClick(Sender: TObject);
    procedure BitBtnShowDebugClick(Sender: TObject);
    procedure BitBtnShowFileMenuClick(Sender: TObject);
    procedure BitBtnShowEditMenuClick(Sender: TObject);
    procedure BitBtnShowActionMenuClick(Sender: TObject);
    procedure BitBtnShowHelpClick(Sender: TObject);
    procedure BitBtnFolderSetupClick(Sender: TObject);
    procedure PopupMenuFileNewClick(Sender: TObject);
    procedure PopupMenuFileOpenPatchClick(Sender: TObject);
    procedure PopupMenuFileLoadlatestpatchClick(Sender: TObject);
    procedure PopupMenuFileLoadpatchtemplate1Click(Sender: TObject);
    procedure PopupMenuFileLoadpatchtemplate2Click(Sender: TObject);
    procedure PopupMEnuFileLoadpatchpresetClick(Sender: TObject);
    procedure PopupMenuFileImportmodulesClick(Sender: TObject);
    procedure PopupMenuFileSavepatchClick(Sender: TObject);
    procedure PopupMenuFileSavepatchasClick(Sender: TObject);
    procedure PopupMenuFileSavepatchastemplate1Click(Sender: TObject);
    procedure PopupMenuFileSavepatchastemplate2Click(Sender: TObject);
    procedure PopupMEnuFileSavepatchastemplatenewClick(Sender: TObject);
    procedure PopupMEnuFileSavepatchpresetClick(Sender: TObject);
    procedure PopupMEnuFileExportselectedmodulesClick(Sender: TObject);
    procedure PopupMenuActionResetClick(Sender: TObject);
    procedure PopupMenuActionRecompileClick(Sender: TObject);
    procedure PopupMenuActionTogglelightsClick(Sender: TObject);
    procedure PopupMenuActionPanicClick(Sender: TObject);
    procedure PopupMenuEditCutClick(Sender: TObject);
    procedure PopupMenuEditCopyClick(Sender: TObject);
    procedure PopupMenuEditPasteClick(Sender: TObject);
    procedure PopupMenuEditPastevaluesClick(Sender: TObject);
    procedure Delete2Click(Sender: TObject);
    procedure PopupMenuEditFindmodulesClick(Sender: TObject);
    procedure PopupMenuEditFindunconnectedmodulesClick(Sender: TObject);
    procedure PopupMenuEditSelectallClick(Sender: TObject);
    procedure PopupMenuEditInvertselectionClick(Sender: TObject);
    procedure PopupMenuEditSelectnoneClick(Sender: TObject);
    procedure PopupMenuEditUndoClick(Sender: TObject);
    procedure PopupMenuEditRedoClick(Sender: TObject);
    procedure PopupMenuActionWigglewiresClick(Sender: TObject);
    procedure PopupMenuActionTogglewiresClick(Sender: TObject);
    procedure DisconnectConnectorClick(Sender: TObject);
    procedure BreakWireClick(Sender: TObject);
    procedure DeleteConnectorClick(Sender: TObject);
    procedure Redwires1Click(Sender: TObject);
    procedure Bluewires1Click(Sender: TObject);
    procedure Yellowwires1Click(Sender: TObject);
    procedure Makegreen1Click(Sender: TObject);
    procedure Defaultwirecolor1Click(Sender: TObject);
    procedure Customwirecolor1Click(Sender: TObject);
    procedure MenuItemModuleColorClick(Sender: TObject);
    procedure MenuItemModuleDefaultcolorClick(Sender: TObject);
    procedure MenuItemModuleHelpClick(Sender: TObject);
    procedure MenuItemModuleCopyClick(Sender: TObject);
    procedure MenuItemModuleCutClick(Sender: TObject);
    procedure MenuItemModuleDeleteClick(Sender: TObject);
    procedure MenuItemModulePasteParamsStrictClick(Sender: TObject);
    procedure SetKnobDefaultvalueClick(Sender: TObject);
    procedure LockKnobClick(Sender: TObject);
    procedure cut1Click(Sender: TObject);
    procedure copy1Click(Sender: TObject);
    procedure paste1Click(Sender: TObject);
    procedure pastevalues1Click(Sender: TObject);
    procedure delete1Click(Sender: TObject);
    procedure selectall1Click(Sender: TObject);
    procedure invertselection1Click(Sender: TObject);
    procedure selectnone1Click(Sender: TObject);
    procedure PopupMenuDataMakerClick(Sender: TObject);
    procedure MenuItemModuleSaveAsPresetClick(Sender: TObject);
    procedure MenuItemModuleLoadFromPresetClick(Sender: TObject);
    procedure UpDownPolyphonyClick(Sender: TObject; Button: TUDBtnType);
  private
    FOrigCaption              : string;
    FHistoryCount             : Integer;                      // Number of files to hold in file history
    FRecentSourceFiles        : TRecentStrings;               // The file history itself
    FLooksFolder              : string;                       // Glyphs & stuff
    FIniLoaded                : Boolean;                      // True when an ini file was actually read, don't save it when it was not ..
    FIniSaved                 : Boolean;                      // True when. on exit, the ini file was saved
    FClosing                  : Boolean;                      // FormClose was called .. synth gets locked, can be used to stop some async stuff
    FLastWiresVisible         : Boolean;
  private
    FLogToFile                : Boolean;                      // To indicate wherther we are logging to a disk file
    FSavedLogs                : TStringList;                  // Used to store log strings when logging is still disabled.
    FCanLog                   : Boolean;                      // Indicates whether logging is enableed or not
    FRegErrors                : Boolean;
    FInFormShow               : Boolean;
    FColorMaps                : TColorMaps;                   // All custom colors and their names
    FCurrentColor             : TColorId;                     // The currently selected color items
  private
    FPolyphony                : Integer;                      // Current polyphony selected
    FSampleCount              : Int64;                        // Number of samples processed, being reset on patch compilation
    FLightsTime               : TSignal;                      // Time spent in lights updater
  private
    FFFTWAvailable            : Boolean;                      // True when the FFTW DLL was found to be present and usable
    FESpeaker                 : TESpeakSpeaker;               // Interface to the espeaj API
    FESpeakAvailable          : Boolean;                      // True whe eSpeak's espeak-sapidll was found, FESpeaker will be non-nil then too
  private
    FMazeList                 : TStringList;
    FSapiVoiceNames           : TStringList;
    FSapiRateNames            : TStringList;
  private
    FPatchReader              : IKnobsPatchReader;            // Patch reader
    FPatchWriter              : IKnobsPatchWriter;            // Patch writer
    FSynthesizer              : TSynthesizer;                 // The sound engine
  private
    FLightsActive             : Boolean;                      // Debug thingie for turning lights updates on / off
  private
    FAudioCacheSize           : UInt64;
    FStateChangeCount         : Integer;
    FAudioRefList             : TAudioRefList;
    FRequestImport            : Boolean;                      // Flag to signal file import request
    FRequestedImportName      : string;                       // Name of file requested to be imported
    FUpdateCounter            : Integer;
  private
    FTextWriterPrefix         : string;                       // Prefix text to use on the TextWriter module's output strings
    FPrependPrefix            : Boolean;                      // Whether to use the TexgtWriterPrefix or not, there also is %prefix% now.
    FUseTextWriter            : Boolean;                      // Whether to use TextWriter file updates at all - oof by default.
    FTextFileName             : string;
    FTextFileString           : string;
    FNewPatchNameTemplate     : string;                       // Template to use for names of newly created patches
    FPopupSender              : TControl;                     // The control that caused a popup menu to be shown
  private
    FUseConfigHints           : Boolean;                      // Whether to use popup hints on the config screens
    FUseModuleTitleHints      : Boolean;                      // Whether to use popup hints on module titles
    FUsePatchHints            : Boolean;                      // Whether to use hints on module controls in the patch
    FWireThickness            : Integer;                      // Pen size for wire drawing
    FCurvedLines              : Boolean;                      // True for curved lines instead of straight ones
    FControlMode              : TDistanceMode;                // Knob control mode (circular, horizontal, vertical)
    FLightsRate               : TSignal;                      // Lights update rate in the patch editor
    FUseWheelOnKnobs          : Boolean;                      // Use the mouse wheel to control knobs
    FWheelSensitivity         : Integer;                      // The amount of mouse wheel action needed for a ceratin knob movement
    FEditorBackgroundColor    : TColor;                       // Color of the patch editor pane
    FStandardColors           : Boolean;                      // When true use the standard color on new modules added.
    FUseConnectorBorders      : Boolean;                      // Whether to draw a border around connectors or not
    FUseTypedConnectorBorders : Boolean;                      // Whether to give connectors a border colored acording to signal type or not
    FUseNoteNames             : Boolean;                      // Whether note names should be used in frequency displays instead of frequencies
    FReloadLastPatch          : Boolean;                      // True when on program start the last patch must be reloaded
    FDoAutoRun                : Boolean;                      // Autorun flag as read from INI file, used to delay autorun going active
    FAutoRun                  : Boolean;                      // True when audio processing must be turned on after program start
  private
    function    HandleContextMenu( const aControl: TControl): Boolean;
    procedure   ShowAcknowledgements;
    procedure   CheckForUpdates;
    procedure   SelectFolders;
    function    FixHtmlTokens( const S: string): string;
    procedure   ShowModuleHelp( aModule: TKnobsCustomModule);
    function    MustMakeDocs: Boolean;
    procedure   MakeDocs( const aFolder : string);
    procedure   ShowMenu      ( const aSender: TControl; const aPopupMenu: TPopupMenu);
    procedure   ShowFileMenu  ( const aSender: TControl);
    procedure   ShowEditMenu  ( const aSender: TControl);
    procedure   ShowActionMenu( const aSender: TControl);
    procedure   ShowHelp;
    procedure   LoadIni( const aFileName: string; LooksOnly: Boolean);
    procedure   SaveIni( const aFileName: string; LooksOnly: Boolean);
  private
    procedure   UpdateCaptions;
    procedure   DoModuleWarning( aType: TModWarningType; const aMsg: string);
    procedure   SetAudioCacheSize( aValue: UInt64);
    procedure   SetLightsActive( aValue: Boolean);
  private
    procedure   SetPolyphony               ( aValue: Integer);
    procedure   SetHistoryCount            ( aValue: Integer);
    procedure   SetUseWheelOnKnobs         ( aValue: Boolean);
    procedure   SetWheelSensitivity        ( aValue: Integer);
    procedure   SetUseConfigHints          ( aValue: Boolean);
    procedure   SetUseModuleTitleHints     ( aValue: Boolean);
    procedure   SetUsePatchHints           ( aValue: Boolean);
    procedure   SetWireThickness           ( aValue: Integer);
    procedure   SetCurvedLines             ( aValue: Boolean);
    procedure   SetControlMode             ( aValue: TDistanceMode);
    procedure   SetLightsRate              ( aValue: TSignal);
    function    GetOffsetLeft              : Integer;
    procedure   SetOffsetLeft              ( aValue: Integer);
    function    GetOffsetTop               : Integer;
    procedure   SetOffsetTop               ( aValue: Integer);
    function    GetModuleColor             : TColor;
    procedure   SetModuleColor             ( aValue: TColor);
    function    GetConnectorBorderColor    : TColor;
    procedure   SetConnectorBorderColor    ( aValue: TColor);
    function    GetSelectorColor           : TColor;
    procedure   SetSelectorColor           ( aValue: TColor);
    function    GetSelectorBorderColor     : TColor;
    procedure   SetSelectorBorderColor     ( aValue: TColor);
    function    GetSelectorBorderColorS    : TColor;
    procedure   SetSelectorBorderColorS    ( aValue: TColor);
    function    GetEditFontColor           : TColor;
    procedure   SetEditFontColor           ( aValue: TColor);
    function    GetDisplayColor            : TColor;
    procedure   SetDisplayColor            ( aValue: TColor);
    function    GetDisplayBorderColor      : TColor;
    procedure   SetDisplayBorderColor      ( aValue: TColor);
    function    GetKnobMIDIColor           : TColor;
    procedure   SetKnobMIDIColor           ( aValue: TColor);
    function    GetKnobFocusColor          : TColor;
    procedure   SetKnobFocusColor          ( aValue: TColor);
    function    GetKnobMIDIFocusColor      : TColor;
    procedure   SetKnobMIDIFocusColor      ( aValue: TColor);
    function    GetViewerColor             : TColor;
    procedure   SetViewerColor             ( aValue: TColor);
    function    GetViewerBorderColor       : TColor;
    procedure   SetViewerBorderColor       ( aValue: TColor);
    function    GetViewerLineColor         : TColor;
    procedure   SetViewerLineColor         ( aValue: TColor);
    function    GetViewerFillColor         : TColor;
    procedure   SetViewerFillColor         ( aValue: TColor);
    function    GetScopeLineColor          : TColor;
    procedure   SetScopeLineColor          ( aValue: TColor);
    function    GetIndBarPeakColor         : TColor;
    procedure   SetIndBarPeakColor         ( aValue: TColor);
    function    GetIndBarValeyColor        : TColor;
    procedure   SetIndBarValeyColor        ( aValue: TColor);
    function    GetGroupIOColor            : TColor;
    procedure   SetGroupIOColor            ( aValue: TColor);
    function    GetGroupOSCColor           : TColor;
    procedure   SetGroupOSCColor           ( aValue: TColor);
    function    GetGroupLFOColor           : TColor;
    procedure   SetGroupLFOColor           ( aValue: TColor);
    function    GetGroupEnvColor           : TColor;
    procedure   SetGroupEnvColor           ( aValue: TColor);
    function    GetGroupFilterColor        : TColor;
    procedure   SetGroupFilterColor        ( aValue: TColor);
    function    GetGroupSwitchColor        : TColor;
    procedure   SetGroupSwitchColor        ( aValue: TColor);
    function    GetGroupMixColor           : TColor;
    procedure   SetGroupMixColor           ( aValue: TColor);
    function    GetGroupControlColor       : TColor;
    procedure   SetGroupControlColor       ( aValue: TColor);
    function    GetGroupNoteColor          : TColor;
    procedure   SetGroupNoteColor          ( aValue: TColor);
    function    GetGroupMathColor          : TColor;
    procedure   SetGroupMathColor          ( aValue: TColor);
    function    GetGroupLogicColor         : TColor;
    procedure   SetGroupLogicColor         ( aValue: TColor);
    function    GetGroupSeqColor           : TColor;
    procedure   SetGroupSeqColor           ( aValue: TColor);
    function    GetGroupGeneratorColor     : TColor;
    procedure   SetGroupGeneratorColor     ( aValue: TColor);
    function    GetGroupFXColor            : TColor;
    procedure   SetGroupFXColor            ( aValue: TColor);
    function    GetGroupDelayColor         : TColor;
    procedure   SetGroupDelayColor         ( aValue: TColor);
    function    GetGroupVoiceColor         : TColor;
    procedure   SetGroupVoiceColor         ( aValue: TColor);
    function    GetGroupUtilityColor       : TColor;
    procedure   SetGroupUtilityColor       ( aValue: TColor);
    function    GetAudioRateColor          : TColor;
    procedure   SetAudioRateColor          ( aValue: TColor);
    function    GetControlRateColor        : TColor;
    procedure   SetControlRateColor        ( aValue: TColor);
    function    GetControlRateLogicColor   : TColor;
    procedure   SetControlRateLogicColor   ( aValue: TColor);
    function    GetLogicColor              : TColor;
    procedure   SetLogicColor              ( aValue: TColor);
    function    GetUnconnectedWireColor    : TColor;
    procedure   SetUnconnectedWireColor    ( aValue: TColor);
    function    GetWireHighlightColor      : TColor;
    procedure   SetWireHighlightColor      ( aValue: TColor);
    function    GetEditorBackgroundColor   : TColor;
    procedure   SetEditorBackgroundColor   ( aValue: TColor);
    procedure   SetStandardColors          ( aValue: Boolean);
    procedure   SetUseConnectorBorders     ( aValue: Boolean);
    procedure   SetUseTypedConnectorBorders( aValue: Boolean);
    procedure   SetUseNoteNames            ( aValue: Boolean);
    function    GetModuleOpacity           : Byte;
    procedure   SetModuleOpacity           ( aValue: Byte);
    function    GetModuleFlatness          : Boolean;
    procedure   SetModuleFlatness          ( aValue: Boolean);
    function    GetModuleTexture           : Boolean;
    procedure   SetModuleTexture           ( aValue: Boolean);
    procedure   SetReloadLastPatch         ( aValue: Boolean);
    procedure   SetAutoRun                 ( aValue: Boolean);
  private
    function    GetColorMapLength: Integer;
    procedure   SetColorMapLength( aValue: Integer);
    procedure   SetCurrentColor( aValue: TColorId);
//    function    GetColor( anIndex: TColorId): TColor;
//    procedure   SetColor( anIndex: TColorId; aValue: TColor);
    procedure   AddColorMap( anId: TColorId; const aName: string; const aGetter: TColorGetter; const aSetter: TColorSetter);
    procedure   FixColorMaps;
    procedure   ChangeColorId( anIndex: Integer);
//    procedure   ChangeCurrentColor( aColor: TColor);
  private
    procedure   ClearLog;
    function    RtcTimeStamp: string;
    procedure   WriteFileLog( const aMsg: string);
    procedure   Log( aLogClass: TLogClass; const aMsg: string; TimeStamped: Boolean = True);                                 overload;
    procedure   Log( aLogClass: TLogClass; const aMsgs: TStrings; DoClear: Boolean);                                         overload;
    procedure   LogFmt( aLogClass: TLogClass; const aFmt: string; const anArgs: array of const);
    procedure   DoSynthLog( const aSender: TObject; aLogClass: Integer; const aMsg: string);
//    procedure   LogRandomizer( const aSender: TObject; const aMsg: string);
//    procedure   LogVariations( const aSender: TObject; const aMsg: string);
    procedure   FormantLogger( const aSender: TObject; aLogClass: TLogClass; const aMsg: string);
    procedure   DoDesignerLog   ( const aSender: TObject; aLogClass: TLogClass; const aMsg: string);
    procedure   DoWirePanelLog  ( const aSender: TObject; aLogClass: TLogClass; const aMsg: string);
  private
    procedure   DoFormDesignerLog          ( const aMsg: string);
    function    GetSynthName               : string;
    procedure   SetSynthName               ( const aValue: string);
    procedure   SetCanLog                  ( aValue: Boolean);
  private
    procedure   AddModule( aModuleType: TKnobsModuleType);
    procedure   DoEditorMenuAddModule( aSender: TObject);
    procedure   DoShowHint( var aHintStr: string; var CanShow: Boolean; var aHintInfo: Vcl.Controls.THintInfo);
    function    DoCreateModuleBitmap( const aModuleType: TKnobsModuleType; UseCache: Boolean): TBitmap;
    function    DoReadModuleComment ( const aModuleType: TKnobsModuleType): string;
    procedure   DoPopupEditorShow( const aSender: TObject);
    procedure   DoPopupEditorHide( const aSender: TObject);
    procedure   DoHistoryItemClicked    ( aSender: TObject);
    procedure   RemoveSelectorMenuItems;
    procedure   DoSelectorMenuItemClick( aSender: TObject);
    procedure   AddSelectorMenuItem( anIndex, aShortcut: Integer; const aName: string);
    procedure   DoShowPopupMenu         ( const aSender: TKnobsWirePanel; const aControl: TControl);
    procedure   DoUnFocus               ( const aSender: TObject);
    function    DoLoadDataGraph         ( const aSender: TObject): string;
    procedure   DoSaveDataGraph         ( const aSender: TObject; const aValue: string);
    function    DoLoadGridControl       ( const aSender: TObject): string;
    procedure   DoSaveGridControl       ( const aSender: TObject; const aValue: string);
    procedure   DoActiveVariationChanged( const aSender: TObject; aVariation: Integer);
    procedure   DoEditHistoryChanged    ( aSender: TObject);
    procedure   DoModulesRenamed        ( const aSender: TObject; const anOldNames, aNewNames: TStringArray);
    procedure   DoModulesRenamedFriendly( const aSender: TObject; const anOldNames, aNewNames: TStringArray);
  private
    function    GetPatchName: string;
    procedure   SetPatchName( const aValue: string);
    function    GetSourceFileName: string;
    procedure   OpenRecentFile( const anIndex: Integer);
    procedure   DorecentMenuItemClick( aSender: TObject);
    procedure   AddRecentMenuItem( anIndex: Integer; const aFileName: string);
    procedure   RemoveRecentMenuItems;
    procedure   SetRecentMenuItems;
    procedure   SetMostRecentFile( const aFileName: string);
    procedure   SetSourceFileName( const aValue: string);
    function    FunctionalPinName( const aName: string): string;
    function    SplitConnectorName( const aName: string; var aMod, aPin: string): Boolean;
    function    ModuleTypeToModule( aType: Integer; const aParent: TSynthesizer; const aName: string; aVersion, aPolyphony: Integer): TMod;
    procedure   CompilePatch( ModulesChanged, AutomaticCompile: Boolean);
    procedure   DoRecompile     ( const aSender: TKnobsWirePanel; ModulesChanged: Boolean);
    function    ChangeSynthesizerTo( const aSynthesizer: TSynthesizer; ModulesChanged: Boolean): Boolean;
    procedure   DoHistoryChanged( const aSender: TObject; anUndoCount, aRedoCount, aSavedMarker: Integer);
    function    DoGetFolder     ( const aSender: TObject; aFolderKind: TKnobsFolderKind): string;
    procedure   DoValueChanged  ( const aSender: TObject; const aPath, aControlType: string; aValue: TSignal; IsFinal, IsAutomation: Boolean);
    procedure   DoTextChanged   ( const aSender: TObject; const aPath, aValue: string);
    function    SplitValueInfo      ( const aPath: string; var aModule: TKnobsCustomModule; var aModuleType, aSignalName: string): Boolean;
    procedure   HandleSpecialValues ( const aPath, aControlType: string; aValue: TSignal; const anOrigin: TKnobsValuedControl);
    procedure   ChangeRangeFor      ( const aModule: TKnobsCustomModule; const aModuleType, aRangeType: string; aValue: TSignal; const anOrigin: TKnobsValuedControl);
    procedure   ChangeModeFor       ( const aModule: TKnobsCustomModule; const aModuleType, aModeType : string; aValue: TSignal);
    procedure   ChangeControlTypeTo ( const aControl: TKnobsValuedControl; const aValue: string );
    procedure   ChangeKnobPositionTo( const aControl: TKnobsValuedControl; const aValue: Integer);
    procedure   ChangeStepCountTo   ( const aControl: TKnobsValuedControl; const aValue: Integer);
    procedure   ChangePairingModeTo ( const aKnob   : TKnobsKnob; aValue: TKnobsPairingMode);
  private
    procedure   FixLocalControls;
    procedure   FixTabColors;
    procedure   ViewEditor;
    procedure   ViewDebug;
    procedure   ViewSettings;
    procedure   LoadModuleGlyphs;
  private
    procedure   UpdateStats;
    procedure   UpdateLights;
  private
    procedure   SetTextWriterPrefix     ( const aValue: string);
    procedure   SetPrependPrefix        ( aValue: Boolean);
    procedure   SetUseTextWriter        ( aValue: Boolean);
    function    SubstituteTextMacros    ( const S: string): string;
    procedure   SetFileText             ( const aFileName, aLine: string);
    procedure   SetFileTextAgain;
    procedure   SetNewPatchNameTemplate ( const aValue: string);
    procedure   DoSignals               ( const aSender: TMod; const anInfo: TLightsInfo);
    procedure   DoLights                ( const aSender: TMod; const anInfo: TLightsInfo);
    procedure   DoData                  ( const aSender: TMod; const anInfo: TDataInfo  );
    procedure   DoXYData                ( const aSender: TMod; const anInfo: TXYDataInfo);
    procedure   DoStringData            ( const aSender: TMod; const anInfo: TStringInfo);
    procedure   DoCursorData            ( const aSender: TMod; const anInfo: TCursorInfo);
    procedure   DoWaveData              ( const aSender: TMod; const anInfo: TWaveInfo  );
  private
    procedure   DoLoadCaptions( const aSender: TObject; const aModule: TKnobsCustomModule; const aControl: TKnobsSelector; const aDependency: TKnobsValuedControl);
    function    CreateSpeechVoiceNames: TStringList;
    function    CreateSpeechRateNames: TStringList;
    procedure   FixModuleStatics;
    procedure   FixDynamicConverter( const aName: string; const aValues: TStringList);
    function    FixTalkieStatics        ( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    function    FixGranulatorStatics    ( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    function    FixSeqRandomStatics     ( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    function    FixScaleQuantizeStatics ( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    function    FixProbSequencerStatics ( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    function    FixFormant2Statics      ( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    function    FixModal2Statics        ( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    function    FixMazeTypeStatics      ( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    function    FixSapiSpeechTypeStatics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    function    FixSpeechRateTypeStatics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    function    FixESpeakVoiceStatics   ( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    function    FixESpeakLanguageStatics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
  private
    function    GetFolderPatches        : string;
    function    GetFolderTemplates      : string;
    function    GetFolderSamplesWaves   : string;
    function    GetFolderRecordings     : string;
    function    GetFolderScala          : string;
    function    GetFolderMidiFiles      : string;
    function    GetFolderPresets        : string;
    function    GetFolderExportsImports : string;
    function    GetFolderCsvFiles       : string;
    function    GetFolderGraphs         : string;
    function    GetFolderGrids          : string;
    function    GetFolderGenes          : string;
    function    GetFolderTextWriter     : string;
  private
    procedure   rlg( const aMsg: string);                     // short for RegistrationLog ...
    procedure   RegisterModuleTypes;
  private
    procedure   DoAudioChanged          ( const aSender: TObject; const anOldFileName, aNewFileName: string);
    procedure   DoBeginStateChange      ( const aSender: TObject);
    procedure   DoEndStateChange        ( const aSender: TObject);
    function    Template1Name  : string;
    function    Template2Name  : string;
    function    TemplateNewName: string;
    function    CheckDiscardPatchChanges( const aReason: string; MustAsk: Boolean): Boolean;
    function    LastPatchName: string;
    procedure   SaveLastPatch;
    procedure   LoadLastPatch;
    function    SavePatchNamed( const aFolder, aFileName: string; WriteAlways: Boolean)  : Boolean;
    function    SavePatch  ( const aFolder: string): Boolean;
    function    SavePatchAs( const aFolder: string): Boolean;
    function    SaveAsTemplate( const aFileName: string): Boolean;
    function    SaveAsTemplate1  : Boolean;
    function    SaveAsTemplate2  : Boolean;
    function    SaveAsTemplateNew: Boolean;
    procedure   ExportModules;
    procedure   DoLoadPatch( const aFileName: string; ChangePatchName: Boolean);
    procedure   DoImportPatch( const aFileName: string);
    procedure   LoadPatch;
    procedure   ImportPatch;
    procedure   NewPatch;
    procedure   LoadPatchPreset;
    procedure   SavePatchPreset;
    procedure   DoOnValuedControlRemoved( const aSender: TKnobsValuedControl);
    procedure   DeleteModules;
    procedure   CutModules;
    procedure   CopyModules;
    procedure   PasteModules;
    procedure   PasteParameters( StrictCopy: Boolean);
    procedure   SelectAll;
    procedure   SelectNone;
    procedure   InvertSelection;
    procedure   WiggleWires;
    procedure   ToggleWires;
    function    WiresOff: Boolean;
    procedure   WiresOn( TurnOn: Boolean);
    procedure   Panic;
    procedure   FindUnconnectedModules;
    function    GetDefaultModuleColor    ( const aSender: TObject; const aModuleType: TKnobsModuleType): TColor;
    procedure   ConnectorDelete          ( const aConnector    : TKnobsConnector    );
    procedure   ConnectorBreak           ( const aConnector    : TKnobsConnector    );
    procedure   ConnectorDisconnect      ( const aConnector    : TKnobsConnector    );
    procedure   SetWireColor             ( const aConnector    : TKnobsConnector; const aColor: TColor);
    procedure   SetWireColorDefault      ( const aConnector    : TKnobsConnector    );
    procedure   SetWireColorCustom       ( const aConnector    : TKnobsConnector    );
    procedure   ModuleCut                ( const aModule       : TKnobsModule       );
    procedure   ModuleCopy               ( const aModule       : TKnobsModule       );
    procedure   ModuleDelete             ( const aModule       : TKnobsModule       );
    procedure   ModulePasteParams        ( const aModule       : TKnobsModule; StrictCopy: Boolean);
    procedure   ChangeModuleColor        ( const aModule       : TKnobsModule       );
    procedure   ModuleColorDefault       ( const aModule       : TKnobsModule       );
    procedure   KnobSetDefaultValue      ( const aValuedControl: TKnobsValuedControl);
    procedure   KnobChangeLock           ( const aKnob         : TKnobsKnob         );
    procedure   LoadPreset( aModule: TKnobsCustomModule);
    procedure   SavePreset( aModule: TKnobsCustomModule);
    procedure   FillDataMaker( const aDataMaker: TKnobsDataMaker; aShape: TKnobsDMShape);
  private
    property    Polyphony                : Integer       read FPolyphony                 write SetPolyphony;
    property    PatchName                : string        read GetPatchName               write SetPatchName;
    property    SynthName                : string        read GetSynthName               write SetSynthName;
    property    SourceFileName           : string        read GetSourceFileName          write SetSourceFileName;
    property    CanLog                   : Boolean       read FCanLog                    write SetCanLog;
    property    AudioCacheSize           : UInt64        read FAudioCacheSize            write SetAudioCacheSize;
    property    LightsActive             : Boolean       read FLightsActive              write SetLightsActive;
    property    TextWriterPrefix         : string        read FTextWriterPrefix          write SetTextWriterPrefix;
    property    PrependPrefix            : Boolean       read FPrependPrefix             write SetPrependPrefix;
    property    UseTextWriter            : Boolean       read FUseTextWriter             write SetUseTextWriter;
    property    NewPatchNameTemplate     : string        read FNewPatchNameTemplate      write SetNewPatchNameTemplate;
  private
    property    FolderPatches            : string        read GetFolderPatches       ;
    property    FolderTemplates          : string        read GetFolderTemplates     ;
    property    FolderSamplesWaves       : string        read GetFolderSamplesWaves  ;
    property    FolderRecordings         : string        read GetFolderRecordings    ;
    property    FolderScala              : string        read GetFolderScala         ;
    property    FolderMidiFiles          : string        read GetFolderMidiFiles     ;
    property    FolderPresets            : string        read GetFolderPresets       ;
    property    FolderExportsImports     : string        read GetFolderExportsImports;
    property    FolderCsvFiles           : string        read GetFolderCsvFiles      ;
    property    FolderGraphs             : string        read GetFolderGraphs        ;
    property    FolderGrids              : string        read GetFolderGrids         ;
    property    FolderGenes              : string        read GetFolderGenes         ;
    property    FolderTextWriter         : string        read GetFolderTextWriter    ;
  private
    property    HistoryCount             : Integer       read FHistoryCount              write SetHistoryCount;
    property    UseWheelOnKnobs          : Boolean       read FUseWheelOnKnobs           write SetUseWheelOnKnobs;
    property    WheelSensitivity         : Integer       read FWheelSensitivity          write SetWheelSensitivity;
    property    UseConfigHints           : Boolean       read FUseConfigHints            write SetUseConfigHints;
    property    UseModuleTitleHints      : Boolean       read FUseModuleTitleHints       write SetUseModuleTitleHints;
    property    UsePatchHints            : Boolean       read FUsePatchHints             write SetUsePatchHints;
    property    WireThickness            : Integer       read FWireThickness             write SetWireThickness;
    property    CurvedLines              : Boolean       read FCurvedLines               write SetCurvedLines;
    property    ControlMode              : TDistanceMode read FControlMode               write SetControlMode;
    property    LightsRate               : TSignal       read FLightsRate                write SetLightsRate;
    property    OffsetLeft               : Integer       read GetOffsetLeft              write SetOffsetLeft;
    property    OffsetTop                : Integer       read GetOffsetTop               write SetOffsetTop;
    property    ModuleColor              : TColor        read GetModuleColor             write SetModuleColor;
    property    ConnectorBorderColor     : TColor        read GetConnectorBorderColor    write SetConnectorBorderColor;
    property    SelectorColor            : TColor        read GetSelectorColor           write SetSelectorColor;
    property    SelectorBorderColor      : TColor        read GetSelectorBorderColor     write SetSelectorBorderColor;
    property    SelectorBorderColorS     : TColor        read GetSelectorBorderColorS    write SetSelectorBorderColorS;
    property    EditFontColor            : TColor        read GetEditFontColor           write SetEditFontColor;
    property    DisplayColor             : TColor        read GetDisplayColor            write SetDisplayColor;
    property    DisplayBorderColor       : TColor        read GetDisplayBorderColor      write SetDisplayBorderColor;
    property    KnobMIDIColor            : TColor        read GetKnobMIDIColor           write SetKnobMIDIColor;
    property    KnobFocusColor           : TColor        read GetKnobFocusColor          write SetKnobFocusColor;
    property    KnobMIDIFocusColor       : TColor        read GetKnobMIDIFocusColor      write SetKnobMIDIFocusColor;
    property    ViewerColor              : TColor        read GetViewerColor             write SetViewerColor;
    property    ViewerBorderColor        : TColor        read GetViewerBorderColor       write SetViewerBorderColor;
    property    ViewerLineColor          : TColor        read GetViewerLineColor         write SetViewerLineColor;
    property    ViewerFillColor          : TColor        read GetViewerFillColor         write SetViewerFillColor;
    property    ScopeLineColor           : TColor        read GetScopeLineColor          write SetScopeLineColor;
    property    IndBarPeakColor          : TColor        read GetIndBarPeakColor         write SetIndBarPeakColor;
    property    IndBarValeyColor         : TColor        read GetIndBarValeyColor        write SetIndBarValeyColor;
    property    GroupIOColor             : TColor        read GetGroupIOColor            write SetGroupIOColor;
    property    GroupOSCColor            : TColor        read GetGroupOSCColor           write SetGroupOSCColor;
    property    GroupLFOColor            : TColor        read GetGroupLFOColor           write SetGroupLFOColor;
    property    GroupEnvColor            : TColor        read GetGroupEnvColor           write SetGroupEnvColor;
    property    GroupFilterColor         : TColor        read GetGroupFilterColor        write SetGroupFilterColor;
    property    GroupSwitchColor         : TColor        read GetGroupSwitchColor        write SetGroupSwitchColor;
    property    GroupMixColor            : TColor        read GetGroupMixColor           write SetGroupMixColor;
    property    GroupControlColor        : TColor        read GetGroupControlColor       write SetGroupControlColor;
    property    GroupNoteColor           : TColor        read GetGroupNoteColor          write SetGroupNoteColor;
    property    GroupMathColor           : TColor        read GetGroupMathColor          write SetGroupMathColor;
    property    GroupLogicColor          : TColor        read GetGroupLogicColor         write SetGroupLogicColor;
    property    GroupSeqColor            : TColor        read GetGroupSeqColor           write SetGroupSeqColor;
    property    GroupGeneratorColor      : TColor        read GetGroupGeneratorColor     write SetGroupGeneratorColor;
    property    GroupFXColor             : TColor        read GetGroupFXColor            write SetGroupFXColor;
    property    GroupDelayColor          : TColor        read GetGroupDelayColor         write SetGroupDelayColor;
    property    GroupVoiceColor          : TColor        read GetGroupVoiceColor         write SetGroupVoiceColor;
    property    GroupUtilityColor        : TColor        read GetGroupUtilityColor       write SetGroupUtilityColor;
    property    AudioRateColor           : TColor        read GetAudioRateColor          write SetAudioRateColor;
    property    ControlRateColor         : TColor        read GetControlRateColor        write SetControlRateColor;
    property    ControlRateLogicColor    : TColor        read GetControlRateLogicColor   write SetControlRateLogicColor;
    property    LogicColor               : TColor        read GetLogicColor              write SetLogicColor;
    property    UnconnectedWireColor     : TColor        read GetUnconnectedWireColor    write SetUnconnectedWireColor;
    property    WireHighlightColor       : TColor        read GetWireHighlightColor      write SetWireHighlightColor;
    property    EditorBackgroundColor    : TColor        read FEditorBackgroundColor     write SetEditorBackgroundColor;
    property    StandardColors           : Boolean       read FStandardColors            write SetStandardColors;
    property    UseConnectorBorders      : Boolean       read FUseConnectorBorders       write SetUseConnectorBorders;
    property    UseTypedConnectorBorders : Boolean       read FUseTypedConnectorBorders  write SetUseTypedConnectorBorders;
    property    UseNoteNames             : Boolean       read FUseNoteNames              write SetUseNoteNames;
    property    ModuleOpacity            : Byte          read GetModuleOpacity           write SetModuleOpacity;
    property    ModuleFlatness           : Boolean       read GetModuleFlatness          write SetModuleFlatness;
    property    ModuleTexture            : Boolean       read GetModuleTexture           write SetModuleTexture;
    property    ReloadLastPatch          : Boolean       read FReloadLastPatch           write SetReloadLastPatch;
    property    AutoRun                  : Boolean       read FAutoRun                   write SetAutoRun;
    property    ColorMapLength           : Integer       read GetColorMapLength          write SetColorMapLength;
    property    CurrentColor             : TColorId      read FCurrentColor              write SetCurrentColor;
  public
  end;

var

  FormDllMain: TFormDllMain;


implementation


// User area

    function IndexToMenuShortCut( aShortcutIndex: Integer): string;
    const
      ShortcutSeparator : string = '   ';
    begin
      if   aShortCutIndex >= 0
      then begin
        if   aShortcutIndex <= 9
        then Result := '&' + Char( aShortcutIndex + Ord( '0')) + ShortcutSeparator
        else if aShortcutIndex <= 35
        then Result := '&' + Char( aShortcutIndex + Ord( 'A') - 10) + ShortcutSeparator
        else Result := '  ' + ShortcutSeparator;
      end
      else Result := '';
    end;


{ ========
  TPageControl = class( Vcl.ComCtrls.TPageControl)
  // Some stuff to fix the border width for TPageControls
  private
}

    procedure   TPageControl.TCMAdjustRect( var Msg: TMessage); // message TCM_ADJUSTRECT;
    begin
      inherited;
      if   Msg.WParam = 0
      then InflateRect( PRect( Msg.LParam)^,  4,  4)
      else InflateRect( PRect( Msg.LParam)^, -4, -4);
    end;


{ ========
  TEdit = class( Vcl.StdCtrls.TEdit)
  // Fix VK_DELETE issues for TEdits
  // Fix TextHint color
  public
    property     TextHintColor : TColor read GetTextHintColor write SetTextHintColor;
  private
}

    procedure    TEdit.CNKeyDown( var aMessage: TWMKeyDown); // message CN_KEYDOWN;
    // Copied from TWinControl.CNKeyDown - but left out the check for IsShortcut
    // as that one was 'stealing away' the menu shortcut keys resulting in the DEL
    // key (VK_DELETE) not working for editing the editor's text. Now this control
    // behaves like there are no menu shortcuts at all, and that is what is wanted
    // here. Then added some checks to make it apply only to certain controls
    // under certain conditions .. oddly enough VK_DELETE works fine on tabs being
    // not the main tab ... so apparently it is the PopupMenuEditor causing the
    // issue with the delete key (and not the MainMenu, but both have DEL as a
    // shortcut key). Anyways .. this solves the issues .. clumsy though.
    var
      Mask        : Integer;
      aShiftState : TShiftState;
      aCharCode   : Word;
    begin
      aShiftState := KeyDataToShiftState( aMessage.KeyData);
      aCharCode   := aMessage.CharCode;

      if   Assigned( FormDllMain)
      and  ( aShiftState = [])
      and  ( aCharCode   = VK_DELETE)
      and
      (
           FormDllMain.EditSearch.Focused
      )
      then begin
        with aMessage
        do begin
          Result := 1;
          UpdateUIState( aMessage.CharCode);

          if   Perform( CM_CHILDKEY, CharCode, Winapi.Windows.LPARAM( Self)) <> 0
          then Exit;

          Mask := 0;

          case CharCode of
            VK_TAB:                                      Mask := DLGC_WANTTAB;
            VK_LEFT, VK_RIGHT, VK_UP, VK_DOWN:           Mask := DLGC_WANTARROWS;
            VK_RETURN, VK_EXECUTE, VK_ESCAPE, VK_CANCEL: Mask := DLGC_WANTALLKEYS;
          end;

          if   ( Mask <> 0)
          and  ( Perform( CM_WANTSPECIALKEY, CharCode, 0) = 0)
          and  ( Perform( WM_GETDLGCODE, 0, 0) and Mask = 0)
          and  ( GetParentForm( Self).Perform( CM_DIALOGKEY, CharCode, KeyData) <> 0)
          then Exit;

          Result := 0;
        end;
      end
      else inherited;
    end;


    function     TEdit.GetTextHintColor: TColor;
    begin
      Result := Tag;
    end;


    procedure    TEdit.SetTextHintColor( aColor: TColor);
    begin
      if   aColor <> Tag
      then begin
        Tag        := aColor;
        WindowProc := FixWndProc;
      end;
    end;


    procedure    TEdit.FixWndProc( var aMessage: TMessage);
    var
      DC           : HDC;
      R            : TRect;
      OldFont      : HFONT;
      OldTextColor : TColorRef;
      OldBackColor : TColorRef;
      Handled      : Boolean;
    begin
      Handled      := False;
      OldFont      := 0;
      OldTextColor := 0;
      OldBackColor := 0;

      if   Assigned( FormDllMain)
      and  ( aMessage.Msg = WM_PAINT)
      and  ( Text         = ''      )
      and  not Focused
      then begin
        Self.WndProc( aMessage);
        self.Perform( EM_GETRECT, 0, LParam( @ R));
        DC := GetDC( Handle);

        try
          OldFont      := SelectObject( DC, Font.Handle);
          OldTextColor := SetTextColor( DC, ColorToRGB( GetTextHintColor));
          OldBackColor := SetBkColor  ( DC, ColorToRGB( FormDllMain.SelectorColor));
          DrawText( DC, PChar( TextHint), Length( TextHint), R, DT_LEFT or DT_VCENTER or DT_SINGLELINE or DT_NOPREFIX);
        finally
          SetTextColor( DC, OldTextColor);
          SetBkColor  ( DC, OldBackColor);
          SelectObject( DC, OldFont);
          ReleaseDC( Handle, DC);
        end;

        Handled := True;
      end;

      if   not Handled
      then WndProc( aMessage);
    end;


//  private

const

  sdfs = 'settings';         // General setting section in ini file


    function    TFormDllMain.HandleContextMenu( const aControl: TControl): Boolean;
    begin
      Result := False;

      if   Assigned( aControl)
      then begin
        if   ( aControl is TKnobsWirePanel    )
        or   ( aControl is TKnobsModule       )
        or   ( aControl is TKnobsConnector    )
        or   ( aControl is TKnobsValuedControl)
        or   ( aControl is TKnobsDataMaker    )
        or   ( aControl is TKnobsGridControl  )
        then begin
          Result := True;
          DoShowPopupMenu( nil, aControl);
        end;
      end;
    end;


    procedure   TFormDllMain.ShowAcknowledgements;
    const
      URL = BaseURL + 'index.php#acknowledgements';
    begin
      ShellExecute(
        Handle,
        Nil,
        PChar( URL),
        Nil,
        Nil,
        SW_SHOWNORMAL
      );
    end;


    procedure   TFormDllMain.CheckForUpdates;
    const
      URL = BaseURL + 'index.php';
    begin
      ShellExecute(
        Handle,
        Nil,
        PChar( Format( '%s?user_version=%s', [ URL, GetFileVersion], AppLocale)),
        Nil,
        Nil,
        SW_SHOWNORMAL
      );
    end;


    procedure   TFormDllMain.SelectFolders;
    begin
      if   Assigned( FormFolderSelector)
      then begin
        if   FormFolderSelector.Execute
        then SaveIni( IniFileName, False);
      end;
    end;


    function    TFormDllMain.FixHtmlTokens( const S: string): string;
    begin
      Result := ReplaceText( S     , '&quot;'  , '"' );
      Result := ReplaceText( Result, '&apos;'  , '''');
      Result := ReplaceText( Result, '&gt;'    , '>' );
      Result := ReplaceText( Result, '&lt;'    , '<' );
      Result := ReplaceText( Result, '&amp;'   , '&' );

      Result := ReplaceText( Result, '<b>'     , ''  );
      Result := ReplaceText( Result, '<table>' , ''  );
      Result := ReplaceText( Result, '<tbody>' , ''  );
      Result := ReplaceText( Result, '<tr>'    , ''  );
      Result := ReplaceText( Result, '<td>'    , ''  );

      Result := ReplaceText( Result, '</b>'    , ''  );
      Result := ReplaceText( Result, '</table>', ''  );
      Result := ReplaceText( Result, '</tbody>', ''  );
      Result := ReplaceText( Result, '</tr>'   , ''  );
      Result := ReplaceText( Result, '</td>'   , ''  );

      Result := ReplaceText( Result, '<br>'    , ''  );
    end;


    procedure   TFormDllMain.ShowModuleHelp( aModule: TKnobsCustomModule);
    var
      S : PChar;
    begin                                                       // GetText uses StrNew ...
      S := aModule.Docs.GetText;                                // ... so we need use StrDispose to free it -
      ShowMessage( FixHtmlTokens( aModule.Title + ^M^M + S));   // this is what the help on GetText sez too, right.
      StrDispose( S);                                           // ok.
    end;


    function    TFormDllMain.MustMakeDocs: Boolean;
    begin
      if   not FileExists( DocsPath + '\modules.html')
      then Result := True
      else begin
        if   System.SysUtils.DirectoryExists( DocsPath + '\images')
        then begin
          if   FileExists( Format( '%s\images\%s', [ DocsPath, GetFileVersion], AppLocale))
          then Result := False
          else Result := True;
        end
        else Result := True;
      end;
    end;


    procedure   TFormDllMain.MakeDocs( const aFolder : string);
    const
      Images = '\images';
    var
      aFileNames   : TStrings;
      aFileName    : string;
      aFile        : TextFile;
      aFileVersion : string;
      aVersion     : string;
      anOldCap     : string;
    begin
      aFileVersion := GetFileVersion;

      if   not System.SysUtils.DirectoryExists ( aFolder)
      then System.SysUtils.ForceDirectories( aFolder);

      if   not System.SysUtils.DirectoryExists ( aFolder + Images)
      then System.SysUtils.ForceDirectories( aFolder + Images);

      aFileNames := CreateFileNames( aFolder + Images, '*.*', faAnyFile - faDirectory, False);

      if   Assigned( aFileNames)
      then begin
        try
          for aFileName in aFileNames
          do DeleteFile( aFileName);
        finally
          aFileNames.DisposeOf;
        end;
      end;

      anOldCap := FormStore.Caption;
      FormStore.Caption := 'generating module help ...';
      FormStore.Show; // must be made visible, or not everything will be rendered ...

      try
        FormStore.Document(
          aFolder, Images,
          aFileVersion
        );
      finally
        FormStore.Caption := anOldCap;
        FormStore.Hide;
      end;

      aVersion := Format( '%s\%s\%s', [ aFolder, Images, aFileVersion], AppLocale);
      AssignFile( aFile, aVersion, CP_UTF8);
      Rewrite( aFile);

      try
        WriteLn( aFile, 'this file is for WREN documentation version checking');
        WriteLn( aFile, aVersion);
        WriteLn( aFile, 'when the path above is not my actual position and name, just delete me please.');
      finally
        CloseFile( aFile);
      end;
    end;


    procedure   TFormDllMain.ShowMenu( const aSender: TControl; const aPopupMenu: TPopupMenu);
    var
      aPoint : TPoint;
    begin
      aPoint := ClientToScreen( Point( aSender.Left, aSender.Top));
      aPopupMenu.Popup( aPoint.X, aPoint.Y);
    end;


    procedure   TFormDllMain.ShowFileMenu( const aSender: TControl);
    begin
      ShowMenu( aSender, PopupMenuFile);
    end;


    procedure   TFormDllMain.ShowEditMenu( const aSender: TControl);
    begin
      ShowMenu( aSender, PopupMenuEdit);
    end;


    procedure   TFormDllMain.ShowActionMenu( const aSender: TControl);
    begin
      ShowMenu( aSender, PopupMenuAction);
    end;


    procedure   TFormDllMain.ShowHelp;
    var
      aHelpIndex : string;
      aModule    : TKnobsCustomModule;
      anURL      : string;
    begin
      aHelpIndex := '';

      if   Assigned( PatchEditor)
      then begin
        aModule := PatchEditor.FindFirstSelectedModule;

        if   Assigned( aModule)
        then aHelpIndex := Format( '#%d', [ aModule.ModuleType], AppLocale);
      end;

      if   MustMakeDocs
      then MakeDocs( DocsPath);

      if   FileExists( DocsPath + '\modules.html')
      then begin
        anURL := Format( '"file:///%s/modules.html%s"', [ DocsPath, aHelpIndex], AppLocale);
        anURL := StringReplace( anURL, '\', '/', [ rfReplaceAll]);

        if   not BrowseURL( anURL)
        then MessageDlg( Format( 'Could not open the docs [%s] in the default web browser ...', [ anURL], AppLocale), mtError, [ mbOK], 0);
      end
      else MessageDlg( Format( 'Could not create docs in [%s]', [ DocsPath], AppLocale), mtError, [ mbOK], 0);
    end;


    procedure   TFormDllMain.LoadIni( const aFileName: string; LooksOnly: Boolean);
    var
      anIniFile : TMemIniFile;
      LRate         : Integer;
    begin
      logFmt( LC_GENERAL, 'Loading inifile: ''%s'', LooksOnly = %s', [ IniFileName, BoolToStr( LooksOnly, True)]);

      try
        anIniFile := TMemIniFile.Create( aFileName, TEncoding.UTF8);
      except
        anIniFile := TMemIniFile.Create( aFileName);
      end;

      LRate := KnobsSmallKnob_LightsRate.KnobPosition;

      with anIniFile
      do begin
        Log( LC_GENERAL, 'Loading looks');

        try
          // User inteerfce stuff
          OffsetLeft               :=                ReadInteger( slooks, 'OffsetLeft'              ,                OffsetLeft               );
          OffsetTop                :=                ReadInteger( slooks, 'OffsetTop'               ,                OffsetTop                );
          UseConnectorBorders      :=                ReadBool   ( slooks, 'UseConnectorBorders'     ,                UseConnectorBorders      );
          UseTypedConnectorBorders :=                ReadBool   ( slooks, 'UseTypedConnectorBorders',                UseTypedConnectorBorders );
          UseNoteNames             :=                ReadBool   ( slooks, 'UseNoteNames'            ,                UseNoteNames             );
          ModuleColor              := StringToColor( ReadString ( slooks, 'ModuleColor'             , ColorToString( ModuleColor            )));
          SelectorColor            := StringToColor( ReadString ( slooks, 'SelectorColor'           , ColorToString( SelectorColor          )));
          SelectorBorderColor      := StringToColor( ReadString ( slooks, 'SelectorBorderColor'     , ColorToString( SelectorBorderColor    )));
          SelectorBorderColorS     := StringToColor( ReadString ( slooks, 'SelectorBorderColorS'    , ColorToString( SelectorBorderColorS   )));
          EditFontColor            := StringToColor( ReadString ( slooks, 'EditFontColor'           , ColorToString( EditFontColor          )));
          DisplayColor             := StringToColor( ReadString ( slooks, 'DisplayColor'            , ColorToString( DisplayColor           )));
          DisplayBorderColor       := StringToColor( ReadString ( slooks, 'DisplayBorderColor'      , ColorToString( DisplayBorderColor     )));
          KnobMIDIColor            := StringToColor( ReadString ( slooks, 'KnobMIDIColor'           , ColorToString( KnobMIDIColor          )));

          KnobFocusColor           := StringToColor( ReadString ( slooks, 'KnobFocusColor'          , ColorToString( KnobFocusColor         )));
          KnobMIDIFocusColor       := StringToColor( ReadString ( slooks, 'KnobMIDIFocusColor'      , ColorToString( KnobMIDIFocusColor     )));
          ViewerColor              := StringToColor( ReadString ( slooks, 'ViewerColor'             , ColorToString( ViewerColor            )));
          ViewerBorderColor        := StringToColor( ReadString ( slooks, 'ViewerBorderColor'       , ColorToString( ViewerBorderColor      )));
          ViewerLineColor          := StringToColor( ReadString ( slooks, 'ViewerLineColor'         , ColorToString( ViewerLineColor        )));
          ViewerFillColor          := StringToColor( ReadString ( slooks, 'ViewerFillColor'         , ColorToString( ViewerFillColor        )));
          ScopeLineColor           := StringToColor( ReadString ( slooks, 'ScopeLineColor'          , ColorToString( ScopeLineColor         )));
          IndBarPeakColor          := StringToColor( ReadString ( slooks, 'IndBarPeakColor'         , ColorToString( IndBarPeakColor        )));
          IndBarValeyColor         := StringToColor( ReadString ( slooks, 'IndBarValeyColor'        , ColorToString( IndBarValeyColor       )));
          GroupIOColor             := StringToColor( ReadString ( slooks, 'GroupIOColor'            , ColorToString( GroupIOColor           )));
          GroupOSCColor            := StringToColor( ReadString ( slooks, 'GroupOSCColor'           , ColorToString( GroupOSCColor          )));
          GroupLFOColor            := StringToColor( ReadString ( slooks, 'GroupLFOColor'           , ColorToString( GroupLFOColor          )));
          GroupEnvColor            := StringToColor( ReadString ( slooks, 'GroupEnvColor'           , ColorToString( GroupEnvColor          )));
          GroupFilterColor         := StringToColor( ReadString ( slooks, 'GroupFilterColor'        , ColorToString( GroupFilterColor       )));
          GroupSwitchColor         := StringToColor( ReadString ( slooks, 'GroupSwitchColor'        , ColorToString( GroupSwitchColor       )));
          GroupMixColor            := StringToColor( ReadString ( slooks, 'GroupMixColor'           , ColorToString( GroupMixColor          )));
          GroupControlColor        := StringToColor( ReadString ( slooks, 'GroupControlColor'       , ColorToString( GroupControlColor      )));
          GroupNoteColor           := StringToColor( ReadString ( slooks, 'GroupNoteColor'          , ColorToString( GroupNoteColor         )));
          GroupMathColor           := StringToColor( ReadString ( slooks, 'GroupMathColor'          , ColorToString( GroupMathColor         )));
          GroupLogicColor          := StringToColor( ReadString ( slooks, 'GroupLogicColor'         , ColorToString( GroupLogicColor        )));
          GroupSeqColor            := StringToColor( ReadString ( slooks, 'GroupSeqColor'           , ColorToString( GroupSeqColor          )));
          GroupGeneratorColor      := StringToColor( ReadString ( slooks, 'GroupGenratorColor'      , ColorToString( GroupGeneratorColor    )));
          GroupFXColor             := StringToColor( ReadString ( slooks, 'GroupFXColor'            , ColorToString( GroupFXColor           )));
          GroupDelayColor          := StringToColor( ReadString ( slooks, 'GroupDelayColor'         , ColorToString( GroupDelayColor        )));
          GroupVoiceColor          := StringToColor( ReadString ( slooks, 'GroupVoiceColor'         , ColorToString( GroupVoiceColor        )));
          GroupUtilityColor        := StringToColor( ReadString ( slooks, 'GroupUtilityColor'       , ColorToString( GroupUtilityColor      )));
          AudioRateColor           := StringToColor( ReadString ( slooks, 'AudioRateColor'          , ColorToString( AudioRateColor         )));

          ControlRateColor         := StringToColor( ReadString ( slooks, 'ControlRateColor'        , ColorToString( ControlRateColor       )));
          ControlRateLogicColor    := StringToColor( ReadString ( slooks, 'ControlRateLogicColor'   , ColorToString( ControlRateLogicColor  )));
          LogicColor               := StringToColor( ReadString ( slooks, 'LogicColor'              , ColorToString( LogicColor             )));
          UnconnectedWireColor     := StringToColor( ReadString ( slooks, 'UnconnectedWireColor'    , ColorToString( UnconnectedWireColor   )));
          WireHighlightColor       := StringToColor( ReadString ( slooks, 'WireHighlightColor'      , ColorToString( WireHighlightColor     )));
          EditorBackgroundColor    := StringToColor( ReadString ( slooks, 'EditorBackgroundColor'   , ColorToString( EditorBackgroundColor  )));
          StandardColors           :=                ReadBool   ( slooks, 'StandardColors'          ,                StandardColors           );
          ModuleOpacity            :=                ReadInteger( slooks, 'ModuleOpacity'           ,                ModuleOpacity            );
          ModuleFlatness           :=                ReadBool   ( slooks, 'ModuleFlatness'          ,                ModuleFlatness           );
          ModuleTexture            :=                ReadBool   ( slooks, 'ModuleTexture'           ,                ModuleTexture            );
          WireThickness            :=                ReadInteger( slooks, 'WireThickness'           ,                WireThickness            );
          CurvedLines              :=                ReadBool   ( slooks, 'CurvedLines'             ,                CurvedLines              );
          LRate                    :=                ReadInteger( slooks, 'LightsRate'              ,                LRate                    );
          ControlMode              := TDistanceMode( ReadInteger( slooks, 'ControlMode'             , Integer(       ControlMode            )));
          UseWheelOnKnobs          :=                ReadBool   ( slooks, 'UseWheelOnKnobs'         ,                UseWheelOnKnobs          );
          WheelSensitivity         :=                ReadInteger( slooks, 'WheelSensitivity'        ,                WheelSensitivity         );

          if   not LooksOnly
          then begin
            // General settings
            Log( LC_GENERAL, 'Loading general settings');

            FormFolderSelector.LoadFromIni( anInifile, 'FormFolderSelector');
            FRecentSourceFiles.LoadFromIni( anIniFile, 'RecentFiles'       );
            HistoryCount := FRecentSourceFiles.MaxDepth;

            SynthName              :=                ReadString ( sdfs, 'SynthName'             ,          SynthName            );
            ReloadLastPatch        :=                ReadBool   ( sdfs, 'ReloadLastPatch'       ,          ReloadLastPatch      );
            FDoAutoRun             :=                ReadBool   ( sdfs, 'AutoRun'               ,          AutoRun              );
            SourceFileName         :=                ReadString ( sdfs, 'SourceFileName'        ,          SourceFileName       );
            UseConfigHints         :=                ReadBool   ( sdfs, 'UseConfigHints'        ,          UseConfigHints       );
            UseModuleTitleHints    :=                ReadBool   ( sdfs, 'UseModuleTitleHints'   ,          UseModuleTitleHints  );
            UsePatchHints          :=                ReadBool   ( sdfs, 'UsePatchHints'         ,          UsePatchHints        );
            NewPatchNameTemplate   :=                ReadString ( sdfs, 'NewPatchNameTemplate'  ,          NewPatchNameTemplate );
            TextWriterPrefix       := UnQuote(       ReadString ( sdfs, 'TextWriterPrefix'      ,          TextWriterPrefix     ), '''', '''');
            PrependPrefix          :=                ReadBool   ( sdfs, 'PrependPrefix'         ,          PrependPrefix        );
            UseTextWriter          :=                ReadBool   ( sdfs, 'UseTextWriter'         ,          UseTextWriter        );

            KnobsConversions.DefaultReferenceA     := ReferenceA    ;
            KnobsConversions.DefaultNotesPerOctave := NotesPerOctave;
            KnobsConversions.DefaultMiddleNote     := MiddleNote    ;
            KnobsConversions.DefaultOctaveSpan     := OctaveSpan    ;
            KnobsConversions.LfoRelRate            := LfoRelRate    ;

            if   Assigned( FormRandomizer)
            then FormRandomizer.LoadSettings( anInifile, 'Randomizer');

            UpdateCaptions;
            KnobsSmallKnob_LightsRate.KnobPosition := LRate;
            LightsRate := KnobsSmallKnob_LightsRate.AsValue;
            FixTabColors;

            if   Assigned( FormLastEdits)
            then FormLastEdits.LoadIni( 'FormLastEdits', anIniFile);

            FIniLoaded := True;
          end;
        finally
          DisposeOf;
        end;
      end;
    end;


    procedure   TFormDllMain.SaveIni( const aFileName: string; LooksOnly: Boolean);
    var
      anIniFile : TMemIniFile;
      LRate     : Integer;
    begin
      LogFmt( LC_GENERAL, 'TFormWren.SaveIni( ''%s'', LooksOnly = %s)', [ aFileName, BoolToStr( LooksOnly, True)]);

      try
        anIniFile := TMemIniFile.Create( aFileName, TEncoding.UTF8);
      except
        LogFmt( LC_GENERAL, 'TFormWren.SaveIni( ''%s'', LooksOnly = %s) failed, trying to repair by erasing some file history', [ aFileName, BoolToStr( LooksOnly, True)]);
        anIniFile := TMemIniFile.Create( aFileName);

        with anIniFile do
        begin
          WriteString ( sdfs, 'SourceFileName', '');
          WriteString ( sdfs, 'SynthName'     , '');

          FRecentSourceFiles.Clear;
          FRecentSourceFiles.SaveToIni( anIniFile, 'RecentFiles');
          UpdateFile;
          DisposeOf;
        end;

        anIniFile := TMemIniFile.Create( aFileName, TEncoding.UTF8);
      end;

      LRate := KnobsSmallKnob_LightsRate.KnobPosition;

      with anIniFile
      do begin
        Log( LC_GENERAL, 'Saving looks');

        try
          // User interface stuff
          EraseSection( slooks);
          WriteBool   ( slooks, 'UseThemes'               ,                UseThemes               );
          WriteBool   ( slooks, 'UseConnectorBorders'     ,                UseConnectorBorders     );
          WriteBool   ( slooks, 'UseTypedConnectorBorders',                UseTypedConnectorBorders);
          WriteBool   ( slooks, 'UseNoteNames'            ,                UseNoteNames            );
          WriteInteger( slooks, 'OffsetLeft'              ,                OffsetLeft              );
          WriteInteger( slooks, 'OffsetTop'               ,                OffsetTop               );
          WriteString ( slooks, 'ModuleColor'             , ColorToString( ModuleColor            ));
          WriteString ( slooks, 'SelectorColor'           , ColorToString( SelectorColor          ));
          WriteString ( slooks, 'SelectorBorderColor'     , ColorToString( SelectorBorderColor    ));
          WriteString ( slooks, 'SelectorBorderColorS'    , ColorToString( SelectorBorderColorS   ));
          WriteString ( slooks, 'EditFontColor'           , ColorToString( EditFontColor          ));
          WriteString ( slooks, 'DisplayColor'            , ColorToString( DisplayColor           ));
          WriteString ( slooks, 'DisplayBorderColor'      , ColorToString( DisplayBorderColor     ));
          WriteString ( slooks, 'KnobMIDIColor'           , ColorToString( KnobMIDIColor          ));
          WriteString ( slooks, 'KnobFocusColor'          , ColorToString( KnobFocusColor         ));
          WriteString ( slooks, 'KnobMIDIFocusColor'      , ColorToString( KnobMIDIFocusColor     ));
          WriteString ( slooks, 'ViewerColor'             , ColorToString( ViewerColor            ));
          WriteString ( slooks, 'ViewerBorderColor'       , ColorToString( ViewerBorderColor      ));
          WriteString ( slooks, 'ViewerLineColor'         , ColorToString( ViewerLineColor        ));
          WriteString ( slooks, 'ViewerFillColor'         , ColorToString( ViewerFillColor        ));
          WriteString ( slooks, 'ScopeLineColor'          , ColorToString( ScopeLineColor         ));
          WriteString ( slooks, 'IndBarPeakColor'         , ColorToString( IndBarPeakColor        ));
          WriteString ( slooks, 'IndBarValeyColor'        , ColorToString( IndBarValeyColor       ));
          WriteString ( slooks, 'GroupIOColor'            , ColorToString( GroupIOColor           ));
          WriteString ( slooks, 'GroupOSCColor'           , ColorToString( GroupOSCColor          ));
          WriteString ( slooks, 'GroupLFOColor'           , ColorToString( GroupLFOColor          ));
          WriteString ( slooks, 'GroupEnvColor'           , ColorToString( GroupEnvColor          ));
          WriteString ( slooks, 'GroupFilterColor'        , ColorToString( GroupFilterColor       ));
          WriteString ( slooks, 'GroupSwitchColor'        , ColorToString( GroupSwitchColor       ));
          WriteString ( slooks, 'GroupMixColor'           , ColorToString( GroupMixColor          ));
          WriteString ( slooks, 'GroupControlColor'       , ColorToString( GroupControlColor      ));
          WriteString ( slooks, 'GroupNoteColor'          , ColorToString( GroupNoteColor         ));
          WriteString ( slooks, 'GroupMathColor'          , ColorToString( GroupMathColor         ));
          WriteString ( slooks, 'GroupLogicColor'         , ColorToString( GroupLogicColor        ));
          WriteString ( slooks, 'GroupSeqColor'           , ColorToString( GroupSeqColor          ));
          WriteString ( slooks, 'GroupGeneratorColor'     , ColorToString( GroupGeneratorColor    ));
          WriteString ( slooks, 'GroupFXColor'            , ColorToString( GroupFXColor           ));
          WriteString ( slooks, 'GroupDelayColor'         , ColorToString( GroupDelayColor        ));
          WriteString ( slooks, 'GroupVoiceColor'         , ColorToString( GroupVoiceColor        ));
          WriteString ( slooks, 'GroupUtilityColor'       , ColorToString( GroupUtilityColor      ));
          WriteString ( slooks, 'AudioRateColor'          , ColorToString( AudioRateColor         ));
          WriteString ( slooks, 'ControlRateColor'        , ColorToString( ControlRateColor       ));
          WriteString ( slooks, 'ControlRateLogicColor'   , ColorToString( ControlRateLogicColor  ));
          WriteString ( slooks, 'LogicColor'              , ColorToString( LogicColor             ));
          WriteString ( slooks, 'UnconnectedWireColor'    , ColorToString( UnconnectedWireColor   ));
          WriteString ( slooks, 'WireHighlightColor'      , ColorToString( WireHighlightColor     ));
          WriteString ( slooks, 'EditorBackgroundColor'   , ColorToString( EditorBackgroundColor  ));
          WriteBool   ( slooks, 'StandardColors'          ,                StandardColors          );
          WriteInteger( slooks, 'ModuleOpacity'           ,                ModuleOpacity           );
          WriteBool   ( slooks, 'ModuleFlatness'          ,                ModuleFlatness          );
          WriteBool   ( slooks, 'ModuleTexture'           ,                ModuleTexture           );
          WriteInteger( slooks, 'WireThickness'           ,                WireThickness           );
          WriteBool   ( slooks, 'CurvedLines'             ,                CurvedLines             );
          WriteInteger( slooks, 'LightsRate'              ,                LRate                   );
          WriteInteger( slooks, 'ControlMode'             , Integer(       ControlMode            ));
          WriteBool   ( slooks, 'UseWheelOnKnobs'         ,                UseWheelOnKnobs         );
          WriteInteger( slooks, 'WheelSensitivity'        ,                WheelSensitivity        );

          if   not LooksOnly
          then begin
            Log( LC_GENERAL, 'Saving general settings');
            // General settings
            EraseSection( sdfs);
            WriteString ( sdfs, 'SynthName'             ,          SynthName            );
            WriteBool   ( sdfs, 'ReloadLastPatch'       ,          ReloadLastPatch      );
            WriteBool   ( sdfs, 'AutoRun'               ,          AutoRun              );
            WriteString ( sdfs, 'SourceFileName'        ,          SourceFileName       );
            WriteBool   ( sdfs, 'UseConfigHints'        ,          UseConfigHints       );
            WriteBool   ( sdfs, 'UseModuleTitleHints'   ,          UseModuleTitleHints  );
            WriteBool   ( sdfs, 'UsePatchHints'         ,          UsePatchHints        );

            WriteString ( sdfs, 'NewPatchNameTemplate'  ,          NewPatchNameTemplate );
            WriteString ( sdfs, 'TextWriterPrefix'      , Quote(   TextWriterPrefix     , '''', ''''));
            WriteBool   ( sdfs, 'PrependPrefix'         ,          PrependPrefix        );
            WriteBool   ( sdfs, 'UseTextWriter'         ,          UseTextWriter        );

            FormFolderSelector.SaveToIni( anInifile, 'FormFolderSelector');
            FRecentSourceFiles.SaveToIni( anIniFile, 'RecentFiles'       );

            if   Assigned( FormRandomizer)
            then FormRandomizer.SaveSettings( anInifile, 'Randomizer');

            if   Assigned( FormLastEdits)
            then FormLastEdits.SaveIni( 'FormLastEdits', anIniFile);
          end;
        finally
          UpdateFile;
          DisposeOf;
        end;
      end;
    end;


    procedure   TFormDllMain.UpdateCaptions;
    begin
    end;


    procedure   TFormDllMain.DoModuleWarning( aType: TModWarningType; const aMsg: string);
    begin
      MessageDlg(
        aMsg,
        mtWarning,
        [ mbOK],
        0
      );
    end;



    procedure   TFormDllMain.SetAudioCacheSize( aValue: UInt64);
    begin
      if   aValue <> FAudioCacheSize
      then begin
        FAudioCacheSize := aValue;
        UpdateCaptions;
      end;
    end;


    procedure   TFormDllMain.SetLightsActive( aValue: Boolean);
    begin
      if   aValue <> FLightsActive
      then FLightsActive := aValue;
    end;


    procedure   TFormDllMain.SetPolyphony( aValue: Integer);
    begin
      aValue := Clip( aValue, MIN_POLYPHONY, MAX_POLYPHONY);

      if   aValue <> FPolyphony
      then begin
        FPolyphony               := aValue;
        LabelPolyphony.Caption   := Format( 'poly:%d', [ Polyphony], AppLocale);
        UpDownPolyphony.Position := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.Polyphony := aValue;
      end;
    end;


    procedure   TFormDllMain.SetHistoryCount( aValue: Integer);
    begin
      if   aValue <> FHistoryCount
      then begin
        FHistoryCount                         := aValue;
        KnobsNoKnob_HistoryCount.KnobPosition := aValue - 1;

        if   Assigned( FRecentSourceFiles)
        then FRecentSourceFiles.MaxDepth := aValue;
      end;
    end;


    procedure   TFormDllMain.SetUseWheelOnKnobs( aValue: Boolean);
    begin
      if   aValue <> FUseWheelOnKnobs
      then begin
        FUseWheelOnKnobs                := aValue;
        CheckBoxUseWheelOnKnobs.Checked := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.WheelSupportOnKnobs := aValue;

        if   Assigned( FormRandomizer)
        then FormRandomizer.WheelSupport := aValue;

        FixLocalControls;
        CheckBoxUseUnFocus.Enabled := aValue;
      end;
    end;


    procedure   TFormDllMain.SetWheelSensitivity( aValue: Integer);
    begin
      if   aValue <> FWheelSensitivity
      then begin
        FWheelSensitivity                  := aValue;
        knob_WheelSensitivity.KnobPosition := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.WheelSensitivity := aValue;

        if   Assigned( FormRandomizer)
        then FormRandomizer.WheelSensitivity := aValue;

        FixLocalControls;
      end;
    end;


    procedure   TFormDllMain.SetUseConfigHints( aValue: Boolean);
    begin
      if   aValue <> FUseConfigHints
      then begin
        FUseConfigHints                := aValue;
        CheckBoxUseConfigHints.Checked := aValue;
        TabSheetSettings.ShowHint      := aValue;
        TabSheetDebug   .ShowHint      := aValue;
      end;
    end;


    procedure   TFormDllMain.SetUseModuleTitleHints( aValue: Boolean);
    begin
      if   aValue <> FUseModuleTitleHints
      then begin
        FUseModuleTitleHints                := aValue;
        CheckBoxUseModuleTitleHints.Checked := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.UseTitleHints := aValue;
      end;
    end;


    procedure   TFormDllMain.SetUsePatchHints( aValue: Boolean);
    begin
      if   aValue <> FUsePatchHints
      then begin
        FUsePatchHints                := aValue;
        CheckBoxUsePatchHints.Checked := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.ShowPatchHints := aValue;
      end;
    end;


    procedure   TFormDllMain.SetWireThickness( aValue: Integer);
    begin
      aValue := Clip( aValue, 0, 6);

      if   aValue <> FWireThickness
      then begin
        FWireThickness := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.WireThickness := aValue;

        UpDownWireThickness1.Position := aValue;
      end;
    end;


    procedure   TFormDllMain.SetCurvedLines( aValue: Boolean);
    begin
      if   aValue <> FCurvedLines
      then begin
        FCurvedLines := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.CurvedLines := aValue;

        CheckBoxCurvedLines.Checked  := aValue;
      end;
    end;


    procedure   TFormDllMain.SetControlMode( aValue: TDistanceMode);
    begin
      if   aValue <> FControlMode
      then begin
        FControlMode := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.ControlMode := FControlMode;

        if   Assigned( FormSelectMidiCC)
        then FormSelectMidiCC.ControlMode := FControlMode;

        if  Assigned( FormRandomizer)
        then FormRandomizer.ControlMode := FControlMode;

        FixLocalControls;
        RadioGroupControlMode.ItemIndex := Integer( FControlMode);
      end;
    end;


    procedure   TFormDllMain.SetLightsRate( aValue: TSignal);
    begin
      aValue := UnitsToLfoFastFrequency( aValue);
      PClip( aValue, 0.254, 401.8);

      if   ( aValue <> FLightsRate)
      and  Assigned( TimerLightsUpdate)
      then begin
        FLightsRate := aValue;
        TimerLightsUpdate.Interval := Round( 1000 / aValue);
      end;
    end;


    function    TFormDllMain.GetOffsetLeft: Integer;
    begin
      if   Assigned( PatchEditor)
      then Result := PatchEditor.OffsetLeft
      else Result := 4;
    end;


    procedure   TFormDllMain.SetOffsetLeft( aValue: Integer);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.OffsetLeft := aValue;

      KnobsSmallKnob_LeftOffset.KnobPosition := aValue;
    end;


    function    TFormDllMain.GetOffsetTop: Integer;
    begin
      if   Assigned( PatchEditor)
      then Result := PatchEditor.OffsetTop
      else Result := 4;
    end;


    procedure   TFormDllMain.SetOffsetTop( aValue: Integer);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.OffsetTop := aValue;

      KnobsSmallKnob_TopOffset.KnobPosition := aValue;
    end;


    function    TFormDllMain.GetModuleColor: TColor;
    begin
      Result := PatchEditor.ModuleColor;
    end;


    procedure   TFormDllMain.SetModuleColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.ModuleColor := aValue;
    end;


    function    TFormDllMain.GetConnectorBorderColor: TColor;
    begin
      Result := PatchEditor.ConnectorBorderColor;
    end;


    procedure   TFormDllMain.SetConnectorBorderColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.ConnectorBorderColor := aValue;

      if   Assigned( FormStore)
      then FormStore.ConnectorBorderColor := aValue;
    end;


    function    TFormDllMain.GetSelectorColor: TColor;
    begin
      Result := PatchEditor.SelectorColor;
    end;


    procedure   TFormDllMain.SetSelectorColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.SelectorColor := aValue;

      if   Assigned( FormStore)
      then FormStore.SelectorColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetSelectorBorderColor: TColor;
    begin
      Result := PatchEditor.SelectorBorderColor;
    end;


    procedure   TFormDllMain.SetSelectorBorderColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.SelectorBorderColor := aValue;

      if   Assigned( FormStore)
      then FormStore.SelectorBorderColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetSelectorBorderColorS: TColor;
    begin
      Result := PatchEditor.SelectorBorderColorSingle;
    end;


    procedure   TFormDllMain.SetSelectorBorderColorS( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.SelectorBorderColorSingle := aValue;

      if   Assigned( FormStore)
      then FormStore.SelectorBorderColorS := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetEditFontColor: TColor;
    begin
      Result := PatchEditor.EditFontColor;
    end;


    procedure   TFormDllMain.SetEditFontColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.EditFontColor := aValue;

      if   Assigned( FormStore)
      then FormStore.EditFontColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetDisplayColor: TColor;
    begin
      Result := PatchEditor.DisplayColor;
    end;


    procedure   TFormDllMain.SetDisplayColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.DisplayColor := aValue;

      if   Assigned( FormStore)
      then FormStore.DisplayColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetDisplayBorderColor: TColor;
    begin
      Result := PatchEditor.DisplayBorderColor;
    end;


    procedure   TFormDllMain.SetDisplayBorderColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.DisplayBorderColor := aValue;

      if   Assigned( FormStore)
      then FormStore.DisplayBorderColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetKnobMIDIColor: TColor;
    begin
      Result := PatchEditor.KnobMIDIColor;
    end;


    procedure   TFormDllMain.SetKnobMIDIColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.KnobMIDIColor := aValue;

      if   Assigned( FormStore)
      then FormStore.KnobMIDIColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetKnobFocusColor: TColor;
    begin
      Result := PatchEditor.KnobFocusColor;
    end;


    procedure   TFormDllMain.SetKnobFocusColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.KnobFocusColor := aValue;

      if   Assigned( FormStore)
      then FormStore.KnobFocusColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetKnobMIDIFocusColor: TColor;
    begin
      Result := PatchEditor.KnobMIDIFocusColor;
    end;


    procedure   TFormDllMain.SetKnobMIDIFocusColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.KnobMIDIFocusColor := aValue;

      if   Assigned( FormStore)
      then FormStore.KnobMIDIFocusColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetViewerColor: TColor;
    begin
      Result := PatchEditor.ViewerColor;
    end;


    procedure   TFormDllMain.SetViewerColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.ViewerColor := aValue;

      if   Assigned( FormStore)
      then FormStore.ViewerColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetViewerBorderColor: TColor;
    begin
      Result := PatchEditor.ViewerBorderColor;
    end;


    procedure   TFormDllMain.SetViewerBorderColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.ViewerBorderColor := aValue;

      if   Assigned( FormStore)
      then FormStore.ViewerBorderColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetViewerLineColor: TColor;
    begin
      Result := PatchEditor.ViewerLineColor;
    end;


    procedure   TFormDllMain.SetViewerLineColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.ViewerLineColor := aValue;

      if   Assigned( FormStore)
      then FormStore.ViewerLineColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetViewerFillColor: TColor;
    begin
      Result := PatchEditor.ViewerFillColor;
    end;


    procedure   TFormDllMain.SetViewerFillColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.ViewerFillColor := aValue;

      if   Assigned( FormStore)
      then FormStore.ViewerFillColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetScopeLineColor: TColor;
    begin
      Result := PatchEditor.ScopeLineColor;
    end;


    procedure   TFormDllMain.SetScopeLineColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.ScopeLineColor := aValue;

      if   Assigned( FormStore)
      then FormStore.ScopeLineColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetIndBarPeakColor: TColor;
    begin
      Result := PatchEditor.IndBarPeakColor;
    end;


    procedure   TFormDllMain.SetIndBarPeakColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.IndBarPeakColor := aValue;

      if   Assigned( FormStore)
      then FormStore.IndBarPeakColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetIndBarValeyColor: TColor;
    begin
      Result := PatchEditor.IndBarValeyColor;
    end;


    procedure   TFormDllMain.SetIndBarValeyColor( aValue: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.IndBarValeyColor := aValue;

      if   Assigned( FormStore)
      then FormStore.IndBarValeyColor := aValue;

      FixLocalControls;
    end;


    function    TFormDllMain.GetGroupIOColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupIOColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupIOColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupIOColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupOSCColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupOSCColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupOSCColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupOSCColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupLFOColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupLFOColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupLFOColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupLFOColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupEnvColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupEnvColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupEnvColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupEnvColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupFilterColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupFilterColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupFilterColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupFilterColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupSwitchColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupSwitchColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupSwitchColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupSwitchColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupMixColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupMixColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupMixColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupMixColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupControlColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupControlColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupControlColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupControlColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupNoteColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupNoteColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupNoteColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupNoteColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupMathColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupMathColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupMathColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupMathColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupLogicColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupLogicColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupLogicColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupLogicColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupSeqColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupSeqColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupSeqColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupSeqColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupGeneratorColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupGeneratorColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupGeneratorColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupGeneratorColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupFXColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupFXColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupFXColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupFXColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupDelayColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupDelayColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupDelayColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupDelayColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupVoiceColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupVoiceColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupVoiceColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupVoiceColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetGroupUtilityColor: TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.GroupUtilityColor
      else Result := clNone;
    end;


    procedure   TFormDllMain.SetGroupUtilityColor( aValue: TColor);
    begin
      if   Assigned( FormStore)
      then begin
        FormStore.GroupUtilityColor := aValue;
        FixTabColors;
      end;
    end;


    function    TFormDllMain.GetAudioRateColor: TColor;
    begin
      Result := ColorAudio;
    end;


    procedure   TFormDllMain.SetAudioRateColor( aValue: TColor);
    begin
      if   aValue <> ColorAudio
      then begin
        ColorAudio := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.InvalidateWires;
      end;
    end;


    function    TFormDllMain.GetControlRateColor: TColor;
    begin
      Result := ColorControl;
    end;


    procedure   TFormDllMain.SetControlRateColor( aValue: TColor);
    begin
      if   aValue <> ColorControl
      then begin
        ColorControl := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.InvalidateWires;
      end;
    end;


    function    TFormDllMain.GetControlRateLogicColor: TColor;
    begin
      Result := ColorContrLogic;
    end;


    procedure   TFormDllMain.SetControlRateLogicColor( aValue: TColor);
    begin
      if   aValue <> ColorContrLogic
      then begin
        ColorContrLogic := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.InvalidateWires;
      end;
    end;


    function    TFormDllMain.GetLogicColor: TColor;
    begin
      Result := ColorLogic;
    end;


    procedure   TFormDllMain.SetLogicColor( aValue: TColor);
    begin
      if   aValue <> ColorLogic
      then begin
        ColorLogic := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.InvalidateWires;
      end;
    end;


    function    TFormDllMain.GetUnconnectedWireColor: TColor;
    begin
      Result := ColorUnknown;
    end;


    procedure   TFormDllMain.SetUnconnectedWireColor( aValue: TColor);
    begin
      if   aValue <> ColorUnknown
      then begin
        ColorUnknown := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.InvalidateWires;
      end;
    end;


    function    TFormDllMain.GetWireHighlightColor: TColor;
    begin
      Result := ColorHighlight;
    end;


    procedure   TFormDllMain.SetWireHighlightColor( aValue: TColor);
    begin
      if   aValue <> ColorHighlight
      then begin
        ColorHighlight := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.InvalidateWires;
      end;
    end;


    function    TFormDllMain.GetEditorBackgroundColor : TColor;
    begin
      Result := FEditorBackgroundColor;
    end;


    procedure   TFormDllMain.SetEditorBackgroundColor( aValue: TColor);
    begin
      if   aValue <> FEditorBackgroundColor
      then begin
        if   Assigned( PatchEditor)
        then PatchEditor.Color := aValue;

        if   Assigned( FormStore)
        then FormStore.Color := aValue;
      end;
    end;


    procedure   TFormDllMain.SetStandardColors( aValue: Boolean);
    begin
      if   aValue <> FStandardColors
      then begin
        FStandardColors := aValue;
      end;
    end;


    procedure   TFormDllMain.SetUseConnectorBorders( aValue: Boolean);
    begin
      if   aValue <> FUseConnectorBorders
      then begin
        FUseConnectorBorders := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.UseConnectorBorders := aValue;

        if   Assigned( FormStore)
        then FormStore.ConnectorBorders := aValue;
      end;
    end;


    procedure   TFormDllMain.SetUseTypedConnectorBorders( aValue: Boolean);
    begin
      if   aValue <> FUseTypedConnectorBorders
      then begin
        FUseTypedConnectorBorders := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.UseTypedConnectorBorders := aValue;

        if   Assigned( FormStore)
        then FormStore.ConnectorTypedBorders := aValue;
      end;
    end;


    procedure   TFormDllMain.SetUseNoteNames( aValue: Boolean);
    begin
      if   aValue <> FUseNoteNames
      then begin
        FUseNoteNames := aValue;
        GUseNoteNames := aValue;
        CheckBoxUseNoteNames.Checked := aValue;

        if   Assigned( PatchEditor)
        then PatchEditor.Invalidate;

        if   Assigned( FormStore)
        then FormStore.Invalidate;
      end;
    end;


    function    TFormDllMain.GetModuleOpacity: Byte;
    begin
      Result := PatchEditor.ModuleOpacity;
    end;


    procedure   TFormDllMain.SetModuleOpacity( aValue: Byte);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.ModuleOpacity := aValue;

      KnobsSmallKnobModuleOpacity.KnobPosition  := aValue;
    end;


    function    TFormDllMain.GetModuleFlatness: Boolean;
    begin
      Result := PatchEditor.ModuleFlatness;
    end;


    procedure   TFormDllMain.SetModuleFlatness( aValue: Boolean);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.ModuleFlatness := aValue;
    end;


    function    TFormDllMain.GetModuleTexture: Boolean;
    begin
      Result := PatchEditor.ModuleTexture;
    end;


    procedure   TFormDllMain.SetModuleTexture( aValue: Boolean);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.ModuleTexture := aValue;
    end;


    procedure   TFormDllMain.SetReloadLastPatch( aValue: Boolean);
    begin
      if   aValue <> FReloadLastPatch
      then begin
        FReloadLastPatch                := aValue;
        CheckBoxReloadLastPatch.Checked := aValue;
      end;
    end;


    procedure   TFormDllMain.SetAutoRun( aValue: Boolean);
    begin
      if   aValue <> FAutoRun
      then begin
        FAutoRun                := aValue;
        CheckBoxAutoRun.Checked := aValue;
      end;
    end;


    function    TFormDllMain.GetColorMapLength: Integer;
    begin
      Result := Length( FColorMaps);
    end;


    procedure   TFormDllMain.SetColorMapLength( aValue: Integer);
    begin
      SetLength( FColorMaps, aValue);
    end;


    procedure   TFormDllMain.SetCurrentColor( aValue: TColorId);
    begin
      if   aValue <> FCurrentColor
      then begin
        FCurrentColor := aValue;
      end;
    end;


//    function    TFormDllMain.GetColor( anIndex: TColorId): TColor;
//    var
//      i : Integer;
//    begin
//      Result := clNone;
//
//      for i := 0 to ColorMapLength - 1
//      do begin
//        if   FColorMaps[ i].Id = anIndex
//        then begin
//          Result := FColorMaps[ i].Getter;
//          Break;
//        end;
//      end;
//    end;


//    procedure   TFormDllMain.SetColor( anIndex: TColorId; aValue: TColor);
//    var
//      i : Integer;
//    begin
//      for i := 0 to ColorMapLength - 1
//      do begin
//        if   FColorMaps[ i].Id = anIndex
//        then begin
//          FColorMaps[ i].Setter( aValue);
//          Break;
//        end;
//      end;
//    end;


    procedure   TFormDllMain.AddColorMap( anId: TColorId; const aName: string; const aGetter: TColorGetter; const aSetter: TColorSetter);
    begin
      ColorMapLength := ColorMapLength + 1;
      FColorMaps[ ColorMapLength - 1].Id     := anId;
      FColorMaps[ ColorMapLength - 1].Name   := aName;
      FColorMaps[ ColorMapLength - 1].Getter := aGetter;
      FColorMaps[ ColorMapLength - 1].Setter := aSetter;
    end;


    procedure   TFormDllMain.FixColorMaps;
    begin
      ColorMapLength := 0;

      AddColorMap( cidEditorBackground    , 'Editor background color'     , GetEditorBackgroundColor, SetEditorBackgroundColor);
      AddColorMap( cidModule              , 'Module color'                , GetModuleColor          , SetModuleColor          );
      AddColorMap( cidConnectorBorder     , 'Connector border color'      , GetConnectorBorderColor , SetConnectorBorderColor );
      AddColorMap( cidSelector            , 'Selector color'              , GetSelectorColor        , SetSelectorColor        );
      AddColorMap( cidSelectorBorder      , 'Selector border color'       , GetSelectorBorderColor  , SetSelectorBorderColor  );
      AddColorMap( cidSelectorBorderSingle, 'Selector border color single', GetSelectorBorderColorS , SetSelectorBorderColorS );
      AddColorMap( cidEditFont            , 'Edit font color'             , GetEditFontColor        , SetEditFontColor        );
      AddColorMap( cidDisplay             , 'Display color'               , GetDisplayColor         , SetDisplayColor         );
      AddColorMap( cidDisplayBorder       , 'Display border color'        , GetDisplayBorderColor   , SetDisplayBorderColor   );
      AddColorMap( cidKnobMIDI            , 'Knob MIDI color'             , GetKnobMIDIColor        , SetKnobMIDIColor        );
      AddColorMap( cidKnobFocus           , 'Knob focus color'            , GetKnobFocusColor       , SetKnobFocusColor       );
      AddColorMap( cidKnobMIDIFocus       , 'Knob MIDI and focus color'   , GetKnobMIDIFocusColor   , SetKnobMIDIFocusColor   );
      AddColorMap( cidViewer              , 'Viewer color'                , GetViewerColor          , SetViewerColor          );
      AddColorMap( cidViewerBorder        , 'Viewer border color'         , GetViewerBorderColor    , SetViewerBorderColor    );
      AddColorMap( cidViewerLine          , 'Viewer line color'           , GetViewerLineColor      , SetViewerLineColor      );
      AddColorMap( cidViewerFill          , 'Viewer fill color'           , GetViewerFillColor      , SetViewerFillColor      );
      AddColorMap( cidScopeLine           , 'Scope line color'            , GetScopeLineColor       , SetScopeLineColor       );
      AddColorMap( cidIndicatorBarPeak    , 'Indicatorbar peak color'     , GetIndBarPeakColor      , SetIndBarPeakColor      );
      AddColorMap( cidIndicatorBarValey   , 'Indicatorbar valey color'    , GetIndBarValeyColor     , SetIndBarValeyColor     );
      AddColorMap( cidAudioRate           , 'Audiorate color'             , GetAudioRateColor       , SetAudioRateColor       );
      AddColorMap( cidControlRate         , 'Controlrate color'           , GetControlRateColor     , SetControlRateColor     );
      AddColorMap( cidControlRateLogic    , 'ControlrateLogic color'      , GetControlRateLogicColor, SetControlRateLogicColor);
      AddColorMap( cidLogic               , 'Logic color'                 , GetLogicColor           , SetLogicColor           );
      AddColorMap( cidUnconnectedWire     , 'Unconnected wire color'      , GetUnconnectedWireColor , SetUnconnectedWireColor );
      AddColorMap( cidWireHghlight        , 'Wire highlight color'        , GetWireHighlightColor   , SetWireHighlightColor   );
      AddColorMap( cidGroupIO             , 'Module group I/O color'      , GetGroupIOColor         , SetGroupIOColor         );
      AddColorMap( cidGroupOSC            , 'Module group OSC color'      , GetGroupOSCColor        , SetGroupOSCColor        );
      AddColorMap( cidGroupLFO            , 'Module group LFO color'      , GetGroupLFOColor        , SetGroupLFOColor        );
      AddColorMap( cidGroupEnv            , 'Module group Env color'      , GetGroupEnvColor        , SetGroupEnvColor        );
      AddColorMap( cidGroupFilter         , 'Module group Filter color'   , GetGroupFilterColor     , SetGroupFilterColor     );
      AddColorMap( cidGroupSwitch         , 'Module group Switch color'   , GetGroupSwitchColor     , SetGroupSwitchColor     );
      AddColorMap( cidGroupMix            , 'Module group Mix color'      , GetGroupMixColor        , SetGroupMixColor        );
      AddColorMap( cidGroupControl        , 'Module group Control color'  , GetGroupControlColor    , SetGroupControlColor    );
      AddColorMap( cidGroupNote           , 'Module group Note color'     , GetGroupNoteColor       , SetGroupNoteColor       );
      AddColorMap( cidGroupMath           , 'Module group Math color'     , GetGroupMathColor       , SetGroupMathColor       );
      AddColorMap( cidGroupLogic          , 'Module group Logic color'    , GetGroupLogicColor      , SetGroupLogicColor      );
      AddColorMap( cidGroupSeq            , 'Module group Seq color'      , GetGroupSeqColor        , SetGroupSeqColor        );
      AddColorMap( cidGroupGenerator      , 'Module group Gen color'      , GetGroupGeneratorColor  , SetGroupGeneratorColor  );
      AddColorMap( cidGroupFX             , 'Module group FX color'       , GetGroupFXColor         , SetGroupFXColor         );
      AddColorMap( cidGroupDelay          , 'Module group Delay color'    , GetGroupDelayColor      , SetGroupDelayColor      );
      AddColorMap( cidGroupDelay          , 'Module group Voice color'    , GetGroupVoiceColor      , SetGroupVoiceColor      );
      AddColorMap( cidGroupUtility        , 'Module group Utility color'  , GetGroupUtilityColor    , SetGroupUtilityColor    );

      CurrentColor := cidGroupOSC;
      ChangeColorId( 0);
    end;


    procedure   TFormDllMain.ChangeColorId( anIndex: Integer);
    begin
    end;


//    procedure   TFormDllMain.ChangeCurrentColor( aColor: TColor);
//    begin
//      SetColor( CurrentColor, aColor);
//    end;


    procedure   TFormDllMain.ClearLog;
    begin
      MemoDebug.Clear;
    end;


    function    TFormDllMain.RtcTimeStamp: string;
    begin
      Result := FormatDateTime( 'yyyy-mm-dd hh:nn:ss:zzz', Now, AppLocale)  // Current PC time
    end;


    procedure   TFormDllMain.WriteFileLog( const aMsg: string);
    var
      aFile : TextFile;
    begin
      try
        AssignFile( aFile, LogFileName, CP_UTF8);

        if   FileExists( LogFileName)
        then Append ( aFile)
        else Rewrite( aFile);
        try
          WriteLn( aFile, aMsg);
        finally
          CloseFile( aFile);
        end;
      except
        on E: Exception
        do KilledException( E);
      end;
    end;


    procedure   TFormDllMain.Log( aLogClass: TLogClass; const aMsg: string; TimeStamped: Boolean = True); // overloadl
    var
      S : string;
    begin
      if   aLogClass = LC_NO_CLASS
      then begin
        if   TimeStamped
        then S := Format( '%s %s', [ RtcTimeStamp, aMsg], AppLocale)
        else S := aMsg
      end
      else begin
        if   TimeStamped
        then S := Format( '%s [%s] %s', [ RtcTimeStamp, LogClassToStr( aLogClass), aMsg], AppLocale)
        else S := Format( '[%s] %s'   , [               LogClassToStr( aLogClass), aMsg], AppLocale);
      end;

      WriteFileLog( S);

      if   CanLog                                                      // We can not log before the form is constructed
      then MemoDebug.Lines.Add( S)
      else begin
        if   not Assigned( FSavedLogs)                                 // When we can not log logs are stored on a string
        then FSavedLogs := TStringList.Create;                         // list to be logged later on.

        FSavedLogs.Add( S);
      end;
    end;


    procedure   TFormDllMain.Log( aLogClass: TLogClass; const aMsgs: TStrings; DoClear: Boolean); // overload;
    var
      S : string;
    begin
      if   Assigned( aMsgs)
      then begin
        for S in aMsgs
        do  Log( LC_NO_CLASS, S, False);                                 // No TimeStamps, no class indication - those were added before.

        if   DoClear
        then aMsgs.Clear;
      end;
    end;


    procedure   TFormDllMain.LogFmt( aLogClass: TLogClass; const aFmt: string; const anArgs: array of const);
    begin
      Log( aLogClass, Format( aFmt, anArgs, AppLocale));
    end;


    procedure   TFormDllMain.DoSynthLog( const aSender: TObject; aLogClass: Integer; const aMsg: string);
    begin
      Log( aLogClass, aMsg);
    end;


//    procedure   TFormDllMain.LogRandomizer( const aSender: TObject; const aMsg: string);
//    begin
//      Log( LC_RANDOMIZER, aMsg);
//    end;
//
//
//    procedure   TFormDllMain.LogVariations( const aSender: TObject; const aMsg: string);
//    begin
//      Log( LC_RANDOMIZER, aMsg);
//    end;


    procedure   TFormDllMain.FormantLogger( const aSender: TObject; aLogClass: TLogClass; const aMsg: string);
    begin
      LogFmt( aLogClass, 'Formantlog: %s', [ aMsg]);
    end;


    procedure   TFormDllMain.DoDesignerLog( const aSender: TObject; aLogClass: TLogClass; const aMsg: string);
    begin
      Log( aLogClass, aMsg);
    end;


    procedure   TFormDllMain.DoWirePanelLog( const aSender: TObject; aLogClass: TLogClass; const aMsg: string);
    begin
      Log( aLogClass, aMsg);
    end;


    procedure   TFormDllMain.DoFormDesignerLog( const aMsg: string);
    begin
      Log( LC_FRM_STORE, aMsg, True);
    end;


    function    TFormDllMain.GetSynthName: string;
    begin
      Result := EditSynthName.Text;
    end;


    procedure   TFormDllMain.SetSynthName( const aValue: string);
    begin
      EditSynthName.Text := aValue;
      UpdateCaptions;
    end;


    procedure   TFormDllMain.SetCanLog( aValue: Boolean);
    begin
      if   aValue <> FCanLog
      then begin
        FCanLog := aValue;

        if   CanLog
        then Log( LC_STARTUP, FSavedLogs, True);     // This will clear FSavedLogs
      end;
    end;


    procedure   TFormDllMain.AddModule( aModuleType: TKnobsModuleType);
    begin
      PatchEditor.BeginStateChange( True);

      try
        SelectNone; // todo: Quick patch for module jumps on insertion issue
                    //       TKnobsModuleList.CreateModule needs some love ...
        FormStore.CreateModule( PatchEditor, aModuleType, DO_DRAG, StandardColors).FixupInsertion;
      finally
        PatchEditor.EndStateChange( True, True);
      end;
    end;


    procedure   TFormDllMain.DoEditorMenuAddModule( aSender: TObject);
    var
      aMenuItem : TMenuItem;
    begin
      if   aSender is TMenuItem
      then begin
        aMenuItem := TMenuItem( aSender);

        if   aMenuItem.Tag >= 0
        then AddModule( aMenuItem.Tag);
      end;
    end;


    procedure   TFormDllMain.DoShowHint( var aHintStr: string; var CanShow: Boolean; var aHintInfo: Vcl.Controls.THintInfo);
    begin
      if   Pos( '~', aHintStr) > 0
      then aHintInfo.HintWindowClass := TKnobsHintWindow
      else aHintInfo.HintWindowClass := TKnobsSimpleHintWindow;
    end;


    function    TFormDllMain.DoCreateModuleBitmap( const aModuleType: TKnobsModuleType; UseCache: Boolean): TBitmap;
    var
      aModuleTypeName : string;
      anImageFileName : string;
      aJPEGImg        : TJPEGImage;
    begin

      // When help docs for the correct program version were created before and an image file is actually present
      // use that image file.

      aModuleTypeName := FormStore.BitmapName( aModuleType);
      anImageFileName := Format( '%s\images\%s', [ DocsPath, aModuleTypeName], AppLocale);

      if   UseCache
      and  FileExists( anImageFileName)
      then begin
        aJPEGImg := TJPEGImage.Create;

        try
          aJPEGImg.LoadFromFile( anImageFileName);
          Result := TBitmap.Create;
          Result.Assign( aJPEGImg);
        finally
          aJPEGImg.DisposeOf;
        end;
      end
      // and otherwise just create a (more basic) image.
      else Result := FormStore.CreateModuleBitmap( aModuleType);
    end;


    function    TFormDllMain.DoReadModuleComment ( const aModuleType: TKnobsModuleType): string;
    begin
      Result := FormStore.ReadModuleComment( aModuleType);
    end;


    procedure   TFormDllMain.DoPopupEditorShow( const aSender: TObject);
    begin
      FLastWiresVisible := WiresOff;
    end;


    procedure   TFormDllMain.DoPopupEditorHide( const aSender: TObject);
    begin
      WiresOn( FLastWiresVisible);
    end;


    procedure   TFormDllMain.DoHistoryItemClicked( aSender: TObject);
    begin
      if   Assigned( FPopupSender)
      and  IsUndoable( FPopupSender)
      and  ( aSender is TMenuItem)
      then ( FPopupSender as IKnobsUndoable).UndoItem( TMenuItem( aSender).Tag);
    end;


    procedure   TFormDllMain.RemoveSelectorMenuItems;
    var
      i : Integer;
    begin
      with PopupMenuSelector
      do begin
        for i := Items.Count - 1 downto 0
        do  Items.Delete( i);
      end;
    end;


    procedure   TFormDllMain.DoSelectorMenuItemClick( aSender: TObject);
    var
      aSelector : TKnobsSelector;
      aMenuItem : TMenuItem;
    begin
      aMenuItem := aSender as TMenuItem;

      if   Assigned( aMenuItem)
      then begin
        aSelector := TKnobsSelector( FPopupSender);
        aSelector.BeginStateChange;

        try
          aSelector.KnobPosition := aMenuItem.Tag;
        finally
          aSelector.EndStateChange;
        end;

        aSelector.FixPeers;
      end;
    end;


    procedure   TFormDllMain.AddSelectorMenuItem( anIndex, aShortcut: Integer; const aName: string);
    var
      aMenuItem : TMenuItem;
    begin
      aMenuItem := TMenuItem.Create( PopupMenuSelector);

      with aMenuItem
      do begin

        if   aName <> '-'
        then begin
          Caption    := Format( '%s (%d) %s', [ IndexToMenuShortCut( aShortcut), anIndex, aName], AppLocale);
          OnClick    := DoSelectorMenuItemClick;
          Tag        := anIndex;
          ImageIndex := anIndex;
        end
        else Caption := aName;
      end;

      PopupMenuSelector.Items.Add( aMenuItem);
    end;


    procedure   TFormDllMain.DoShowPopupMenu( const aSender: TKnobsWirePanel; const aControl: TControl);

      procedure PopUp( aPopUp: TPopupMenu; aControl: TControl);
      var
        aPoint : TPoint;
      begin
        FPopupSender := aControl;
        aPoint := Mouse.CursorPos;
        aPopup.Popup( aPoint.X + 10, aPoint.Y + 10);
      end;

      procedure AddHistoryItems( aMenuItem: TMenuItem; anUndoable: IKnobsUndoable; aMaxItems: Integer);
      var
        aList        : TStringList;
        i            : Integer;
        aSubMenuItem : TMenuItem;
      begin
        if   Assigned( anUndoable)
        then begin
          with aMenuItem
          do begin
            for i := Count - 1 downto 0
            do Delete( i);
          end;

          aMenuItem.Enabled := False;
          aList             := anUndoable.CreateStringList( aMaxItems); // max 36 items - 0 .. 9. A .. Z

          try
            for i := 0 to aList.Count - 1
            do begin
              aSubMenuItem := TMenuItem.Create( aMenuItem);

              with aSubMenuItem
              do begin
                Caption := Format( '%s%s', [ IndexToMenuShortCut( i), aList[ i]], AppLocale);
                Tag     := i;
                OnClick := DoHistoryItemClicked;
                Enabled := True;
              end;

              aMenuItem.Add( aSubMenuItem);
            end;
          finally
            aList.DisposeOf;
          end;

          aMenuItem.Enabled := aMenuItem.Count > 0;
        end;
      end;


      procedure ShowPopupModule( aPopup: TPopupMenu; aControl: TControl);
      var
        aModule    : TKnobsCustomModule;
      begin
        aModule := TKnobsCustomModule( aControl);
        MenuItemModuleExludeFromRandomization.Checked := not aModule.AllowRandomization;

        if   IsUndoable( aControl)
        then AddHistoryItems( MenuItemModuleHistory, aControl as IKnobsUndoable, MODULE_HISTORY_COUNT);

        PopUp( aPopup, aControl);
      end;


      procedure ShowPopupSelector( aPopup: TPopupMenu; aControl: TControl);
      var
        i         : Integer;
        p         : Integer;
        aMenuItem : TMenuItem;
        aRand     : IKnobsVariation;
        aSelect   : TKnobsSelector;
      begin
        if   Assigned( aControl)
        and  ( aControl is TKnobsSelector)
        then begin
          aSelect       := TKnobsSelector( aControl);
          p             := 0;
          aPopup.Images := aSelect.Glyphs;
          RemoveSelectorMenuItems;

          if   IsVariation( aControl)
          and  Assigned( aSender)
          then begin
            aRand     := aControl as IKnobsVariation;
            aMenuItem := TMenuItem.Create( PopupMenuSelector);

            with aMenuItem
            do begin
              Caption := Format( '%sRandomize', [ IndexToMenuShortCut( p)], AppLocale);
           // OnClick := MenuItemKnobRandomizeClick;
              Enabled := aRand.CanAllowRandomization and aRand.AllowRandomization;
            end;

            PopupMenuSelector.Items.Add( aMenuItem);
            Inc( p);

            aMenuItem := TMenuItem.Create( PopupMenuSelector);

            with aMenuItem
            do begin
              Caption := Format( '%sExclude from randomization', [ IndexToMenuShortCut( p)], AppLocale);
              Checked := not aRand.AllowRandomization;
           // OnClick := ToggleSelectorRandomizationClick;
              Enabled := aRand.CanAllowRandomization;
            end;

            PopupMenuSelector.Items.Add( aMenuItem);
            Inc( p);

            AddSelectorMenuItem( 0, -1, '-');
          end;

          if   IsUndoable( aControl)
          and  Assigned( aSender)
          and  ( aSelect.StepCount > 1)
          then begin
            aMenuItem := TMenuItem.Create( PopupMenuSelector);

            with aMenuItem
            do begin
              Caption := Format( '%sHistory', [ IndexToMenuShortCut( p)], AppLocale);
              Inc( p);
              OnClick := nil;
            end;

            PopupMenuSelector.Items.Add( aMenuItem);
            AddHistoryItems( aMenuItem, aControl as IKnobsUndoable, CONTROL_HISTORY_COUNT);
            AddSelectorMenuItem( 0, -1, '-');
          end;

          for i := 0 to aSelect.Captions.Count - 1
          do begin
            if   aSelect.Captions[ i] = '-'
            then AddSelectorMenuItem( -1, -1, '-')
            else begin
              AddSelectorMenuItem( i, p, aSelect.Captions[ i]);
              Inc( p);
            end;
          end;

          PopUp( aPopup, aControl);
        end;
      end;


      procedure ShowPopupKnob( aPopup: TPopupMenu; aControl: TControl);
      begin
        if   aControl is TKnobsKnob
        then LockKnob.Checked := TKnobsKnob( aControl).Locked
        else LockKnob.Visible := False;

        if   IsVariation( aControl)
        then begin
          MenuItemKnobExcludeFromRandomization.Checked := not ( aControl as IKnobsVariation).AllowRandomization;
          MenuItemKnobExcludeFromRandomization.Enabled := ( aControl as IKnobsVariation).CanAllowRandomization;
          MenuItemKnobRandomize               .Enabled := ( aControl as IKnobsVariation).CanAllowRandomization and ( aControl as IKnobsVariation).AllowRandomization;
        end
        else begin
          MenuItemKnobExcludeFromRandomization.Checked := True;
          MenuItemKnobExcludeFromRandomization.Enabled := False;
          MenuItemKnobRandomize               .Enabled := False;
        end;

        if   IsUndoable( aControl)
        then AddHistoryItems( MenuItemKnobHistory, aControl as IKnobsUndoable, CONTROL_HISTORY_COUNT);

        PopUp( aPopup, aControl);
      end;


      procedure ShowPopupXYControl( aPopup: TPopupMenu; aControl: TControl);
      begin
        if   IsVariation( aControl)
        then begin
          MenuItemXYControlExcludeFromRandomization.Checked := not ( aControl as IKnobsVariation).AllowRandomization;
          MenuItemXYControlExcludeFromRandomization.Enabled := ( aControl as IKnobsVariation).CanAllowRandomization;
          MenuItemXYControlRandomize               .Enabled := ( aControl as IKnobsVariation).CanAllowRandomization and ( aControl as IKnobsVariation).AllowRandomization;
        end
        else begin
          MenuItemXYControlExcludeFromRandomization.Checked := True;
          MenuItemXYControlExcludeFromRandomization.Enabled := False;
          MenuItemXYControlRandomize               .Enabled := False;
        end;

        PopUp( aPopup, aControl);
      end;


      procedure ShowPopupDataMaker( aPopup: TPopupMenu; aControl: TControl);
      begin
        if   IsVariation( aControl)
        then begin
          MenuItemDataMakerExcludeFromRandomization.Checked := not ( aControl as IKnobsVariation).AllowRandomization;
          MenuItemDataMakerExcludeFromRandomization.Enabled := ( aControl as IKnobsVariation).CanAllowRandomization;
          MenuItemDataMakerRandomize               .Enabled := ( aControl as IKnobsVariation).CanAllowRandomization and ( aControl as IKnobsVAriation).AllowRandomization;
        end
        else begin
          MenuItemDataMakerExcludeFromRandomization.Checked := True;
          MenuItemDataMakerExcludeFromRandomization.Enabled := False;
          MenuItemDataMakerRandomize               .Enabled := False;
        end;

        PopUp( aPopup, aControl);
      end;


      procedure ShowPopupEditor( aPopup: TPopupMenu; aControl: TControl);
      begin
        if   IsUndoable( aControl)
        then AddHistoryItems( MenuItemEditorHistory, aControl as IKnobsUndoable, EDITOR_HISTORY_COUNT);

        PopUp( aPopup, aControl);
      end;


      procedure ShowPopupTextControl( aPopup: TPopupMenu; aControl: TControl);
      begin
        TextMenuClear.Enabled := TKnobsTextControl( aControl).AllowClear;
        PopUp( aPopup, aControl);
      end;

      procedure ShowPopupSimple( aPopup: TPopupMenu; aControl: TControl);
      begin
        PopUp( aPopup, aControl);
      end;

    begin
      if      aControl is TKnobsConnector
      then ShowPopupSimple( PopupMenuConnector, aControl)
      else if aControl is TKnobsDataMaker
      then ShowPopupDataMaker( PopupMenuDataMaker, aControl)
      else if aControl is TKnobsWirePanel
      then ShowPopupEditor( PopupMenuEditor, aControl)
      else if aControl is TKnobsModule
      then ShowPopupModule( PopupMenuModule, aControl)
      else if aControl is TKnobsSelector
      then ShowPopupSelector( PopupMenuSelector, aControl)
      else if aControl is TKnobsValuedControl
      then ShowPopupKnob( PopupMenuKnob, aControl)
      else if aControl is TKnobsXYControl
      then ShowPopupXYControl( PopupMenuXYControl, aControl)
      else if aControl is TKnobsGridControl
      then ShowPopupSimple( PopupMenuGridControl, aControl)
      else if aControl is TKnobsTextControl
      then ShowPopupTextControl( PopupMenuTextControl, aControl);
    end;


    procedure   TFormDllMain.DoUnFocus( const aSender: TObject);
    begin
      EditPark.SetFocus;
    end;


    function    TFormDllMain.DoLoadDataGraph( const aSender: TObject): string;
    var
      aFile : TextFile;
    begin
      Result := '';

      try
        with OpenDialogGraphs
        do begin
          FileName   := ExtractFileName( FileName);
          InitialDir := FolderGraphs;

          if   Execute
          then begin
            AssignFile( aFile, FileName, CP_UTF8);
            Reset( aFile);

            try
              ReadLn( aFile, Result);
            finally
              CloseFile( aFile);
            end;
          end;
        end;
      except
        on E: Exception
        do KilledException( E);
      end;
    end;


    procedure   TFormDllMain.DoSaveDataGraph( const aSender: TObject; const aValue: string);
    var
      aFile : TextFile;
    begin
      try
        with SaveDialogGraphs
        do begin
          FileName   := ExtractFileName( FileName);
          InitialDir := FolderGraphs;

          if   Execute
          then begin
            AssignFile( aFile, FileName, CP_UTF8);
            Rewrite( aFile);

            try
              WriteLn( aFile, aValue);
            finally
              CloseFile( aFile);
            end;
          end;
        end;
      except
        on E: Exception
        do KilledException( E);
      end;
    end;


    function    TFormDllMain.DoLoadGridControl( const aSender: TObject): string;
    var
      aFile : TextFile;
    begin
      Result := '';

      try
        with OpenDialogGridControl
        do begin
          FileName   := ExtractFileName( FileName);
          InitialDir := FolderGrids;

          if   Execute
          then begin
            AssignFile( aFile, FileName, CP_UTF8);
            Reset( aFile);

            try
              ReadLn( aFile, Result);
            finally
              CloseFile( aFile);
            end;
          end;
        end;
      except
        on E: Exception
        do KilledException( E);
      end;
    end;


    procedure   TFormDllMain.DoSaveGridControl( const aSender: TObject; const aValue: string);
    var
      aFile : TextFile;
    begin
      try
        with SaveDialogGridControl
        do begin
          FileName   := ExtractFileName( FileName);
          InitialDir := FolderGrids;

          if   Execute
          then begin
            AssignFile( aFile, FileName, CP_UTF8);
            Rewrite( aFile);

            try
              WriteLn( aFile, aValue);
            finally
              CloseFile( aFile);
            end;
          end;
        end;
      except
        on E: Exception
        do KilledException( E);
      end;
    end;


    procedure   TFormDllMain.DoActiveVariationChanged( const aSender: TObject; aVariation: Integer);
    begin
    end;


    procedure   TFormDllMain.DoEditHistoryChanged( aSender: TObject);
    begin
      if   Assigned( FormLastEdits)
      then FormLastEdits.HistoryChanged;
    end;


    procedure   TFormDllMain.DoModulesRenamed( const aSender: TObject; const anOldNames, aNewNames: TStringArray);
    begin
    end;


    procedure   TFormDllMain.DoModulesRenamedFriendly( const aSender: TObject; const anOldNames, aNewNames: TStringArray);
    begin
    end;


    function    TFormDllMain.GetPatchName: string;
    begin
      if   Assigned( PatchEditor)
      then Result := PatchEditor.Title
      else Result := 'no name';
    end;


    procedure   TFormDllMain.SetPatchName( const aValue: string);
    begin
      if   ( aValue <> PatchName)
      and  Assigned( PatchEditor)
      then begin
        PatchEditor.Title := aValue;
        UpdateCaptions;
      end;
    end;


    function    TFormDllMain.GetSourceFileName: string;
    begin
      if   Assigned( PatchEditor)
      then Result := PatchEditor.Filename
      else Result := '';
    end;


    procedure   TFormDllMain.OpenRecentFile( const anIndex: Integer);
    var
      aFileName: string;
    begin
      if   Assigned( FRecentSourceFiles)
      and  ( anIndex >= 0)
      and  ( anIndex < FRecentSourceFiles.Count)
      then begin
        aFileName := FRecentSourceFiles[ anIndex];

        if   FileExists( aFileName)
        then DoLoadPatch( aFileName, True)
        else begin
          FRecentSourceFiles.Delete( anIndex);
          SetRecentMenuItems;
        end;
      end;
    end;


    procedure   TFormDllMain.DorecentMenuItemClick( aSender: TObject);
    var
      aMenuItem: TMenuItem;
    begin
      aMenuItem := aSender as TMenuItem;

      if   Assigned( aMenuItem)
      then OpenRecentFile( aMenuItem.Tag);
    end;


    procedure   TFormDllMain.AddRecentMenuItem( anIndex: Integer; const aFileName: string);
    var
      aMenuItem : TMenuItem;
    begin
      if   FileExists( aFileName)
      then begin
        aMenuItem := TMenuItem.Create( PopupMenuFileReopenPatch);

        with aMenuItem
        do begin
          Caption  := Format( '%s%s', [ IndexToMenuShortCut( anIndex), aFileName], AppLocale);
          Tag      := anIndex;
          OnClick  := DorecentMenuItemClick;
        end;

        PopupMenuFileReopenPatch.Add( aMenuItem);
      end;
    end;


    procedure   TFormDllMain.RemoveRecentMenuItems;
    begin
      while PopupMenuFileReopenPatch.Count > 0
      do    PopupMenuFileReopenPatch[ 0].DisposeOf;
    end;


    procedure   TFormDllMain.SetRecentMenuItems;
    var
      i : Integer;
    begin
      RemoveRecentMenuItems;

      for i := 0 to FRecentSourceFiles.Count - 1
      do  AddRecentMenuItem( i, FRecentSourceFiles[ i]);
    end;


    procedure   TFormDllMain.SetMostRecentFile( const aFileName: string);
    begin
      FRecentSourceFiles.SetMostRecent( aFileName);
      SetRecentMenuItems;
    end;


    procedure   TFormDllMain.SetSourceFileName( const aValue: string);
    begin
      if   ( aValue <> SourceFileName)
      and  Assigned( PatchEditor)
      then begin
        PatchEditor.Filename := aValue;
        SetMostRecentFile( aValue);
        UpdateCaptions;
      end;
    end;


    function    TFormDllMain.FunctionalPinName( const aName: string): string;
    begin
      Result := ParsePostfix( aName);
    end;


    function    TFormDllMain.SplitConnectorName( const aName: string; var aMod, aPin: string): Boolean;
    var
      aList: TStringList;
    begin
      aMod   := '';
      aPin   := '';
      Result := False;
      aList  := Explode( aName, '.');

      try
        if   aList.Count = 2
        then begin
          aMod   := aList[ 0];
          aPin   := aList[ 1];
          Result := not ( SameText( aMod, '<nil>') or SameText( aPin, '<nil>'));

          if   Result
          then aPin := FunctionalPinName( aPin);
        end;
      finally
        aList.DisposeOf;
      end;
    end;


    function    TFormDllMain.ModuleTypeToModule( aType: Integer; const aParent: TSynthesizer; const aName: string; aVersion, aPolyphony: Integer): TMod;
    // This couples an editor module ID to it's synth implementation
    // Stuff that needs or can not be compiled, i.e. 'editor only' modules should return nil here
    var
      M : TModuleClass;
    begin
      Result := nil;
      M      := LookupModuleClass( aType);

      if   Assigned( M)
      and  Assigned( aParent)
      then Result := M.Create( aParent, aName, aVersion, aPolyphony);
    end;


    procedure   TFormDllMain.CompilePatch( ModulesChanged, AutomaticCompile: Boolean);
    // This creates a synth patch from an editor patch, deletes the old synth patch
    // and plugs in the new one.
    var
      aSynthesizer   : TSynthesizer;
      i             : Integer;
      aPatchModule  : TKnobsCustomModule;
      aModuleType   : Integer;
      aModuleName   : string;
      aModule       : TMod;
      aVersion      : Integer;
      aSrcMod       : string;
      aSrcPin       : string;
      aDstMod       : string;
      aDstPin       : string;
    begin
      Log( LC_COMPILER, 'compiling synth patch');

      if   Assigned( PatchEditor)
      then begin
        try
          Polyphony := PatchEditor.Polyphony;
          FixModuleStatics;

          aSynthesizer := TSynthesizer.Create( nil, PatchName, PatchEditor.Version, Polyphony);
          aSynthesizer.Initialize;
          TimerLightsUpdate.Enabled := False;

          try
            with PatchEditor
            do begin
              MakeSpedupSlow;
              FixAllWires;      // Get the wire directions right before wire compilation
              SortGraph;        // Sort modules according to strongly connected components to get shortest loop delays

              Log( LC_COMPILER, 'compiling synth patch - modules');

              for i := 0 to ModuleCount - 1
              do begin
                aPatchModule := Module[ i];
                aModuleType  := aPatchModule.ModuleType;
                aModuleName  := aPatchModule.Name;
                aVersion     := aPatchModule.Version;
                aModule      := ModuleTypeToModule( aModuleType, aSynthesizer, aModuleName, aVersion, Polyphony);

                if   Assigned( aModule)                    // Module translates, add it, set it's OSC properties and speed it up if needed
                then begin
                  aModule.Initialize;
                  aSynthesizer.AddModule( aModule);

                  if   aPatchModule.IsSpedUp
                  then begin
                    if   aModule.IsFast
                    then LogFmt( LC_COMPILER, 'Tried to speed up fast module type %s, this is not allowed', [ aModule.ClassName])
                    else aModule.MakeFast;
                  end;
                end;
              end;

              BuildModuleIndex;
              Log( LC_COMPILER, 'compiling synth patch - wires');

              for i := 0 to WireCount - 1
              do begin
                if   SplitConnectorName( Wire[ i].SourceName     , aSrcMod, aSrcPin)
                and  SplitConnectorName( Wire[ i].DestinationName, aDstMod, aDstPin)
                then begin
                  try
                    aSynthesizer.AddConnection( aSrcMod, aSrcPin, aDstMod, aDstPin);
                  except
                    on E: Exception
                    do LogFmt( LC_COMPILER, 'Error adding connection: ''%s''', [ E.ToString]);
                  end;
                end;
              end;

              aSynthesizer.CompileLinks;
              aSynthesizer.MidiChannel := 16; // Omni mode for now
              Log( LC_COMPILER, 'compiling synth patch - switching in new synth patch');

              // Try to switch in the newly created synth patch

              if   not ChangeSynthesizerTo( aSynthesizer, ModulesChanged)
              then FreeAndNil( aSynthesizer);

              if   Assigned( FSynthesizer)
              then begin
                Log( LC_COMPILER, 'compiling synth patch - post processing');
                FSynthesizer.FixDeZippers;                                      // Must fix the zippers in the new patch, always

                for i := 0 to RangeCount - 1
                do  FSynthesizer.Morph[ i] := Knobs2013.RangeValue( i);

                FSynthesizer.Reset;
                PatchEditor.FixAllSynthParams( False);
                FSynthesizer.GatherSignals   ( '', DoSignals   );
                FSynthesizer.GatherLights    ( '', DoLights    );
                FSynthesizer.GatherData      ( '', DoData      );
                FSynthesizer.GatherXYData    ( '', DoXYData    );
                FSynthesizer.GatherStringData( '', DoStringData);
                FSynthesizer.GatherCursorData( '', DoCursorData);
                FSynthesizer.GatherWaveData  ( '', DoWaveData  );
                PatchEditor.FixAllSynthParams( False);

                // Some debug stuff - write the just created synth patch to test.txt - in the current (application) directory

              end
              else Log( LC_COMPILER, 'compiling synth patch - No new synth patch, this is odd, but no post processing needed in that case');
            end; // with WirePanel ..

            TimerLightsUpdate.Enabled := True;
            FSampleCount              := 0;
            Log( LC_COMPILER, 'compiled synth patch');
          except
            on E: Exception
            do begin
              LogFmt( LC_COMPILER, 'Error compiling synth patch: %s', [ E.ToString]);
              FreeAndNil( aSynthesizer);
            end;
          end;
        except
          on E: Exception
          do LogFmt( LC_COMPILER, 'Error in compilation: %s', [ E.ToString]);
        end;
      end; // if Assigned( PatchEditor)
    end;


    procedure   TFormDllMain.DoRecompile( const aSender: TKnobsWirePanel; ModulesChanged: Boolean);
    begin
      CompilePatch( ModulesChanged, True);
    end;


    function    TFormDllMain.ChangeSynthesizerTo( const aSynthesizer: TSynthesizer; ModulesChanged: Boolean): Boolean;
    // Swap in a new synth patch, return false when the patch did not change.
    var
      OldSynth    : TSynthesizer;
    begin
      Result := False;

      if   Assigned( aSynthesizer)
      then begin
        // Do not change the patch if it's 'the same' or 'equivalent' at least.
        // But always do change when modules were changed, or otherwise the
        // mapping of the patch to the synth may be wrong.
        // When IsSamePatch retured true there still may be added or deleted moodules,
        // in the case such modules are uncompiled ones, like blanks, comments etc.

        if   not aSynthesizer.IsSamePatch( FSynthesizer)
        or   ModulesChanged
        then begin
          aSynthesizer.Locked          := True;                    // Do not allow execution of the new patch yet
       // aSynthesizer.OnSendShortMidi := DoMidiTxShortMessage;    // Set up callbacks for new patch
       // aSynthesizer.OnSendOSCString := DoSendOSCBytes;
       // aSynthesizer.OnSendMidiBytes := DoSendMidiBytes;
          aSynthesizer.OnLog           := DoSynthLog;
          aSynthesizer.FixDeZippers;                               // Set the dezipper registers to the knob values

          if   Assigned( FSynthesizer)
          then begin
            FSynthesizer.Locked          := True;                // Stop executing the old patch
            FSynthesizer.OnSendOSCString := nil;
          end;

          OldSynth  := AtomicExchange( Pointer( FSynthesizer), Pointer( aSynthesizer));
          PatchName := FSynthesizer.Name;
          FSynthesizer.Locked := False;                          // Allow execution for new patch

          if   Assigned( OldSynth)
          then FreeAndNil( OldSynth);

          Result := True;
        end;
      end
      else begin
        Log( LC_COMPILER, 'BUG: ChangeSynthesizer, aSynthesizer not assigned');
      end;
    end;


    procedure   TFormDllMain.DoHistoryChanged( const aSender: TObject; anUndoCount, aRedoCount, aSavedMarker: Integer);
    begin
      PopupMenuEditUndo.Enabled := anUndoCount  > 0;
      PopupMenuEditRedo.Enabled := aRedoCount   > 0;
    // PatchChanged         := anUndoCount <> aSavedMarker;
    end;


    function    TFormDllMain.DoGetFolder( const aSender: TObject; aFolderKind: TKnobsFolderKind): string;
    begin
      case aFolderKind of
        fkFolderPatches       : Result := FolderPatches;
        fkFolderTemplates     : Result := FolderTemplates;
        fkFolderSamplesWaves  : Result := FolderSamplesWaves;
        fkFolderRecordings    : Result := FolderRecordings;
        fkFolderScala         : Result := FolderScala;
        fkFolderMidiFiles     : Result := FolderMidiFiles;
        fkFolderPresets       : Result := FolderPresets;
        fkFolderExportsImports: Result := FolderExportsImports;
        fkFolderCsvFiles      : Result := FolderCsvFiles;
        fkFolderGraphs        : Result := FolderGraphs;
        fkFolderGrids         : Result := FolderGrids;
        fkFolderGenes         : Result := FolderGenes;
        fkFolderTextWriter    : Result := FolderTextWriter;
      else
                                Result := ApplicationPath;
      end;
    end;


    procedure   TFormDllMain.DoValueChanged( const aSender: TObject; const aPath, aControlType: string; aValue: TSignal; IsFinal, IsAutomation: Boolean);
    // An editor module's control changed value, to be reflected in a synth setting
    var
      aPrefix        : string;
      aFullname      : string;
      aValuedControl : TKnobsValuedControl;
    begin
      if   Assigned( FSynthesizer)
      then begin
        if   IsFinal or IsAutomation
        then begin
          if   aSender is TKnobsValuedControl
          then aValuedControl := TKnobsValuedControl( aSender)
          else aValuedControl := nil;

          HandleSpecialValues( aPath, aControlType, aValue, aValuedControl); // Handle some special cases here - stuff in the editor may need change
        end;

        ParsePrefix( aPath, aPrefix, aFullname);                     // Parse off the container prefix - and ignore that.

        if   IsAutomation
        then FSynthesizer.AcceptAutomationParam( aFullname, aValue)   // Pass on the value to the Synth as an automation
        else FSynthesizer.AcceptParam          ( aFullname, aValue);  // Pass on the value to the Synth as a user action
      end;
    end;


    procedure   TFormDllMain.DoTextChanged( const aSender: TObject; const aPath, aValue: string);
    // An editor module's text control changed value, to be reflected in a synth setting
    // These changes are assumed to always be 'final' - i.e. 'inbetween' edits should
    // not make it to here. Text changes can be automated .. for a grid for instance or graph thingies
    var
      aPrefix        : string;
      aRest          : string;
      aModuleName    : string;
      aFullParamName : string;
      aModuleIndex   : Integer;
      aModule        : TMod;
      aParts         : TStringList;
    begin
      if   Assigned( FSynthesizer)
      then begin
        ParsePrefix( aPath, aPrefix, aRest);                   // Parse off the container prefix - and ignore that.
        ParsePrefix( aRest, aModuleName, aFullParamName);
        aModuleIndex := FSynthesizer.FindModule( aModuleName);

        if   aModuleIndex >= 0
        then begin
          aModule := FSynthesizer.Module[ aModuleIndex];

          if   Assigned( aModule)
          then begin

            if   ( aModule is TModTextSequencer) and SameText( aFullParamName, 'textsequencer_values')
            then TModTextSequencer( aModule).SetStepValues( ParseTextSequencerValue( aValue))
            else if
                 (( aModule is TModWavePlayer   ) and SameText( aFullParamName, 'waveplayer_fileselector_filename'   ))
              or (( aModule is TModWavePlayTrig ) and SameText( aFullParamName, 'waveplaytrig_fileselector_filename' ))
              or (( aModule is TModWavePlayTrig2) and SameText( aFullParamName, 'waveplaytrig2_fileselector_filename'))
              or (( aModule is TModCsvData      ) and SameText( aFullParamName, 'csvdata_fileselector_filename'      ))
              or (( aModule is TModGenePlayer   ) and SameText( aFullParamName, 'geneplayer_fileselector_filename'   ))
              or (( aModule is TModMidiPlayer   ) and SameText( aFullParamName, 'midiplayer_fileselector_filename'   ))
              or (( aModule is TModPad          ) and SameText( aFullParamName, 'pad_fileselector_filename'          ))
              or (( aModule is TModSwanSong     ) and SameText( aFullParamName, 'swansong_fileselector_filename'     ))
              or (( aModule is TModTextWriter   ) and SameText( aFullParamName, 'textwriter_fileselector_filename'   ))
              or (( aModule is TModRecorder     ) and SameText( aFullParamName, 'recorder_fileselector_filename'     ))
              or (( aModule is TModScala        ) and SameText( aFullParamName, 'scala_fileselector_mapping'         ))
              or (( aModule is TModScala        ) and SameText( aFullParamName, 'scala_fileselector_scale'           ))
              or (( aModule is TModMorse        ) and SameText( aFullParamName, 'morse_display_text'                 ))
              or (( aModule is TModTSS          ) and SameText( aFullParamName, 'tss_display_text'                   ))
              or (( aModule is TModSong         ) and SameText( aFullParamName, 'song_display_text'                  ))
              or (( aModule is TModSapiVoice    ) and SameText( aFullParamName, 'sapivoice_display_text'             ))
              or (( aModule is TModESpeakVoice  ) and SameText( aFullParamName, 'espeakvoice_display_text'           ))
              or (( aModule is TModOscMessage   ) and SameText( aFullParamName, 'oscmessage_display_address'         ))
              or (( aModule is TModTextWriter   ) and SameText( aFullParamName, 'textwriter_display_text'            ))
              or (( aModule is TModValues       ) and SameText( aFullParamName, 'values_display_values'              ))
              or (( aModule is TModSieve        ) and SameText( aFullParamName, 'sieve_display_formula'              ))
              or (( aModule is TModClockedSieve ) and SameText( aFullParamName, 'clockedsieve_display_formula'       ))
              or (( aModule is TModMidiSysexOut ) and SameText( aFullParamName, 'midisysexout_display_message'       ))
              or (( aModule is TModSeqValues    ) and SameText( aFullParamName, 'seqvalues_valuedata'                ))
              or (( aModule is TModSeqValues    ) and SameText( aFullParamName, 'seqvalues_espdata'                  ))
              or (( aModule is TModLifeSeq      ) and SameText( aFullParamName, 'lifeseq_grid'                       ))
              or (( aModule is TModLifeSeq      ) and SameText( aFullParamName, 'lifeseq_stops'                      ))
              or (( aModule is TModSmallLifeSeq ) and SameText( aFullParamName, 'smalllifeseq_grid'                  ))
              or (( aModule is TModSmallLifeSeq ) and SameText( aFullParamName, 'vitaphobium_grid'                   ))
              or (( aModule is TModSeqPattern   ) and SameText( aFullParamName, 'seqpattern_bitdata'                 ))
              or (( aModule is TModSeqPattern   ) and SameText( aFullParamName, 'seqpattern_espdata'                 ))
            then begin
              aModule.StringValue[ ParsePostfix( aFullParamName)] := aValue;
            end
            else if
              (( aModule is TModLoopPlayer   ) and SameText( aFullParamName, 'loopplayer_fileselector_filename'   ))
            then begin
              aModule.StringValue[ ParsePostfix( aFullParamName)] := aValue;
            end
            else if
                 (( aModule is TModDataGraph   ) and SameText( aFullParamName, 'datagraph_data'                   ))
              or (( aModule is TModAudioGraph  ) and SameText( aFullParamName, 'audiograph_data'                  ))
            then begin
              aModule.StringValue[ ParsePostfix( aFullParamName)] := aValue;
            end
            else if
                 ( aModule is TModRewriter      )
              or ( aModule is TModRewriterNote  )
              or ( aModule is TModTuneSmithy    )
              or ( aModule is TModEqDivQuantizer)
            then begin
              aParts := Explode( aFullParamName, '_');

              try
                if   ( aParts.Count = 3)
                and  SameText( aParts[ 1], 'display')
                then aModule.StringValue[ aParts[ 2]] := aValue;
              finally
                aParts.DisposeOf;
              end;
            end

            else begin
              aParts := Explode( aFullParamName, '_');

              try
                if   ( aParts.Count = 3)
                and  SameText( aParts[ 1], 'display')
                then begin
                  if   aSender is TKnobsValuedControl
                  then aModule.InternalStringValue[ TKnobsValuedControl( aSender).ControlType, aParts[ 2]] := aValue
                  else aModule.InternalStringValue[ ''                                       , aParts[ 2]] := aValue;   // Which can set both a param value or an internal value
                end;
              finally
                aParts.DisposeOf;
              end;
            end;
          end;
        end;
      end;
    end;


    function    TFormDllMain.SplitValueInfo( const aPath: string; var aModule: TKnobsCustomModule; var aModuleType, aSignalName: string): Boolean;
    var
      aParts      : TStringList;
      bParts      : TStringList;
      aModuleName : string;
      aMod        : TKnobsCustomModule;
    begin
      Result      := False;
      aModule     := nil;
      aModuleType := '';
      aSignalName := '';
      aParts := Explode( aPath, '/');

      try
        if   aParts.Count >= 2
        then begin
          aModuleName := aParts[ aParts.Count - 2];
          aMod        := PatchEditor.FindModule( aModuleName);

          if   Assigned( aMod)
          then begin
            aSignalName := aParts[ aParts.Count - 1];
            bParts      := Explode( aSignalName, '_');

            try
              if   bParts.Count = 2
              then begin
                aModuleType := bParts[ 0];
                aSignalName := bParts[ 1];
                aModule     := aMod;
                Result      := True;
              end;
            finally
              bParts.DisposeOf;
            end;
          end;
        end;
      finally
        aParts.DisposeOf;
      end;
    end;


    procedure   TFormDllMain.HandleSpecialValues( const aPath, aControlType: string; aValue: TSignal; const anOrigin: TKnobsValuedControl);
    var
      aPostFix    : string;
      aModule     : TKnobsCustomModule;
      aModuleType : string;
      aSignalName : string;
    begin
      if   SplitValueInfo( aPath, aModule, aModuleType, aSignalName)
      then begin
        aPostFix := ParsePostFix( aPath);

        if   SameText( aPostFix, 'range'      )
        or   SameText( aPostFix, 'arange'     )
        or   SameText( aPostFix, 'hrange'     )
        or   SameText( aPostFix, 'irange'     )
        or   SameText( aPostFix, 'drange'     )
        or   SameText( aPostFix, 'rrange'     )
        or   SameText( aPostFix, 'clr'        )
        or   SameText( aPostFix, 'clrone'     )
        or   SameText( aPostFix, 'clrsome'    )
        or   SameText( aPostFix, 'rnd'        )
        or   SameText( aPostFix, 'rndone'     )
        or   SameText( aPostFix, 'rndsome'    )
        or   SameText( aPostFix, 'shuffle'    )
        or   SameText( aPostFix, 'rndf'       )
        or   SameText( aPostFix, 'rndq'       )
        or   SameText( aPostFix, 'rnda'       )
        or   SameText( aPostFix, 'voice'      )
        then ChangeRangeFor( aModule, aModuleType, aPostFix, aValue, anOrigin)
        else
        if   SameText( aPostFix, 'mode'       )
        or   SameText( aPostFix, 'mode1'      )
        or   SameText( aPostFix, 'mode2'      )
        or   SameText( aPostFix, 'mode3'      )
        or   SameText( aPostFix, 'mode4'      )
        or   SameText( aPostFix, 'lock'       )
        or   SameText( aPostFix, 'lockxy'     )
        or   SameText( aPostFix, 'valmode'    )
        or   SameText( aPostFix, 'autoscale'  )
        then ChangeModeFor( aModule, aModuleType, aPostFix, aValue);
      end;
    end;


    procedure   TFormDllMain.ChangeRangeFor( const aModule: TKnobsCustomModule; const aModuleType, aRangeType: string; aValue: TSignal; const anOrigin: TKnobsValuedControl);
    // This is meant to be used when a range value for a module changes, it is used to dynamically alter
    // control types for knobs affected by the curent range.
    var
      aKnob       : TKnobsKnob;
      aSelector   : TKnobsSelector;
      aKnobName   : string;
      i           : Integer;
      p           : Integer;
      anEnvRange  : TEnvRange;
      anLfoRange  : TLfoRange;
      aDelayRange : TDelayRange;
    begin
      if   Assigned( aModule)
      then begin
        with aModule
        do begin
          if   SameText( aModuleType, 'envahd'     )
          or   SameText( aModuleType, 'envarretrig')
          or   SameText( aModuleType, 'envadsr'    )
          or   SameText( aModuleType, 'pulsedelay' )
          then begin
            anEnvRange := SignalToEnvRange( aValue);

            for i := 0 to ControlCount - 1
            do begin
              if   Controls[ i] is TKnobsKnob
              then begin
                aKnob     := TKnobsKnob( Controls[ i]);
                aKnobName := aKnob.Name;

                if   ( SameText( aKnobName, 'envahd_attack'      ) and SameText( aRangeType, 'arange'))
                or   ( SameText( aKnobName, 'envahd_hold'        ) and SameText( aRangeType, 'hrange'))
                or   ( SameText( aKnobName, 'envahd_decay'       ) and SameText( aRangeType, 'drange'))
                or   ( SameText( aKnobName, 'envadsr_attack'     ) and SameText( aRangeType, 'arange'))
                or   ( SameText( aKnobName, 'envadsr_decay'      ) and SameText( aRangeType, 'drange'))
                or   ( SameText( aKnobName, 'envadsr_release'    ) and SameText( aRangeType, 'rrange'))
                or   ( SameText( aKnobName, 'envarretrig_attack' ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'envarretrig_release') and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'pulsedelay_delay'   ) and SameText( aRangeType, 'range' ))
                then begin
                  if   aModule.IsSpedUp
                  then begin
                    case anEnvRange of
                      erFast   : ChangeControlTypeTo( aKnob, 'EnvTimeFastAr'  );
                      erMedium : ChangeControlTypeTo( aKnob, 'EnvTimeMediumAr');
                      erSlow   : ChangeControlTypeTo( aKnob, 'EnvTimeSlowAr'  );
                      erVFast  : ChangeControlTypeTo( aKnob, 'EnvTimeVFastAr' );
                    end;
                  end
                  else begin
                    case anEnvRange of
                      erFast   : ChangeControlTypeTo( aKnob, 'EnvTimeFast'  );
                      erMedium : ChangeControlTypeTo( aKnob, 'EnvTimeMedium');
                      erSlow   : ChangeControlTypeTo( aKnob, 'EnvTimeSlow'  );
                      erVFast  : ChangeControlTypeTo( aKnob, 'EnvTimeVFast' );
                    end;
                  end;
                end;
              end;
            end;
          end

          else if SameText( aModuleType, 'rndgranulator'   )
               or SameText( aModuleType, 'granulator'      )
               or SameText( aModuleType, 'simplegranulator')
          then begin
            anEnvRange  := SignalToEnvRange( aValue);
            aDelayRange := SignalToDelayRange( aValue);

            for i := 0 to ControlCount - 1
            do begin
              if   Controls[ i] is TKnobsKnob
              then begin
                aKnob     := TKnobsKnob( Controls[ i]);
                aKnobName := aKnob.Name;

                if   ( SameText( aKnobName, 'rndgranulator_interonset'      ) and SameText( aRangeType, 'irange'))
                or   ( SameText( aKnobName, 'rndgranulator_grainduration'   ) and SameText( aRangeType, 'drange'))
                or   ( SameText( aKnobName, 'granulator_interonset'         ) and SameText( aRangeType, 'irange'))
                or   ( SameText( aKnobName, 'granulator_grainduration'      ) and SameText( aRangeType, 'drange'))
                or   ( SameText( aKnobName, 'simplegranulator_grainduration') and SameText( aRangeType, 'drange'))
                then begin
                  case anEnvRange of
                    erFast   : ChangeControlTypeTo( aKnob, 'EnvTimeFast' );
                    erMedium : ChangeControlTypeTo( aKnob, 'EnvTimeVFast'); // <-- this got mapped to vfast in reality ...
                  end;
                end
                else if ( SameText( aKnobName, 'rndgranulator_delaytime'   ) and SameText( aRangeType, 'range' ))
                     or ( SameText( aKnobName, 'granulator_delaytime'      ) and SameText( aRangeType, 'range' ))
                     or ( SameText( aKnobName, 'simplegranulator_delaytime') and SameText( aRangeType, 'range' ))
                then begin
                  case aDelayRange of
                    dsShort   : begin ChangeControlTypeTo( aKnob, 'DelayTimeShort'  ); ChangeStepCountTo( aKnob,  256); end;
                    dsMedium  : begin ChangeControlTypeTo( aKnob, 'DelayTimeMedium' ); ChangeStepCountTo( aKnob,  256); end;
                    dsLong    : begin ChangeControlTypeTo( aKnob, 'DelayTimeLong'   ); ChangeStepCountTo( aKnob,  256); end;
                    dsShort2  : begin ChangeControlTypeTo( aKnob, 'DelayTimeShort2' ); ChangeStepCountTo( aKnob, 8192); end;
                    dsMedium2 : begin ChangeControlTypeTo( aKnob, 'DelayTimeMedium2'); ChangeStepCountTo( aKnob, 8192); end;
                    dsLong2   : begin ChangeControlTypeTo( aKnob, 'DelayTimeLong2'  ); ChangeStepCountTo( aKnob, 8192); end;
                  end;
                end;
              end;
            end;
          end

          else if SameText( aModuleType, 'lfo'           )
               or SameText( aModuleType, 'fastlfo'       )
               or SameText( aModuleType, 'lfomultiphase' )
               or SameText( aModuleType, 'squarelfo'     )
               or SameText( aModuleType, 'squaresinelfo' )
               or SameText( aModuleType, 'lfotrig'       )
               or SameText( aModuleType, 'squarelfotrig' )
               or SameText( aModuleType, 'randsig'       )
               or SameText( aModuleType, 'randsigs'      )
               or SameText( aModuleType, 'randomwalklfo' )
               or SameText( aModuleType, 'attractorLfo'  )
               or SameText( aModuleType, 'vanderpolLfo'  )
               or SameText( aModuleType, 'logisticmaplfo')
               or SameText( aModuleType, 'pulses'        )
          then begin
            anLfoRange := SignalToLfoRange( aValue);

            for i := 0 to ControlCount - 1
            do begin
              if   Controls[ i] is TKnobsKnob
              then begin
                aKnob     := TKnobsKnob( Controls[ i]);
                aKnobName := aKnob.Name;

                if   ( SameText( aKnobName, 'lfo_frequency'           ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'fastlfo_frequency'       ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'lfomultiphase_frequency' ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'lfotrig_frequency'       ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'squarelfo_frequency'     ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'squaresinelfo_frequency' ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'squarelfotrig_frequency' ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'randsig_frequency'       ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'randsigs_frequency'      ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'randomwalklfo_frequency' ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'attractorlfo_frequency'  ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'vanderpollfo_frequency'  ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'logisticmaplfo_frequency') and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'pulses_frequency'        ) and SameText( aRangeType, 'range' ))
                then begin
                  case anLfoRange of
                    lsFast    : ChangeControlTypeTo( aKnob, 'LfoFreqCoarseFast'   );
                    lsMedium  : ChangeControlTypeTo( aKnob, 'LfoFreqCoarseMedium' );
                    lsSlow    : ChangeControlTypeTo( aKnob, 'LfoFreqCoarseSlow'   );
                    lsBPM     : ChangeControlTypeTo( aKnob, 'LfoFreqCoarseBPM'    );
                    lsLin     : ChangeControlTypeTo( aKnob, 'LfoFreqCoarseLin'    );
                    lsFast2   : ChangeControlTypeTo( aKnob, 'LfoFreqCoarseFast2'  );
                    lsMedium2 : ChangeControlTypeTo( aKnob, 'LfoFreqCoarseMedium2');
                    lsSlow2   : ChangeControlTypeTo( aKnob, 'LfoFreqCoarseSlow2'  );
                  end;
                end
              end;
            end;
          end

          else if SameText( aModuleType, 'delay'           )
               or SameText( aModuleType, 'delaystereo'     )
               or SameText( aModuleType, 'delaymod'        )
               or SameText( aModuleType, 'delaymix'        )
               or SameText( aModuleType, 'delay8'          )
               or SameText( aModuleType, 'looper'          )
               or SameText( aModuleType, 'stereolooper'    )
               or SameText( aModuleType, 'graindelay'      )
          then begin
            aDelayRange := SignalToDelayRange( aValue);

            for i := 0 to ControlCount - 1
            do begin
              if   Controls[ i] is TKnobsKnob
              then begin
                aKnob     := TKnobsKnob( Controls[ i]);
                aKnobName := aKnob.Name;

                if   ( SameText( aKnobName, 'delay_length'              ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'delaystereo_length'        ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'delaymod_length'           ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'delaymix_length'           ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'delay8_length'             ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'looper_length'             ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'stereolooper_length'       ) and SameText( aRangeType, 'range' ))
                or   ( SameText( aKnobName, 'graindelay_length'         ) and SameText( aRangeType, 'range' ))
                then begin
                  case aDelayRange of
                    dsShort   : begin ChangeControlTypeTo( aKnob, 'DelayTimeShort'  ); ChangeStepCountTo( aKnob,  256); end;
                    dsMedium  : begin ChangeControlTypeTo( aKnob, 'DelayTimeMedium' ); ChangeStepCountTo( aKnob,  256); end;
                    dsLong    : begin ChangeControlTypeTo( aKnob, 'DelayTimeLong'   ); ChangeStepCountTo( aKnob,  256); end;
                    dsShort2  : begin ChangeControlTypeTo( aKnob, 'DelayTimeShort2' ); ChangeStepCountTo( aKnob, 8192); end;
                    dsMedium2 : begin ChangeControlTypeTo( aKnob, 'DelayTimeMedium2'); ChangeStepCountTo( aKnob, 8192); end;
                    dsLong2   : begin ChangeControlTypeTo( aKnob, 'DelayTimeLong2'  ); ChangeStepCountTo( aKnob, 8192); end;
                  end;
                end;
              end;
            end;
          end

          else if SameText( aModuleType, 'sequencer'   )
               or SameText( aModuleType, 'gateseq'     )
               or SameText( aModuleType, 'seq16'       )
               or SameText( aModuleType, 'seq16rep'    )
               or SameText( aModuleType, 'leakygateseq')
               or SameText( aModuleType, 'vcps'        )
               or SameText( aModuleType, 'lut'         )
               or SameText( aModuleType, 'multicompare')
          then begin
            p := Random( Round( aValue));

            for i := 0 to ControlCount - 1
            do begin
              if   Controls[ i] is TKnobsKnob
              then begin
                aKnob := TKnobsKnob( Controls[ i]);

                if   ( aKnob.Tag = 1)
                and  not aKnob.Locked
                then begin
                  if   SameText( aRangeType, 'rnd')
                  then ChangeKnobPositionTo( aKnob, Random( aKnob.StepCount))
                  else if SameText( aRangeType, 'clr')
                  then begin
                    if   SameText( aKnob.ControlType, 'NoteName')
                    then ChangeKnobPositionTo( aKnob, 69)
                    else ChangeKnobPositionTo( aKnob,  0);
                  end
                  else if SameText( aRangeType, 'rndone')
                  then begin
                    if p = 0
                    then ChangeKnobPositionTo( aKnob, Random( aKnob.StepCount));

                    Dec( p);
                  end
                end;
              end
              else if  Controls[ i] is TKnobsSelector        // For the gateseq and leakygateseq only, to select gate modes
              then begin
                aSelector := TKnobsSelector( Controls[ i]);

                if   ( aSelector.Tag = 1)
                and  not aSelector.Locked
                then begin
                  if   SameText( aRangeType, 'rnd')
                  then ChangeKnobPositionTo( aSelector, Random( aSelector.StepCount))
                  else if SameText( aRangeType, 'clr')
                  then ChangeKnobPositionTo( aSelector, 0);
                end;
              end;
            end;
          end

          else if SameText( aModuleType, 'modal')
          then begin
            for i := 0 to ControlCount - 1
            do begin
              if   Controls[ i] is TKnobsKnob
              then begin
                aKnob := TKnobsKnob( Controls[ i]);

                if   SameText( aRangeType, 'rndf')          // Random frequency
                then begin
                  if   aKnob.Tag = 1
                  then ChangeKnobPositionTo( aKnob, aKnob.StepCount div 4 + Random( aKnob.StepCount div 2));
                end
                else if SameText( aRangeType, 'rndq')       // Random Q
                then begin
                  if   aKnob.Tag = 2
                  then ChangeKnobPositionTo( aKnob, aKnob.StepCount div 2 + Random( aKnob.StepCount div 2 - 3));
                end
                else if SameText( aRangeType, 'rnda')       // Random amplitude
                then begin
                  if   aKnob.Tag = 3
                  then ChangeKnobPositionTo( aKnob, 3 * aKnob.StepCount div 4 + Random( aKnob.StepCount div 4));
                end;
              end;
            end;
          end

          else if SameText( aModuleType, 'probsequencer')
          then begin
            for i := 0 to ControlCount - 1
            do begin
              if   SameText( aRangeType, 'rnd')
              then begin
                if   Controls[ i] is TKnobsSlider
                then begin
                  aKnob := TKnobsKnob( Controls[ i]);

                  if   aKnob.Tag = 1
                  then ChangeKnobPositionTo( aKnob, Random( aKnob.StepCount));
                end;
              end
              else if SameText( aRangeType, 'clr')
                then begin
                if   Controls[ i] is TKnobsSlider
                then begin
                  aKnob := TKnobsKnob( Controls[ i]);

                  if   aKnob.Tag = 1
                  then ChangeKnobPositionTo( aKnob, aKnob.DefaultPosition);
                end;
              end
            end;
          end

          else if SameText( aModuleType, 'espeakvoice')
          then begin
            with aModule
            do begin
              for i := 0 to ControlCount - 1
              do begin
                if   ( Controls[ i] is TKnobsSelector)
                and  ( Controls[ i].Tag = 1)
                and  SameText( aRangeType, 'voice')
                then FixESpeakLanguageStatics( Controls[ i], nil, anOrigin);
              end;
            end;
          end

          else if SameText( aModuletype, 'rangeconverter' )
          then begin
            with aModule
            do begin
              for i := 0 to ControlCount - 1
              do begin
                if   Controls[ i] is TKnobsKnob
                then begin
                  aKnob := TKnobsKnob( Controls[ i]);

                  case Round( aValue) of
                    0 :  ChangeControlTypeTo( aKnob, 'linear[-4,4]');
                    1 :  ChangeControlTypeTo( aKnob, 'dB[0,4]'     );
                    2 :  ChangeControlTypeTo( aKnob, 'linear[-1,1]');
                    3 :  ChangeControlTypeTo( aKnob, 'dB[0,1]'     );
                    4 :  ChangeControlTypeTo( aKnob, 'linear[0,4]' );
                    5 :  ChangeControlTypeTo( aKnob, 'linear[0,1]' );
                  end;
                end;
              end;
            end;
          end
        end;
      end;
    end;


    procedure   TFormDllMain.ChangeModeFor( const aModule: TKnobsCustomModule; const aModuleType, aModeType: string; aValue: TSignal);
    // This is used when the mode for a control on a module changes, to change the control type
    // of affected controls. Or to change its dynamics ..
    var
      aName     : string;
      aKnob     : TKnobsKnob;
      aSelector : TKnobsSelector;
      aSteps    : Integer;
      i         : Integer;
    begin
      if   Assigned( aModule)
      then begin
        if   SameText( aModuleType, 'scaler')
        then begin
          aKnob := nil;

          with aModule
          do begin
            for i := 0 to ControlCount - 1
            do begin
              if   Controls[ i] is TKnobsKnob
              then begin
                aKnob := TKnobsKnob( Controls[ i]); // Pick the first Knob found ... it should
                Break;                              // be paired in the designer already.
              end;
            end;
          end;

          if   Assigned( aKnob)
          then ChangePairingModeTo( aKnob, TKnobsPairingMode( Round( aValue)));
        end

        else if SameText( aModuleType, 'mixingscaler')
        then begin
          aKnob := nil;

          with aModule
          do begin
            for i := 0 to ControlCount - 1
            do begin
              if   Controls[ i] is TKnobsKnob
              then begin
                aName := TKnobsKnob( Controls[ i]).Name;

                if   ( SameText( aModeType, 'mode' ) and SameText( aName, 'mixingscaler_gain' ))
                or   ( SameText( aModeType, 'mode2') and SameText( aName, 'mixingscaler_gain2'))
                or   ( SameText( aModeType, 'mode3') and SameText( aName, 'mixingscaler_gain3'))
                or   ( SameText( aModeType, 'mode4') and SameText( aName, 'mixingscaler_gain4'))
                then begin
                  aKnob := TKnobsKnob( Controls[ i]);  // Pick the gain Knob only ... it should be paired
                  Break;                               // with the offset knob in the designer already.
                end;
              end;
            end;
          end;

          if   Assigned( aKnob)
          then ChangePairingModeTo( aKnob, TKnobsPairingMode( Round( aValue)));
        end

        else if SameText( aModuleType, 'xyscope')
        then begin
          aKnob := nil;

          with aModule
          do begin
            for i := 0 to ControlCount - 1
            do begin
              if   ( Controls[ i] is TKnobsKnob)
              and  ( TKnobsKnob( Controls[ i]).Name = 'xyscope_levx')
              then begin
                aKnob := TKnobsKnob( Controls[ i]); // Pick the X level Knob ... it should be paired in the designer already with the Y level one.
                Break;
              end;
            end;
          end;

          if   Assigned( aKnob)
          then ChangePairingModeTo( aKnob, TKnobsPairingMode( Round( aValue)));
        end

        else if SameText( aModuleType, 'hardclip')
        then begin
          aKnob := nil;

          with aModule
          do begin
            for i := 0 to ControlCount - 1
            do begin
              if   ( Controls[ i] is TKnobsKnob)
              and  ( TKnobsKnob( Controls[ i]).Name = 'hardclip_lowlevel')
              then begin
                aKnob := TKnobsKnob( Controls[ i]); // Pick the low level Knob ... it should be paired in the designer already with the high level one.
                Break;
              end;
            end;
          end;

          if   Assigned( aKnob)
          then ChangePairingModeTo( aKnob, TKnobsPairingMode( Round( aValue)));
        end

        else if SameText( aModuletype, 'mixm3to1' )
             or SameText( aModuletype, 'mixm4to1' )
             or SameText( aModuletype, 'mixs2to1' )
             or SameText( aModuletype, 'mixs5to1' )
             or SameText( aModuletype, 'mixt1to1' )
             or SameText( aModuletype, 'mixq1to1' )
             or SameText( aModuletype, 'mixm6to6' )
        then begin
          with aModule
          do begin
            for i := 0 to ControlCount - 1
            do begin
              if   Controls[ i] is TKnobsKnob
              then begin
                case  Round( aValue) of
                  0 : ChangeControlTypeTo( TKnobsKnob( Controls[ i]), 'linear[-1,1]');
                  1 : ChangeControlTypeTo( TKnobsKnob( Controls[ i]), 'dB[0,1]'     );
                  2 : ChangeControlTypeTo( TKnobsKnob( Controls[ i]), 'linear[0,1]' );
                  3 : ChangeControlTypeTo( TKnobsKnob( Controls[ i]), 'linear[0,4]' );
                  4 : ChangeControlTypeTo( TKnobsKnob( Controls[ i]), 'linear[-4,4]');
                  5 : ChangeControlTypeTo( TKnobsKnob( Controls[ i]), 'dB[0,4]'     );
                end;
              end;
            end;
          end;
        end

        else if SameText( aModuletype, 'mixm16to1' )
             or SameText( aModuletype, 'mixmm16to1')
             or SameText( aModuletype, 'mixms8to1' )
        then begin
          with aModule
          do begin
            for i := 0 to ControlCount - 1
            do begin
              if   Controls[ i] is TKnobsKnob
              then begin
                aKnob := TKnobsKnob( Controls[ i]);

                case Round( aValue) of
                  0 :  ChangeControlTypeTo( aKnob, 'linear[-4,4]');
                  1 :  ChangeControlTypeTo( aKnob, 'dB[0,4]'     );
                  2 :  ChangeControlTypeTo( aKnob, 'linear[-1,1]');
                  3 :  ChangeControlTypeTo( aKnob, 'dB[0,1]'     );
                  4 :  ChangeControlTypeTo( aKnob, 'linear[0,4]' );
                  5 :  ChangeControlTypeTo( aKnob, 'linear[0,1]' );
                end;
              end;
            end;
          end;
        end

        else if SameText( aModuletype, 'amplifier')
        then begin
          with aModule
          do begin
            for i := 0 to ControlCount - 1
            do begin
              if   Controls[ i] is TKnobsKnob
              then begin
                case  Round( aValue) of
                  0 : ChangeControlTypeTo( TKnobsKnob( Controls[ i]), 'linear[-4,4]');   // 0 : for compatibility with older patches
                  1 : ChangeControlTypeTo( TKnobsKnob( Controls[ i]), 'dB[0,4]'     );   // 1 : for compatibility with older patches
                  2 : ChangeControlTypeTo( TKnobsKnob( Controls[ i]), 'linear[-1,1]');   // 2 : extended : lin -1,1
                  3 : ChangeControlTypeTo( TKnobsKnob( Controls[ i]), 'dB[0,1]'     );   // 3 : extended : dB 0
                  4 : ChangeControlTypeTo( TKnobsKnob( Controls[ i]), 'linear[0,1]' );   // 4 : extended : lin 0,1
                  5 : ChangeControlTypeTo( TKnobsKnob( Controls[ i]), 'linear[0,4]' );   // 5 : extended : lin 0,4
                end;
              end;
            end;
          end;
        end

        else if SameText( aModuletype, 'constant'    )
             or SameText( aModuletype, 'seq16'       )
             or SameText( aModuletype, 'seq16rep'    )
             or SameText( aModuleType, 'vcps'        )
             or SameText( aModuletype, 'lut'         )
             or SameText( aModuletype, 'rewriternote')
             or SameText( aModuletype, 'seqvalues'   )
        then begin
          with aModule
          do begin
            if   SameText( aModuletype, 'constant' )
            or   SameText( aModuletype, 'seqvalues')
            then aSteps := 513
            else if
              SameText( aModuletype, 'lut'      )
            then aSteps := 257
            else aSteps := 128;

            if   SameText( aModuleType, 'seqvalues')
            then begin
              for i := 0 to ControlCount - 1
              do begin
                if   ( Controls[ i].Tag = 1)
                and  ( Controls[ i] is TKnobsSelector)
                then begin
                  aSelector := TKnobsSelector( Controls[ i]);

                  if   Round( aValue) = 0
                  then begin aSelector.Glyphs := FormStore.ImageListDotsGreen ; aSelector.Invalidate; end
                  else begin aSelector.Glyphs := FormStore.ImageListDotsYellow; aSelector.Invalidate; end;
                end;
              end;
            end;

            for i := 0 to ControlCount - 1
            do begin
              if   Controls[ i] is TKnobsKnob
              then begin
                aKnob := TKnobsKnob( Controls[ i]);

                if   aKnob.Tag = 1
                then begin
                  case Round( aValue) of
                    0 :  begin ChangeControlTypeTo( aKnob, 'linear[-4,4]'); ChangeStepCountTo( aKnob, aSteps); end; // 0 : for compatibility with older patches
                    1 :  begin ChangeControlTypeTo( aKnob, 'dB[0,4]'     ); ChangeStepCountTo( aKnob, aSteps); end; // 1 : for compatibility with older patches
                    2 :  begin ChangeControlTypeTo( aKnob, 'linear[-1,1]'); ChangeStepCountTo( aKnob, aSteps); end;
                    3 :  begin ChangeControlTypeTo( aKnob, 'dB[0,1]'     ); ChangeStepCountTo( aKnob, aSteps); end;
                    4 :  begin ChangeControlTypeTo( aKnob, 'linear[0,4]' ); ChangeStepCountTo( aKnob, aSteps); end;
                    5 :  begin ChangeControlTypeTo( aKnob, 'linear[0,1]' ); ChangeStepCountTo( aKnob, aSteps); end;
                    else begin ChangeControlTypeTo( aKnob, 'NoteName'    ); ChangeStepCountTo( aKnob,    128); end;
                  end;
                end;
              end;
            end;
          end;
        end

        else if SameText( aModuletype, 'knobs4' )
        then begin
          with aModule
          do begin
            for i := 0 to ControlCount - 1
            do begin
              if   Controls[ i] is TKnobsKnob
              then begin
                aKnob := TKnobsKnob( Controls[ i]);

                if   (( aKnob.Tag = 1) and SameText( aModeType, 'mode1'))
                or   (( aKnob.Tag = 2) and SameText( aModeType, 'mode2'))
                or   (( aKnob.Tag = 3) and SameText( aModeType, 'mode3'))
                or   (( aKnob.Tag = 4) and SameText( aModeType, 'mode4'))
                then begin
                  case Round( aValue) of
                    0 :  begin ChangeControlTypeTo( aKnob, 'linear[-4,4]'); ChangeStepCountTo( aKnob, 1025); end;
                    1 :  begin ChangeControlTypeTo( aKnob, 'dB[0,4]'     ); ChangeStepCountTo( aKnob, 1025); end;
                    2 :  begin ChangeControlTypeTo( aKnob, 'linear[-1,1]'); ChangeStepCountTo( aKnob, 1025); end;
                    3 :  begin ChangeControlTypeTo( aKnob, 'dB[0,1]'     ); ChangeStepCountTo( aKnob, 1025); end;
                    4 :  begin ChangeControlTypeTo( aKnob, 'linear[0,4]' ); ChangeStepCountTo( aKnob, 1025); end;
                    5 :  begin ChangeControlTypeTo( aKnob, 'linear[0,1]' ); ChangeStepCountTo( aKnob, 1025); end;
                    else begin ChangeControlTypeTo( aKnob, 'NoteName'    ); ChangeStepCountTo( aKnob,  128); end;
                  end;
                end;
              end;
            end;
          end;
        end

        else if SameText( aModuletype, 'seqpattern')
             or SameText( aModuletype, 'seqvalues' )
        then begin
          with aModule
          do begin
            for i := 0 to ControlCount - 1
            do begin
              if   ( Controls[ i].Tag = 1)
              and  ( Controls[ i] is TKnobsSelector)
              then begin
                aSelector := TKnobsSelector( Controls[ i]);

                if   Round( aValue) = 0
                then begin aSelector.Glyphs := FormStore.ImageListDotsGreen ; aSelector.Invalidate; end
                else begin aSelector.Glyphs := FormStore.ImageListDotsYellow; aSelector.Invalidate; end;
              end;
            end;
          end;
        end

        else if SameText( aModuletype, 'quantizer' )
             or SameText( aModuletype, 'seqrandval')
        then begin
          with aModule
          do begin
            for i := 0 to ControlCount - 1
            do begin
              if   ( Controls[ i].Tag = 1)
              and  ( Controls[ i] is TKnobsKnob)
              then begin
                aKnob := TKnobsKnob( Controls[ i]);

                if   Round( aValue) = 0
                then begin ChangeControlTypeTo( aKnob, 'QSteps'   ); aKnob.FriendlyName := 'Notes';      end
                else begin ChangeControlTypeTo( aKnob, 'Fractions'); aKnob.FriendlyName := 'Fractions';  end;
              end;
            end;
          end;
        end

        else if SameText( aModuleType, 'audiograph')
        then begin
          with aModule
          do begin
            for i := 0 to ControlCount - 1
            do begin
              if   ( Controls[ i].Tag = 1)
              and  ( Controls[ i] is TKnobsDataMaker)
              then TKnobsDataMaker( Controls[ i]).AutoScale := Round( aValue) <> 0;
            end;
          end;
        end;

      end;
    end;


    procedure   TFormDllMain.ChangeControlTypeTo( const aControl: TKnobsValuedControl; const aValue: string);
    begin
      aControl.BeginStateChange;

      try
        aControl.ControlType := aValue;
      finally
        aControl.EndStateChange;
      end;
    end;


    procedure   TFormDllMain.ChangeKnobPositionTo( const aControl: TKnobsValuedControl; const aValue: Integer);
    begin
      aControl.BeginStateChange;

      try
        aControl.KnobPosition := aValue;
      finally
        aControl.EndStateChange;
        aControl.AddToHistory;
      end;
    end;


    procedure   TFormDllMain.ChangeStepCountTo( const aControl: TKnobsValuedControl; const aValue: Integer);
    begin
      aControl.BeginStateChange;

      try
        aControl.StepCount := aValue;
      finally
        aControl.EndStateChange;
      end;
    end;


    procedure   TFormDllMain.ChangePairingModeTo( const aKnob: TKnobsKnob; aValue: TKnobsPairingMode);
    begin
      aKnob.BeginStateChange;

      try
        aKnob.PairingMode := aValue;
      finally
        aKnob.EndStateChange;
      end;
    end;


    procedure   TFormDllMain.FixLocalControls;
    var
      i : Integer;
    begin
      for i := 0 to ComponentCount - 1
      do  begin
        if   Components[ i] is TKnobsKnob
        then begin
          TKnobsKnob(            Components[ i]).WheelSupport      := UseWheelOnKnobs;
          TKnobsKnob(            Components[ i]).WheelSensitivity  := WheelSensitivity;
          TKnobsKnob(            Components[ i]).MIDIColor         := KnobMIDIColor;
          TKnobsKnob(            Components[ i]).FocusColor        := KnobFocusColor;
          TKnobsKnob(            Components[ i]).MIDIFocusColor    := KnobMIDIFocusColor;
          if   not ( Components[ i] is TKnobsNoKnob)
          then TKnobsKnob(       Components[ i]).ControlMode       := ControlMode;
        end
        else if Components[ i] is TKnobsSelector
        then begin
          TKnobsSelector(        Components[ i]).Color             := SelectorColor;
          TKnobsSelector(        Components[ i]).BorderColor       := SelectorBorderColor;
          TKnobsSelector(        Components[ i]).BorderColorSingle := SelectorBorderColorS;
        end
        else if Components[ i] is TKnobsDisplay
        then begin
          TKnobsDisplay(         Components[ i]).Color             := DisplayColor;
          TKnobsDisplay(         Components[ i]).BorderColor       := DisplayBorderColor;
        end
        else if Components[ i] is TKnobsDataViewer
        then begin
          TKnobsDataViewer(      Components[ i]).Color             := ViewerColor;
          TKnobsDataViewer(      Components[ i]).BorderColor       := ViewerBorderColor;
          TKnobsDataViewer(      Components[ i]).LineColor         := ViewerLineColor;
          TKnobsDataViewer(      Components[ i]).FillColor         := ViewerFillColor;
        end
        else if Components[ i] is TKnobsPendulumViewer
        then begin
          TKnobsPendulumViewer(  Components[ i]).Color             := ViewerColor;
          TKnobsPendulumViewer(  Components[ i]).BorderColor       := ViewerBorderColor;
        end
        else if Components[ i] is TKnobsMazeGraphViewer
        then begin
          TKnobsMazeGraphViewer( Components[ i]).Color             := ViewerColor;
          TKnobsMazeGraphViewer( Components[ i]).BorderColor       := ViewerBorderColor;
        end
        else if Components[ i] is TKnobsDataMaker
        then begin
          TKnobsDataMaker(       Components[ i]).LineColor         := ViewerLineColor;
        end
        else if Components[ i] is TKnobsIndicatorBar
        then begin
          TKnobsIndicatorBar(    Components[ i]).PeakColor         := IndBarPeakColor;
          TKnobsIndicatorBar(    Components[ i]).ValeyColor        := IndBarValeyColor;
        end
        else if Components[ i] is TEdit
        then begin
          TEdit(                 Components[ i]).Font.Color        := EditFontColor;
          TEdit(                 Components[ i]).Color             := DisplayColor;
          TEdit(                 Components[ i]).TextHintColor     := clSilver;
        end
      end;
    end;


    procedure   TFormDllMain.FixTabColors;
    begin
      if   Assigned( ModuleSelector)
      and  Assigned( FormStore)
      then ModuleSelector.GetTabColors( FormStore.GetTabColor, clSilver, 128, 192);
    end;


    procedure   TFormDllMain.ViewEditor;
    begin
      PageControlMain.ActivePage := TabSheetEditor;
    end;


    procedure   TFormDllMain.ViewDebug;
    begin
      PageControlMain.ActivePage := TabSheetDebug;
    end;


    procedure   TFormDllMain.ViewSettings;
    begin
      PageControlMain.ActivePage := TabSheetSettings;
    end;


    procedure   TFormDllMain.LoadModuleGlyphs;
    begin
      ModuleSelector.ReloadGlyphs ( FLooksFolder);
      knobs2013     .CreateBitmaps( FLooksFolder);
      PatchEditor   .FixBitmaps;
    end;


    procedure   TFormDllMain.UpdateStats;
    begin
      AudioCacheSize := GetAudioCacheSize;

      if   FRequestImport
      then begin
        FRequestImport := False;
        DoImportPatch( FRequestedImportName);
      end;
    end;


    procedure   TFormDllMain.UpdateLights;
    const
      DECAY = 0.001;
    var
      aClocks  : Int64;
      aTime    : TSignal;
      OldEM    : TArithmeticExceptionMask;
      OldMXCSR : UInt32;
    begin
      if   Assigned( FSynthesizer)
      then begin
        OldEM    := SetExceptionMask( exAllArithmeticExceptions);
        OldMXCSR := GetMXCSR;
        SetMXCSR( OldMXCSR or $8040);                                     // Turn on FTZ and DAZ flags to let denormals be zero always

        try
          aClocks        := FSampleCount;
          FUpdateCounter := ( FUpdateCounter + 1) and 3;

          if   LightsActive
          then begin
            case FUpdateCounter of

              0 :
                begin
                  FSynthesizer.GatherLights    ( '', DoLights    );
                end;

              1 :
                begin
                  FSynthesizer.GatherSignals   ( '', DoSignals   );
                  FSynthesizer.GatherData      ( '', DoData      );
                  FSynthesizer.GatherStringData( '', DoStringData);
                end;

              2 :
                begin
                  FSynthesizer.GatherLights    ( '', DoLights    );
                  FSynthesizer.GatherWaveData  ( '', DoWaveData  );
                end;

              3 :
                begin
                  FSynthesizer.GatherXYData    ( '', DoXYData    );
                  FSynthesizer.GatherCursorData( '', DoCursorData);
                end;

            end;
          end
          else begin
            case FUpdateCounter of
             1 : FSynthesizer.GatherSignals   ( '', DoSignals   );
             2 : FSynthesizer.GatherWaveData  ( '', DoWaveData  );
             3 : FSynthesizer.GatherStringData( '', DoStringData);
            end;
          end;

          aTime := (( FSampleCount - aClocks) * System_Rate_Rec) * 1000;

          if   aTime > FLightsTime
          then FLightsTime := aTime
          else FLightsTime := DECAY * ( aTime - FlightsTime) + FLightsTime;
        finally
          ClearFPUExceptions( False);                                     // Clear any FPU exceptions without (re)raising
          ClearSSEExceptions( False);                                     // Likewise for SSE
          SetExceptionMask( OldEM);                                       // Restore FPU exception mask to original value
          SetMXCSR( OldMXCSR);                                            // Restore MXCSR to default mode
        end;
      end;
    end;


    procedure   TFormDllMain.SetTextWriterPrefix( const aValue: string);
    begin
      if   aValue <> FTextWriterPrefix
      then begin
        FTextWriterPrefix         := aValue;
        EditTextWriterPrefix.Text := aValue;
        SetFileTextAgain;
      end;
    end;


    procedure   TFormDllMain.SetPrependPrefix( aValue: Boolean);
    begin
      if   aValue <> FPrependPrefix
      then begin
        FPrependPrefix                := aValue;
        CheckBoxPrependPrefix.Checked := aValue;
        SetFileTextAgain;
      end;
    end;


    procedure   TFormDllMain.SetUseTextWriter( aValue: Boolean);
    begin
      if   aValue <> FUseTextWriter
      then begin
        FUseTextWriter                := aValue;
        CheckBoxUseTextWriter.Checked := aValue;

        if   aValue
        then SetFileTextAgain;
      end;
    end;


    function    TFormDllMain.SubstituteTextMacros( const S: string): string;
    var
      CurrentPatchName : string;
    begin
      CurrentPatchName := ExtractFileName( PatchName);
      Result := S;
      Result := StringReplace( Result, '%patchname%', CurrentPatchName, [ rfReplaceAll, rfIgnoreCase]);

      if   not PrependPrefix
      then Result := StringReplace( Result, '%prefix%'   , TextWriterPrefix, [ rfReplaceAll, rfIgnoreCase]);
    end;


    procedure   TFormDllMain.SetFileText( const aFileName, aLine: string);
    var
      aFile       : TextFile;
      aFolderName : string;
      Contents    : string;
    begin
      if   UseTextWriter
      then begin
        aFolderName := ExtractFilePath( aFileName);

        if   DirectoryExists( aFolderName)
        then begin
          FTextFileName   := aFileName;
          FTextFileString := aLine;
          AssignFile( aFile, aFileName, CP_UTF8);

          try
            Rewrite( aFile);

            try
              if   PrependPrefix
              then Contents := TextWriterPrefix + SubstituteTextMacros( aLine)
              else Contents :=                    SubstituteTextMacros( aLine);

              WriteLn( aFile, Contents);
            finally
              CloseFile( aFile);
            end;
          except
            on E: Exception
            do KilledException( E);
          end;
        end
        else LogFmt( LC_GENERAL, 'could not write FileText to file ''%s'' as the folder ''%s'' does not exist', [ aFileName, aFolderName]);
      end;
    end;


    procedure   TFormDllMain.SetFileTextAgain;
    begin
      if   not FtextFileName  .IsEmpty
      and  not FTextFileString.IsEmpty
      then SetFileText( FTextFileName, FTextFileString);
    end;


    procedure   TFormDllMain.SetNewPatchNameTemplate ( const aValue: string);
    begin
      if   aValue <> FNewPatchNameTemplate
      then begin
        FNewPatchNameTemplate         := aValue;
        EditNewPatchNameTemplate.Text := aValue;
      end;
    end;


    procedure   TFormDllMain.DoSignals( const aSender: TMod; const anInfo: TLightsInfo);
    // A synth signal to be reflected in a module's valued control or in a
    // datamaker (graph).
    var
      aMod : TKnobsCustomModule;
    begin
      aMod := PatchEditor.FindModule( anInfo.Name);         // Find the module in the editor

      if   Assigned( aMod)
      then aMod.SetSignal( anInfo.Signal, anInfo.Data);
    end;


    procedure   TFormDllMain.DoLights( const aSender: TMod; const anInfo: TLightsInfo);
    // A synth signal to be reflected in a module light ... or another control ...
    var
      aMod : TKnobsCustomModule;
    begin
      aMod := PatchEditor.FindModule( anInfo.Name);         // Find the module in the editor

      if   Assigned( aMod)
      then aMod.SetLight( anInfo.Signal, anInfo.Data);
    end;


    procedure   TFormDllMain.DoData( const aSender: TMod; const anInfo: TDataInfo);
    // An array of synth signals to be reflected in a module control in the editor
    var
      aMod : TKnobsCustomModule;
    begin
      aMod := PatchEditor.FindModule( anInfo.Name);         // Find the module in the editor

      if   Assigned( aMod)
      then aMod.SetData( anInfo.Signal, anInfo.Data);
    end;


    procedure   TFormDllMain.DoXYData( const aSender: TMod; const anInfo: TXYDataInfo);
    // A series of X, Y data points to be sent from a synth module to some
    // graphic control in the editor
    var
      aMod : TKnobsCustomModule;
    begin
      aMod := PatchEditor.FindModule( anInfo.Name);        // Find the module in the editor

      if   Assigned( aMod)
      then aMod.SetXYData( anInfo.Signal, anInfo.Data);
    end;


    procedure   TFormDllMain.DoStringData( const aSender: TMod; const anInfo: TStringInfo);
    // A synth string signal to be reflected in a module display or grid in the editor
    // can be a file update request too now
    var
      aMod      : TKnobsCustomModule;
      aFileName : string;
    begin
      if   anInfo.Signal.Contains( FILE_WRITE_COMMAND)
      then begin
        aFileName := anInfo.Signal.Substring( FILE_WRITE_COMMAND);
        SetFileText( aFileName, anInfo.Data);
      end
      else begin
        aMod := PatchEditor.FindModule( anInfo.Name);         // Find the module in the editor

        if   Assigned( aMod)
        then aMod.SetInfo( anInfo.Signal, anInfo.Data);
      end;
    end;


    procedure   TFormDllMain.DoCursorData( const aSender: TMod; const anInfo: TCursorInfo);
    // Cursor tracking from synth patch to a DataMaker or Grid in the editor
    var
      aMod : TKnobsCustomModule;
    begin
      aMod := PatchEditor.FindModule( anInfo.Name);         // Find the module in the editor

      if   Assigned( aMod)
      then aMod.SetCursorData( anInfo.Signal, anInfo.Data);
    end;


    procedure   TFormDllMain.DoWaveData( const aSender: TMod; const anInfo: TWaveInfo);
    // Wave data tracking from synth patch to a DataViewer in the editor
    var
      aMod : TKnobsCustomModule;
    begin
      aMod := PatchEditor.FindModule( anInfo.Name);         // Find the module in the editor

      if   Assigned( aMod)
      then aMod.SetWaveData( anInfo.Signal, anInfo.Data);
    end;


    procedure   TFormDllMain.DoLoadCaptions( const aSender: TObject; const aModule: TKnobsCustomModule; const aControl: TKnobsSelector; const aDependency: TKnobsValuedControl);
    begin
      if   Assigned( aModule)
      and  Assigned( aControl)
      then begin
        if      ( aModule.ModuleType =  335) and ( aControl is TKnobsSelector) and SameText( aControl.Name, 'scalequantizer_scale'    ) then FixScaleQuantizeStatics ( aControl, TKnobsSelector, aDependency)
        else if ( aModule.ModuleType =  510) and ( aControl is TKnobsSelector) and SameText( aControl.Name, 'talkie_bank'             ) then FixTalkieStatics        ( aControl, TKnobsSelector, aDependency)
        else if ( aModule.ModuleType =  528) and ( aControl is TKnobsSelector) and SameText( aControl.Name, 'sapivoice_voice'         ) then FixSapiSpeechTypeStatics( aControl, TKnobsSelector, aDependency)
        else if ( aModule.ModuleType =  528) and ( aControl is TKnobsSelector) and SameText( aControl.Name, 'sapivoice_rate'          ) then FixSpeechRateTypeStatics( aControl, TKnobsSelector, aDependency)
        else if ( aModule.ModuleType =  529) and ( aControl is TKnobsSelector) and SameText( aControl.Name, 'espeakvoice_voice'       ) then FixESpeakVoiceStatics   ( aControl, TKnobsSelector, aDependency)
        else if ( aModule.ModuleType =  529) and ( aControl is TKnobsSelector) and SameText( aControl.Name, 'espeakvoice_language'    ) then FixESpeakLanguageStatics( aControl, TKnobsSelector, aDependency)
        else if ( aModule.ModuleType =  709) and ( aControl is TKnobsSelector) and SameText( aControl.Name, 'rndgranulator_envtype'   ) then FixGranulatorStatics    ( aControl, TKnobsSelector, aDependency)
        else if ( aModule.ModuleType =  710) and ( aControl is TKnobsSelector) and SameText( aControl.Name, 'granulator_envtype'      ) then FixGranulatorStatics    ( aControl, TKnobsSelector, aDependency)
        else if ( aModule.ModuleType =  711) and ( aControl is TKnobsSelector) and SameText( aControl.Name, 'simplegranulator_envtype') then FixGranulatorStatics    ( aControl, TKnobsSelector, aDependency)
        else if ( aModule.ModuleType = 1012) and ( aControl is TKnobsSelector) and SameText( aControl.Name, 'formant2_vowelset'       ) then FixFormant2Statics      ( aControl, TKnobsSelector, aDependency)
        else if ( aModule.ModuleType = 1023) and ( aControl is TKnobsSelector) and SameText( aControl.Name, 'modal2_preset'           ) then FixModal2Statics        ( aControl, TKnobsSelector, aDependency)
        else if ( aModule.ModuleType = 1208) and ( aControl is TKnobsSelector) and SameText( aControl.Name, 'seqrandom_scale'         ) then FixSeqRandomStatics     ( aControl, TKnobsSelector, aDependency)
        else if ( aModule.ModuleType = 1214) and ( aControl is TKnobsSelector) and SameText( aControl.Name, 'probsequencer_scale'     ) then FixProbSequencerStatics ( aControl, TKnobsSelector, aDependency)
        else if ( aModule.ModuleType = 1227) and ( aControl is TKnobsSelector) and SameText( aControl.Name, 'swansong_mazetype'       ) then FixMazeTypeStatics      ( aControl, TKnobsSelector, aDependency);
      end;
    end;

    function    TFormDllMain.CreateSpeechVoiceNames: TStringList;
    var
      aVoice  : TSpVoice;
      aTokens : ISpeechObjectTokens;
      i       : Integer;
    begin
      Result := TStringList.Create;

      try
        try
          aVoice  := TSpVoice.Create( nil);
          aTokens := aVoice.GetVoices( '', '');

          for i := 0 to aTokens.Count - 1
          do Result.Add( aTokens.Item( i).GetDescription( 0));
        except
          Result.Insert( 0, '* Error: SAPI not present? *');
        end;
      finally
        FreeAndNil( aVoice);
      end;
    end;


    function    TFormDllMain.CreateSpeechRateNames: TStringList;
    begin
      Result := TStringList.Create;
      Result.Add( '8 kHz' );
      Result.Add( '11 kHz');
      Result.Add( '12 kHz');
      Result.Add( '16 kHz');
      Result.Add( '22 kHz');
      Result.Add( '24 kHz');
      Result.Add( '32 kHz');
      Result.Add( '44 kHz');
      Result.Add( '48 kHz');
    end;


    procedure   TFormDllMain.FixModuleStatics;
    // This is to fix controls with a dynamic step count. These are set from here with a stringlist for their captions,
    // setting the string list will change the step count. At load time of a patch this information is not available yet
    // so there is an additional clutch, the DynamicStepCount property which should be set to true for the
    // ValuedControls handled here.
    begin
      if   Assigned( PatchEditor)
      then begin
        try
          PatchEditor.VisitControls(  335, TKnobsSelector, 'scalequantizer_scale'    , FixScaleQuantizeStatics , nil);
          PatchEditor.VisitControls(  510, TKnobsSelector, 'talkie_bank'             , FixTalkieStatics        , nil);
          PatchEditor.VisitControls(  528, TKnobsSelector, 'sapivoice_voice'         , FixSapiSpeechTypeStatics, nil);
          PatchEditor.VisitControls(  528, TKnobsSelector, 'sapivoice_rate'          , FixSpeechRateTypeStatics, nil);
          PatchEditor.VisitControls(  529, TKnobsSelector, 'espeakvoice_voice'       , FixESpeakVoiceStatics   , nil);
          PatchEditor.VisitControls(  529, TKnobsSelector, 'espeakvoice_language'    , FixESpeakLanguageStatics, nil);
          PatchEditor.VisitControls(  709, TKnobsSelector, 'rndgranulator_envtype'   , FixGranulatorStatics    , nil);
          PatchEditor.VisitControls(  710, TKnobsSelector, 'granulator_envtype'      , FixGranulatorStatics    , nil);
          PatchEditor.VisitControls(  711, TKnobsSelector, 'simplegranulator_envtype', FixGranulatorStatics    , nil);
          PatchEditor.VisitControls(  719, TKnobsSelector, 'graindelay_grainshape'   , FixGranulatorStatics    , nil);
          PatchEditor.VisitControls( 1012, TKnobsSelector, 'formant2_vowelset'       , FixFormant2Statics      , nil);
          PatchEditor.VisitControls( 1023, TKnobsSelector, 'modal2_preset'           , FixModal2Statics        , nil);
          PatchEditor.VisitControls( 1208, TKnobsSelector, 'seqrandom_scale'         , FixSeqRandomStatics     , nil);
          PatchEditor.VisitControls( 1214, TKnobsSelector, 'probsequencer_scale'     , FixProbSequencerStatics , nil);
          PatchEditor.VisitControls( 1227, TKnobsSelector, 'swansong_mazetype'       , FixMazeTypeStatics      , nil);
        except
          on E: Exception
          do KilledException( E);
        end;
      end;
    end;


    procedure   TFormDllMain.FixDynamicConverter( const aName: string; const aValues: TStringList);
    var
      aConverter : TKnobsDynamicConverter;
    begin
      aConverter := Converters.GetDynamicConverter( aName);

      if   Assigned( aConverter)
      then aConverter.Captions := aValues;
    end;


    function    TFormDllMain.FixTalkieStatics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    var
      aSelector   : TKnobsSelector;
      aPhraseSets : TTalkiePhraseSets;
      aStringList : TStringList;
    begin
      Result      := True;
      aStringList := nil;
      aSelector   := aControl as TKnobsSelector;
      aPhraseSets := TalkiePhraseSets;

      try
        if   Assigned( aPhraseSets)
        then aStringList := aPhraseSets.CreateBankNames;

        if   not Assigned( aStringList)
        then begin
          aStringList := TStringList.Create;
          aStringList.Add( 'words.lpc missing?');
        end;

        if   Assigned( aStringList)
        then begin
          aSelector.Captions := aStringList;
          FixDynamicConverter( 'Dyn Talkie bank', aStringList);
        end;
      finally
        FreeAndNil( aStringList);
      end;
    end;


    function    TFormDllMain.FixGranulatorStatics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    var
      aSelector   : TKnobsSelector;
      aStringList : TStringList;
    begin
      Result    := True;
      aSelector := aControl as TKnobsSelector;

      if   Assigned( aSelector)
      then begin
        aStringList := CreateEnvelopeTypeNames;

        try
          aSelector.Captions := aStringList;
          FixDynamicConverter( 'Dyn Grain envelopes', aStringList);
        finally
          FreeAndNil( aStringList);
        end;
      end;
    end;


    function    TFormDllMain.FixSeqRandomStatics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    var
      aSelector   : TKnobsSelector;
      aStringList : TStringList;
    begin
      Result      := True;
      aSelector   := aControl as TKnobsSelector;
      aStringList := SystemScales.CreateScaleNames;

      try
        if   Assigned( aStringList)
        then begin
          aSelector.Captions := aStringList;
          FixDynamicConverter( 'Dyn Scale', aStringList);
        end;
      finally
        FreeAndNil( aStringList);
      end;
    end;


    function    TFormDllMain.FixScaleQuantizeStatics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    var
      aSelector   : TKnobsSelector;
      aStringList : TStringList;
    begin
      Result      := True;
      aSelector   := aControl as TKnobsSelector;
      aStringList := SystemScales.CreateScaleNames;

      try
        if   Assigned( aStringList)
        then begin
          aSelector.Captions := aStringList;
          FixDynamicConverter( 'Dyn Scale', aStringList);
        end;
      finally
        FreeAndNil( aStringList);
      end;
    end;


    function    TFormDllMain.FixProbSequencerStatics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    var
      aSelector   : TKnobsSelector;
      aStringList : TStringList;
    begin
      Result      := True;
      aSelector   := aControl as TKnobsSelector;
      aStringList := SystemScales.CreateScaleNames;

      try
        if   Assigned( aStringList)
        then begin
          aSelector.Captions := aStringList;
          FixDynamicConverter( 'Dyn Scale', aStringList);
        end;
      finally
        FreeAndNil( aStringList);
      end;
    end;


    function    TFormDllMain.FixFormant2Statics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    var
      aSelector   : TKnobsSelector;
      aStringList : TStringList;
    begin
      Result      := True;
      aSelector   := aControl as TKnobsSelector;
      aStringList := FormantControl.CreateFormantNames;

      try
        if   Assigned( aStringList)
        then begin
          aSelector.Captions := aStringList;
          FixDynamicConverter( 'Dyn Formant2 vowels', aStringList);
        end;
      finally
        FreeAndNil( aStringList);
      end;
    end;


    function    TFormDllMain.FixModal2Statics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    var
      aSelector   : TKnobsSelector;
      aStringList : TStringList;
    begin
      Result      := True;
      aSelector   := aControl as TKnobsSelector;
      aStringList := FormantControl.CreateModalNames;

      try
        if   Assigned( aStringList)
        then begin
          aSelector.Captions := aStringList;
          FixDynamicConverter( 'Dyn Modal2', aStringList);
        end;
      finally
        FreeAndNil( aStringList);
      end;
    end;


    function    TFormDllMain.FixMazeTypeStatics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    var
      aSelector : TKnobsSelector;
    begin
      Result    := True;
      aSelector := aControl as TKnobsSelector;

      if   Assigned( FMazeList)
      then begin
        aSelector.Captions := FMazeList;
        FixDynamicConverter( 'Dyn maze type', FMazeList);
      end;
    end;


    function    TFormDllMain.FixSapiSpeechTypeStatics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    var
      aSelector : TKnobsSelector;
    begin
      Result    := True;
      aSelector := aControl as TKnobsSelector;

      if   Assigned( FSapiVoiceNames)
      then begin
        aSelector.Captions := FSapiVoiceNames;
        FixDynamicConverter( 'Dyn sapi speech voices', FSapiVoiceNames);
      end;
    end;


    function    TFormDllMain.FixSpeechRateTypeStatics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    var
      aSelector : TKnobsSelector;
    begin
      Result    := True;
      aSelector := aControl as TKnobsSelector;

      if   Assigned( FSapiRateNames)
      then begin
        aSelector.Captions := FSapiRateNames;
        FixDynamicConverter( 'Dyn sapi speech rates', FSapiRateNames);
      end;
    end;


    function    TFormDllMain.FixESpeakVoiceStatics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    var
      aSelector : TKnobsSelector;
      aVoices   : TStringList;
    begin
      Result    := True;
      aSelector := aControl as TKnobsSelector;

      if   FESpeakAvailable
      and  Assigned( FESpeaker)
      then begin
        aVoices := FEspeaker.CreateVoiceNames;

        try
          aSelector.Captions := aVoices;
          FixDynamicConverter( 'Dyn eSpeak speech voices', aVoices);
        finally
          aVoices.DisposeOf;
        end;
      end;
    end;


    function    TFormDllMain.FixESpeakLanguageStatics( const aControl: TControl; aControlClass: TControlClass; const aUserData: TObject): Boolean;
    var
      aVoiceSelector    : TKnobsSelector;
      aLanguageSelector : TKnobsSelector;
      aVoiceIndex       : Integer;
      aLanguages        : TStringList;
    begin
      Result := True;

      if   FESpeakAvailable
      and  Assigned( FESpeaker)
      and  ( aUserData is TKnobsSelector)
      then begin
        aLanguageSelector := aControl  as TKnobsSelector;
        aVoiceSelector    := aUserData as TKnobsSelector;
        aVoiceIndex       := aVoiceSelector.KnobPosition;
        aLanguages        := FESpeaker.CreateLanguageNames( aVoiceIndex);

        try
          aLanguageSelector.Captions := aLanguages;
        finally
          aLanguages.DisposeOf;
        end;
      end;
    end;


    function    TFormDllMain.GetFolderPatches: string;
    begin
      Result := FormFolderSelector.FolderPatches;
    end;


    function    TFormDllMain.GetFolderTemplates: string;
    begin
      Result := FormFolderSelector.FolderTemplates;
    end;


    function    TFormDllMain.GetFolderSamplesWaves: string;
    begin
      Result := FormFolderSelector.FolderSamplesWaves;
    end;


    function    TFormDllMain.GetFolderRecordings: string;
    begin
      Result := FormFolderSelector.FolderRecordings;
    end;


    function    TFormDllMain.GetFolderScala: string;
    begin
      Result := FormFolderSelector.FolderScala;
    end;


    function    TFormDllMain.GetFolderMidiFiles: string;
    begin
      Result := FormFolderSelector.FolderMidiFiles;
    end;


    function    TFormDllMain.GetFolderPresets: string;
    begin
      Result := FormFolderSelector.FolderPresets;
    end;


    function    TFormDllMain.GetFolderExportsImports: string;
    begin
      Result := FormFolderSelector.FolderExportsImports;
    end;


    function    TFormDllMain.GetFolderCsvFiles: string;
    begin
      Result := FormFolderSelector.FolderCsvFiles;
    end;


    function    TFormDllMain.GetFolderGraphs: string;
    begin
      Result := FormFolderSelector.FolderGraphs;
    end;


    function    TFormDllMain.GetFolderGrids: string;
    begin
      Result := FormFolderSelector.FolderGrids;
    end;


    function    TFormDllMain.GetFolderGenes: string;
    begin
      Result := FormFolderSelector.FolderGenes;
    end;


    function    TFormDllMain.GetFolderTextWriter: string;
    begin
      Result := FormFolderSelector.FolderTextWriter;
    end;


    procedure   TFormDllMain.DoAudioChanged( const aSender: TObject; const anOldFileName, aNewFileName: string);
    var
      aCache : TAudioCache;
    begin
      aCache := GetAudioCache;

      if   Assigned( aCache)
      then begin
        if   anOldFileName <> aNewFileName
        then begin
          aCache.AddWaveInfo ( aNewFileName );
          aCache.FreeWaveInfo( anOldFileName);
        end;
      end;
    end;


    procedure   TFormDllMain.DoBeginStateChange( const aSender: TObject);
    begin
      if   FStateChangeCount = 0
      then begin
        if   Assigned( FSynthesizer)
        then begin
          FAudioRefList := CreateAudioRefList;
          FSynthesizer.CollectAudioRefs( FAudioRefList);

          if   FAudioRefList.Count > 0
          then IncAudioRefCountsFor( FAudioRefList)
          else FreeAndNil( FAudioRefList);
        end
        else FreeAndNil( FAudioRefList);
      end;

      Inc( FStateChangeCount);
    end;


    procedure   TFormDllMain.DoEndStateChange( const aSender: TObject);
    begin
      Dec( FStateChangeCount);

      if   FStateChangeCount = 0
      then begin
        if   Assigned( FAudioRefList)
        then begin
          if   FAudioRefList.Count > 0
          then DecAudioRefCountsFor( FAudioRefList);

          FreeAndNil( FAudioRefList);
        end;
      end;
    end;


    function    TFormDllMain.Template1Name: string;
    begin
      Result := IncludeTrailingPathDelimiter( FolderTemplates) + 'template1' + EXT_WREN;
    end;


    function    TFormDllMain.Template2Name: string;
    begin
      Result := IncludeTrailingPathDelimiter( FolderTemplates) + 'template2' + EXT_WREN;
    end;


    function    TFormDllMain.TemplateNewName: string;
    begin
      Result := IncludeTrailingPathDelimiter( FolderTemplates) + 'templatenew' + EXT_WREN;
    end;


    function    TFormDllMain.CheckDiscardPatchChanges( const aReason: string; MustAsk: Boolean): Boolean;
//    var
//      MResult : TModalResult;
    begin
//      if   FPatchChanged
//      and  MustAsk
//      then begin
//        MResult := MessageDlg
//        (
//          'Requested operation:' + aReason                                     + ^M^M +
//          '** The current patch has unsaved changes **'                        + ^M^M +
//          'Do you want to save your changes before proceeding?'                + ^M^M +
//          '- Yes - to save and proceeed with the requested operation'          + ^M   +
//          '- No - to not save and proceed with the requested operation anyway' + ^M   +
//          '- Cancel - to not do anything',
//          mtWarning,
//          [ mbYes, mbNo, mbCancel],
//          0,
//          mbYes
//        );
//
//        if   MResult = mrYes
//        then Result := SavePatch( FolderPatches)
//        else if MResult = mrNo
//        then Result := True
//        else Result := False;
//      end
//      else
      Result := True;
    end;


    function    TFormDllMain.LastPatchName: string;
    begin
      Result := IncludeTrailingPathDelimiter( ApplicationPath) + '__l__a__t__e__s__t__' + EXT_WREN;
    end;


    procedure   TFormDllMain.SaveLastPatch;
    var
      aFileName : string;
    begin
      aFileName := LastPatchName;

      try
        LogFmt( LC_GENERAL, 'Saving "%s"', [ aFileName]);

        try
          FPatchWriter.WriteFile( PatchEditor, aFileName);
        except
          on E: Exception
          do LogFmt( LC_GENERAL, 'Failed to write "%s", reason: %s', [ aFileName, E.ToString]);
        end;
      except
        on E: Exception
        do KilledException( E);
      end;
    end;


    procedure   TFormDllMain.LoadLastPatch;
    var
      aFileName : string;
    begin
      aFileName := LastPatchName;

      if   FileExists( aFileName)
      then begin
        try
          DoLoadPatch( aFileName, True);
        except
          on E: Exception
          do KilledException( E);
        end;
      end;
    end;


    function    TFormDllMain.SavePatchNamed( const aFolder, aFileName: string; WriteAlways: Boolean): Boolean;
    begin
      if   FileExists( aFileName) or WriteAlways
      and  Assigned( PatchEditor)
      then begin
        LogFmt( LC_GENERAL, 'Saving "%s"', [ aFileName]);

        try
          FPatchWriter.WriteFile( PatchEditor, aFileName);
        except
          on E: Exception
          do LogFmt( LC_GENERAL, 'Failed to write "%s", reason: %s', [ aFileName, E.ToString]);
        end;

        PatchEditor.SetSavedMarker;
        Result := True;
      end
      else Result := SavePatchAs( aFolder);
    end;


    function    TFormDllMain.SavePatch( const aFolder: string): Boolean;
    begin
      Result := SavePatchNamed( FolderPatches, PatchEditor.Filename, False);
    end;


    function    TFormDllMain.SavePatchAs( const aFolder: string): Boolean;
    begin
      Result := False;

      with SaveDialogPatch
      do begin
        if   FileName = ''
        then Filename := PatchEditor.Filename;

        InitialDir := ExtractFilePath( Filename);
        FileName   := ExtractFileName( Filename);

        if   InitialDir = ''
        then InitialDir := aFolder;

        if   Execute
        then begin
          SourceFileName := FileName;
          PatchName      := CleanFileName( FileName);
          LogFmt( LC_GENERAL, 'Saving patch as "%s"', [ FileName]);

          try
            FPatchWriter.WriteFile( PatchEditor, FileName);
          except
            on E: Exception
            do LogFmt( LC_GENERAL, 'Failed to write "%s", reason: %s', [ FileName, E.ToString]);
          end;

          PatchEditor.SetSavedMarker;
          Result := True;
        end;
      end;

      SetFileTextAgain;
    end;


    function    TFormDllMain.SaveAsTemplate( const aFileName: string): Boolean;
    var
      DoSave  : Boolean;
      OldName : string;
    begin
      if   FileExists( aFileName)
      then DoSave := MessageDlg( Format( '"%s" already exists, overwrite it?', [ aFileName], AppLocale), mtConfirmation, [ mbYes, mbNo], 0) = mrYes
      else DoSave := True;

      if   DoSave
      then begin
        OldName   := PatchName;
        Result    := SavePatchNamed( FolderTemplates, aFileName, True);
        PatchName := OldName;
      end
      else Result := False;
    end;


    function    TFormDllMain.SaveAsTemplate1: Boolean;
    begin
      Result := SaveAsTemplate( Template1Name);
    end;


    function    TFormDllMain.SaveAsTemplate2: Boolean;
    begin
      Result := SaveAsTemplate( Template2Name);
    end;


    function    TFormDllMain.SaveAsTemplateNew: Boolean;
    begin
      Result := SaveAsTemplate( TemplateNewName);
    end;


    procedure   TFormDllMain.ExportModules;
    begin
      with SaveDialogPatch
      do begin
        FileName   := '';
        InitialDir := FolderExportsImports;

        if   Execute
        then begin
          LogFmt( LC_GENERAL, 'Exporting selected modules to "%s"', [ FileName]);

          try
            FPatchWriter.WriteFile( PatchEditor, FileName, wmSelected);
          except
            on E: Exception
            do LogFmt( LC_GENERAL, 'Failed to write "%s", reason: %s', [ FileName, E.ToString]);
          end;
        end;
      end;
    end;


    procedure   TFormDllMain.DoLoadPatch( const aFileName: string; ChangePatchName: Boolean);
    var
      OldName   : string;
      OldSource : string;
      OldGuid   : string;
      OldActive : Integer;
    begin
      if   FileExists( aFileName)
      then begin
        if   CheckDiscardPatchChanges( 'Load a new patch', False)
        then begin
          OldName   := PatchName;
          OldSource := SourceFileName;
          OldGuid   := PatchEditor.Guid;
          OldActive := PatchEditor.ActiveRange;
          PatchEditor.BlockWireUpdates;

          try
            LogFmt( LC_GENERAL, 'loading file %s', [ aFileName]);
            SourceFileName           := aFileName;
            SaveDialogPatch.FileName := '';

            try
              FPatchReader.ReadFile(
                aFileName,
                PatchEditor,
                DO_STRICT,
                NO_DRAG,
                DO_COPY_MIDI,
                False,
                False,
                rmReplace
              );

              if   ChangePatchName
              then PatchName := PatchEditor.Title
              else begin
                PatchName        := OldName;
                SourceFileName   := OldSource;
                PatchEditor.Guid := OldGuid;
              end;

              PatchEditor.ClearUndoRedo;
              PatchEditor.UnSelectAllModules;
              PatchEditor.FixUserSettings;
              PatchEditor.ClearHistory;
              PatchEditor.FixActiveRange( OldActive);
            except
              on E: Exception
              do begin
                LogFmt( LC_GENERAL, 'loading failed with message ''%s''', [ E.ToString]);
                PatchEditor.Undo;
                MessageDlg( Format( 'Patch load failed with message ''%s''', [ E.ToString], AppLocale), mtError, [ mbOK], 0);
              end;
            end;
          finally
            PatchEditor.UnblockWireUpdates;
          end;
        end;
      end;
    end;


    procedure   TFormDllMain.DoImportPatch( const aFileName: string);
    // modules need be in drag mode after paste ... and MIDI CCs should not be copied
    begin
      if   FileExists( aFileName)
      and  Assigned( FPatchReader)
      then begin
        PatchEditor.BlockWireUpdates;

        try
          LogFmt( LC_GENERAL, 'importing file %s', [ aFileName]);
          try
            FPatchReader.ImportFile( aFileName, PatchEditor, DO_STRICT, DO_DRAG, NO_COPY_MIDI, NO_COPY_TUNING, NO_COPY_RATE, rmAppend);
          except
            on E: Exception
            do begin
              LogFmt( LC_GENERAL, 'importing failed with message ''%s''', [ E.ToString]);
              PatchEditor.Undo;
              MessageDlg( Format( 'Patch import fialed with message ''%s''', [ E.ToString], AppLocale), mtError, [ mbOK], 0);
            end;
          end;
        finally
          PatchEditor.UnblockWireUpdates;
        end;
      end;
    end;


    procedure   TFormDllMain.LoadPatch;
    begin
      with OpenDialogPatch
      do begin
        if   FileName = ''
        then Filename := PatchEditor.Filename;

        InitialDir := FolderPAtches;
        FileName   := ExtractFileName( Filename);

        if   Execute
        then DoLoadPatch( FileName, True);
      end;
    end;


    procedure   TFormDllMain.ImportPatch;
    begin
      with OpenDialogPatch
      do begin
        InitialDir := FolderExportsImports;

        if   Execute
        then begin
          FRequestedImportName := FileName;
          FRequestImport       := True;
        end;
      end;
    end;


    procedure   TFormDllMain.NewPatch;
    begin
      if   Assigned( PatchEditor)
      then begin
        PatchEditor.FreeModules( DoOnValuedControlRemoved);
        PatchEditor  .Guid  := '';     // Force creation of a new GUID
        SourceFileName      := UniqueFilenameFromTemplate( FolderPatches, NewPatchNameTemplate, EXT_WREN, PatchEditor.Guid);
        PatchEditor  .Title := Trim( UnQuote( ChangeFileExt( ExtractFileName( SourceFileName), ''), '', '.'));
        PatchName           := PatchEditor.Title;
        DoLoadPatch( TemplateNewName, False);
      end
      else begin
        SourceFileName := 'new';
        PatchName      := SourceFileName;
      end;
    end;


    procedure   TFormDllMain.LoadPatchPreset;
    begin
      try
        PatchEditor.SelectAllModules;

        with OpenDialogPreset
        do begin
          FileName   := ChangeFileExt( ExtractFileName( PatchEditor.Filename), EXT_PRESET);
          InitialDir := FolderPresets;

          if   Execute
          then begin
            FPatchReader.WarnOnStructMismatch := True;
            FPatchReader.ParamsFromFile( FileName, PatchEditor, DO_STRICT);
          end;
        end;
      finally
        PatchEditor.UnSelectAllModules;
      end;
    end;


    procedure   TFormDllMain.SavePatchPreset;
    begin
      try
        PatchEditor.SelectAllModules;

        with SaveDialogPreset
        do begin
          FileName   := ChangeFileExt( ExtractFileName( PatchEditor.Filename), EXT_PRESET);
          InitialDir := FolderPresets;

          if   Execute
          then begin
            try
              FPatchWriter.WriteFile( PatchEditor, FileName, wmParams);
            except
              on E: Exception
              do LogFmt( LC_GENERAL, 'Failed to write "%s", reason: %s', [ FileName, E.ToString]);
            end;
          end;
        end;
      finally
        PatchEditor.UnSelectAllModules;
      end;
    end;


    procedure   TFormDllMain.DoOnValuedControlRemoved( const aSender: TKnobsValuedControl);
    begin
      if   Assigned( FormLastEdits)
      then FormLastEdits.RemoveValuedControl( aSender);
    end;


    procedure   TFormDllMain.DeleteModules;
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.FreeSelectedModules( DoOnValuedControlRemoved);
    end;


    procedure   TFormDllMain.CutModules;
    begin
      CopyModules;
      DeleteModules;
    end;


    procedure   TFormDllMain.CopyModules;
    begin
      if   Assigned( FPatchWriter)
      then FPatchWriter.WriteToClipboard( PatchEditor, wmSelected);
    end;


    procedure   TFormDllMain.PasteModules;
    begin

      // Note : PasteModules (ReadFromClipboard) should not alter Midi CC assignments.

      if   Assigned( FPatchReader)
      then FPatchReader.ReadFromClipboard( PatchEditor, DO_STRICT, DO_DRAG, NO_COPY_MIDI, NO_COPY_TUNING, NO_COPY_RATE, rmAppend);
    end;


    procedure   TFormDllMain.PasteParameters( StrictCopy: Boolean);
    begin

      // Note : PasteParameters (ParamsFromClipboard) should not alter Midi CC assignments.

      if   Assigned( FPatchReader)
      then begin
        FPatchReader.WarnOnStructMismatch := True;
        FPatchReader.ParamsFromClipboard( PatchEditor, StrictCopy);
      end;
    end;


    procedure   TFormDllMain.SelectAll;
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.SelectAllModules;
    end;


    procedure   TFormDllMain.SelectNone;
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.UnselectAllModules;
    end;


    procedure   TFormDllMain.InvertSelection;
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.InvertModuleSelection;
    end;


    procedure   TFormDllMain.WiggleWires;
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.WiggleWires;
    end;


    procedure   TFormDllMain.ToggleWires;
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.ToggleWires;
    end;


    function    TFormDllMain.WiresOff: Boolean;
    begin
      if   Assigned( PatchEditor)
      then Result := PatchEditor.WiresOff
      else Result := True;
    end;


    procedure   TFormDllMain.WiresOn( TurnOn: Boolean);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.WiresOn( TurnOn);
    end;


    procedure   TFormDllMain.Panic;
    begin
      if   Assigned( FSynthesizer)
      then FSynthesizer.Panic;
    end;


    procedure   TFormDllMain.FindUnconnectedModules;
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.FindUnconnectedModules;
    end;


    function    TFormDllMain.GetDefaultModuleColor( const aSender: TObject; const aModuleType: TKnobsModuleType): TColor;
    begin
      if   Assigned( FormStore)
      then Result := FormStore.DefaultModuleColor( aModuleType, ModuleColor)
      else Result := ModuleColor;
    end;


    procedure   TFormDllMain.ConnectorDelete( const aConnector: TKnobsConnector);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.ConnectorDelete( aConnector);
    end;


    procedure   TFormDllMain.ConnectorBreak( const aConnector: TKnobsConnector);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.ConnectorBreak( aConnector);
    end;


    procedure   TFormDllMain.ConnectorDisconnect( const aConnector: TKnobsConnector);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.ConnectorDisconnect( aConnector);
    end;


    procedure   TFormDllMain.SetWireColor( const aConnector : TKnobsConnector; const aColor: TColor);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.SetCustomWireColor( aConnector, aColor);
    end;


    procedure   TFormDllMain.SetWireColorDefault( const aConnector: TKnobsConnector);
    begin
      if   Assigned( PatchEditor)
      then PatchEditor.SetWireColorDefault( aConnector);
    end;


    procedure   TFormDllMain.SetWireColorCustom( const aConnector: TKnobsConnector);
    begin
      if   Assigned( PatchEditor)
      and  Assigned( aConnector)
      then begin
        with ColorDialogModule
        do begin
          Color := aConnector.WireColor;

          if   Execute
          then PatchEditor.SetCustomWireColor( aConnector, Color);
        end;
      end;
    end;


    procedure   TFormDllMain.ModuleCut( const aModule: TKnobsModule);
    begin
      if   Assigned( PatchEditor)
      and  Assigned( aModule)
      then begin
        aModule.FixSelection;
        CutModules;
      end;
    end;


    procedure   TFormDllMain.ModuleCopy( const aModule: TKnobsModule);
    begin
      if   Assigned( PatchEditor)
      and  Assigned( aModule)
      then begin
        aModule.FixSelection;
        CopyModules;
      end;
    end;


    procedure   TFormDllMain.ModuleDelete( const aModule: TKnobsModule);
    begin
      if   Assigned( PatchEditor)
      and  Assigned( aModule)
      then begin
        aModule.FixSelection;
        DeleteModules;
      end;
    end;


    procedure   TFormDllMain.ModulePasteParams( const aModule: TKnobsModule; StrictCopy: Boolean);
    begin
      if   Assigned( PatchEditor)
      and  Assigned( aModule)
      then PasteParameters( StrictCopy);
    end;


    procedure   TFormDllMain.ChangeModuleColor( const aModule: TKnobsModule);
    begin
      if   Assigned( PatchEditor)
      and  Assigned( aModule)
      then begin
        aModule.FixSelection;

        with ColorDialogModule
        do begin
          Color := aModule.Color;

          if   Execute
          then PatchEditor.SetSelectedModuleColor( Color);
        end;
      end;
    end;


    procedure   TFormDllMain.ModuleColorDefault ( const aModule: TKnobsModule);
    begin
      if   Assigned( PatchEditor)
      and  Assigned( FormStore  )
      and  Assigned( aModule    )
      then begin
        aModule.FixSelection;
        PatchEditor.SetSelectedModuleColorDefault( GetDefaultModuleColor);
      end;
    end;


    procedure   TFormDllMain.KnobSetDefaultValue( const aValuedControl: TKnobsValuedControl);
    begin
      if   Assigned( aValuedControl)
      then aValuedControl.SetDefault;
    end;


    procedure   TFormDllMain.KnobChangeLock( const aKnob: TKnobsKnob);
    begin
      if   Assigned( aKnob)
      then aKnob.Locked := not aKnob.Locked;
    end;


    procedure   TFormDllMain.LoadPreset( aModule: TKnobsCustomModule);
    begin
      if   Assigned( aModule)
      then begin
        with OpenDialogPreset
        do begin
          if   aModule.TemplateName <> ''
          then begin
            FileName   := ChangeFileExt( ExtractFileName( aModule.TemplateName), EXT_PRESET);
            InitialDir := ExtractFilePath( aModule.TemplateName);
          end
          else begin
            FileName   := ChangeFileExt( aModule.Title, EXT_PRESET);
            InitialDir := FolderPresets;
          end;

          if   Execute
          then begin
            aModule.FixSelection;
            aModule.TemplateName := FileName;
            FPatchReader.WarnOnStructMismatch := True;
            FPatchReader.ParamsFromFile( FileName, PatchEditor, DO_STRICT);
          end;
        end;
      end;
    end;


    procedure   TFormDllMain.SavePreset( aModule: TKnobsCustomModule);
    begin
      if   Assigned( aModule)
      then begin
        with SaveDialogPreset
        do begin
          if   aModule.TemplateName <> ''
          then begin
            FileName   := ChangeFileExt( ExtractFileName( aModule.TemplateName), EXT_PRESET);
            InitialDir := ExtractFilePath( aModule.TemplateName);
          end
          else begin
            FileName   := ChangeFileExt( aModule.Title, EXT_PRESET);
            InitialDir := FolderPresets;
          end;

          if   Execute
          then begin
            aModule.FixSelection;
            aModule.TemplateName := FileName;

            try
              FPatchWriter.WriteFile( PatchEditor, FileName, wmParams);
            except
              on E: Exception
              do LogFmt( LC_GENERAL, 'Failed to write "%s", reason: %s', [ FileName, E.ToString]);
            end;
          end;
        end;
      end;
    end;


    procedure   TFormDllMain.FillDataMaker( const aDataMaker: TKnobsDataMaker; aShape: TKnobsDMShape);
    begin
      if   Assigned( aDataMaker)
      then aDataMaker.MakeWave( aShape);
    end;


//  public


// Delphi area


{$R *.dfm}

procedure TFormDllMain.selectnone1Click(Sender: TObject);
begin
  SelectNone;
end;

procedure TFormDllMain.invertselection1Click(Sender: TObject);
begin
  InvertSelection;
end;

procedure TFormDllMain.selectall1Click(Sender: TObject);
begin
  SelectAll;
end;

procedure TFormDllMain.pastevalues1Click(Sender: TObject);
begin
  PasteParameters( True);
end;

procedure TFormDllMain.paste1Click(Sender: TObject);
begin
  PasteModules;
end;

procedure TFormDllMain.copy1Click(Sender: TObject);
begin
  CopyModules;
end;

procedure TFormDllMain.LockKnobClick(Sender: TObject);
begin
  KnobChangeLock( FPopupSender as TKnobsKnob);
end;

procedure TFormDllMain.SetKnobDefaultvalueClick(Sender: TObject);
begin
  KnobSetDefaultValue( FPopupSender as TKnobsValuedcontrol)
end;

procedure TFormDllMain.Customwirecolor1Click(Sender: TObject);
begin
  SetWireColorCustom( FPopupSender as TKnobsConnector);
end;

procedure TFormDllMain.cut1Click(Sender: TObject);
begin
  CutModules;
end;

procedure TFormDllMain.Makegreen1Click(Sender: TObject);
begin
  SetWireColor( FPopupSender as TKnobsConnector, clLime);
end;

procedure TFormDllMain.MenuItemModuleColorClick(Sender: TObject);
begin
  ChangeModuleColor( FPopupSender as TKnobsModule);
end;

procedure TFormDllMain.MenuItemModuleCopyClick(Sender: TObject);
begin
  ModuleCopy( FPopupSender as TKnobsModule);
end;

procedure TFormDllMain.MenuItemModuleCutClick(Sender: TObject);
begin
  ModuleCut( FPopupSender as TKnobsModule);
end;

procedure TFormDllMain.MenuItemModuleDefaultcolorClick(Sender: TObject);
begin
  ModuleColorDefault( FPopupSender as TKnobsModule);
end;

procedure TFormDllMain.MenuItemModuleDeleteClick(Sender: TObject);
begin
  ModuleDelete( FPopupSender as TKnobsModule);
end;

procedure TFormDllMain.MenuItemModuleHelpClick(Sender: TObject);
begin
  ShowModuleHelp( FPopupSender as TKnobsModule);
end;

procedure TFormDllMain.MenuItemModuleLoadFromPresetClick(Sender: TObject);
begin
  if   FPopupSender is TKnobsCustomModule
  then LoadPreset( FPopupSender as TKnobsCustomModule);
end;

procedure TFormDllMain.MenuItemModulePasteParamsStrictClick(Sender: TObject);
begin
  ModulePasteParams( FPopupSender as TKnobsModule, True);
end;

procedure TFormDllMain.MenuItemModuleSaveAsPresetClick(Sender: TObject);
begin
  if   FPopupSender is TKnobsCustomModule
  then SavePreset( FPopupSender as TKnobsCustomModule);
end;

procedure TFormDllMain.Yellowwires1Click(Sender: TObject);
begin
  SetWireColor( FPopupSender as TKnobsConnector, clYellow);
end;

procedure TFormDllMain.DisconnectConnectorClick(Sender: TObject);
begin
  ConnectorDisconnect( FPopupSender as TKnobsConnector);
end;

procedure TFormDllMain.Defaultwirecolor1Click(Sender: TObject);
begin
  SetWireColorDefault( FPopupSender as TKnobsConnector);
end;

procedure TFormDllMain.delete1Click(Sender: TObject);
begin
  DeleteModules;
end;

procedure TFormDllMain.Delete2Click(Sender: TObject);
begin
  DeleteModules;
end;

procedure TFormDllMain.DeleteConnectorClick(Sender: TObject);
begin
  ConnectorDelete( FPopupSender as TKnobsConnector);
end;

procedure TFormDllMain.PopupMenuActionPanicClick(Sender: TObject);
begin
  Panic;
end;

procedure TFormDllMain.PopupMenuActionRecompileClick(Sender: TObject);
begin
  CompilePatch( False, False);
end;

procedure TFormDllMain.PopupMenuActionResetClick(Sender: TObject);
begin
  if   Assigned( FSynthesizer)
  then FSynthesizer.Reset;
end;

procedure TFormDllMain.PopupMenuActionTogglelightsClick(Sender: TObject);
begin
  LightsActive := not LightsActive;
end;

procedure TFormDllMain.PopupMenuActionTogglewiresClick(Sender: TObject);
begin
  ToggleWires;
end;

procedure TFormDllMain.PopupMenuActionWigglewiresClick(Sender: TObject);
begin
  WiggleWires;
end;

procedure TFormDllMain.PopupMenuDataMakerClick(Sender: TObject);
begin
  FillDataMaker( FPopupSender as TKnobsDataMaker, TKnobsDMShape(( Sender as TMenuItem).Tag));
end;

procedure TFormDllMain.PopupMenuEditCopyClick(Sender: TObject);
begin
  CopyModules;
end;

procedure TFormDllMain.PopupMenuEditCutClick(Sender: TObject);
begin
  CutModules;
end;

procedure TFormDllMain.PopupMenuEditFindmodulesClick(Sender: TObject);
begin
  if   EditSearch.Enabled
  then EditSearch.SetFocus;
end;

procedure TFormDllMain.PopupMenuEditFindunconnectedmodulesClick(Sender: TObject);
begin
  FindUnconnectedModules;
end;

procedure TFormDllMain.PopupMenuEditInvertselectionClick(Sender: TObject);
begin
  InvertSelection;
end;

procedure TFormDllMain.PopupMenuEditPasteClick(Sender: TObject);
begin
  PasteModules;
end;

procedure TFormDllMain.PopupMenuEditPastevaluesClick(Sender: TObject);
begin
  PasteParameters( True);
end;

procedure TFormDllMain.PopupMenuEditRedoClick(Sender: TObject);
begin
  if   Assigned( PatchEditor)
  then begin
    PatchEditor.Redo;
    //RegisterMidiControls;
    //RegisterRandomParamsAll;
  end;
end;

procedure TFormDllMain.PopupMenuEditSelectallClick(Sender: TObject);
begin
  SelectAll;
end;

procedure TFormDllMain.PopupMenuEditSelectnoneClick(Sender: TObject);
begin
  SelectNone;
end;

procedure TFormDllMain.PopupMenuEditUndoClick(Sender: TObject);
begin
  if   Assigned( PatchEditor)
  then begin
    PatchEditor.Undo;
    //RegisterMidiControls;
    //RegisterRandomParamsAll;
  end;
end;

procedure TFormDllMain.PopupMEnuFileExportselectedmodulesClick(Sender: TObject);
begin
  ExportModules;
end;

procedure TFormDllMain.PopupMenuFileImportmodulesClick(Sender: TObject);
begin
  ImportPatch;
end;

procedure TFormDllMain.PopupMenuFileLoadlatestpatchClick(Sender: TObject);
begin
  LoadLastPatch;
end;

procedure TFormDllMain.PopupMEnuFileLoadpatchpresetClick(Sender: TObject);
begin
  LoadPatchPreset;
end;

procedure TFormDllMain.PopupMenuFileLoadpatchtemplate1Click(Sender: TObject);
begin
  DoLoadPatch( Template1Name, False);
end;

procedure TFormDllMain.PopupMenuFileLoadpatchtemplate2Click(Sender: TObject);
begin
  DoLoadPatch( Template2Name, False);
end;

procedure TFormDllMain.PopupMenuFileNewClick(Sender: TObject);
begin
  NewPatch;
end;

procedure TFormDllMain.PopupMenuFileOpenPatchClick(Sender: TObject);
begin
  LoadPatch;
end;

procedure TFormDllMain.PopupMenuFileSavepatchasClick(Sender: TObject);
begin
  SavePatchAs( FolderPatches);
end;

procedure TFormDllMain.PopupMenuFileSavepatchastemplate1Click(Sender: TObject);
begin
  SaveAsTemplate1;
end;

procedure TFormDllMain.PopupMenuFileSavepatchastemplate2Click(Sender: TObject);
begin
  SaveAsTemplate2;
end;

procedure TFormDllMain.PopupMEnuFileSavepatchastemplatenewClick(Sender: TObject);
begin
  SaveAsTemplateNew;
end;

procedure TFormDllMain.PopupMenuFileSavepatchClick(Sender: TObject);
begin
  SavePatch( FolderPatches);
end;

procedure TFormDllMain.PopupMEnuFileSavepatchpresetClick(Sender: TObject);
begin
  SavePatchPreset;
end;

procedure TFormDllMain.KnobsNoKnob_HistoryCountValueChanged(const aSender: TObject; const aPath, aControlType: string;
  aValue: Double; IsFinal, IsAutomation: Boolean);
begin
  HistoryCount := Round( aValue);
end;

procedure TFormDllMain.KnobsSmallKnob_LightsRateValueChanged(const aSender: TObject; const aPath, aControlType: string;
  aValue: Double; IsFinal, IsAutomation: Boolean);
begin
  if   IsFinal
  then LightsRate := aValue;
end;

procedure TFormDllMain.BitBtnCheckForUpdatesClick(Sender: TObject);
begin
  CheckForUpdates;
end;

procedure TFormDllMain.BitBtnFolderSetupClick(Sender: TObject);
begin
  SelectFolders;
end;

procedure TFormDllMain.BitBtnAcknowledgementsClick(Sender: TObject);
begin
  ShowAcknowledgements;
end;

procedure TFormDllMain.BitBtnDocumentClick(Sender: TObject);
begin
  MakeDocs( DocsPath);    // Forcibly make docs.
  ShowHelp;
end;

procedure TFormDllMain.BitBtnShowActionMenuClick(Sender: TObject);
begin
  ShowActionMenu( Sender as TControl);
end;

procedure TFormDllMain.BitBtnShowDebugClick(Sender: TObject);
begin
  ViewDebug;
end;

procedure TFormDllMain.BitBtnShowEditMenuClick(Sender: TObject);
begin
  ShowEditMenu( Sender as TControl);
end;

procedure TFormDllMain.BitBtnShowEditorClick(Sender: TObject);
begin
  ViewEditor;
end;

procedure TFormDllMain.BitBtnShowFileMenuClick(Sender: TObject);
begin
  ShowFileMenu( Sender as TControl);
end;

procedure TFormDllMain.BitBtnShowHelpClick(Sender: TObject);
begin
  ShowHelp;
end;

procedure TFormDllMain.BitBtnShowSettingsClick(Sender: TObject);
begin
  ViewSettings;
end;

procedure TFormDllMain.Bluewires1Click(Sender: TObject);
begin
  SetWireColor( FPopupSender as TKnobsConnector, clBlue);
end;

procedure TFormDllMain.BreakWireClick(Sender: TObject);
begin
  ConnectorBreak( FPopupSender as TKnobsConnector);
end;

procedure TFormDllMain.TimerLightsUpdateTimer(Sender: TObject);
begin
  UpdateLights;
end;

procedure TFormDllMain.TimerLiveMorphVisualsTimer(Sender: TObject);
begin
  // UpdateLiveMorphVisuals;
end;

procedure TFormDllMain.TimerStatsUpdateTimer(Sender: TObject);
begin
  UpdateStats;
end;

procedure TFormDllMain.UpDownPolyphonyClick(Sender: TObject; Button: TUDBtnType);
begin
  if   Button = btNext
  then Polyphony := Polyphony + 1
  else Polyphony := Polyphony - 1;
end;

procedure TFormDllMain.UpDownWireThickness1Click(Sender: TObject; Button: TUDBtnType);
begin
  if   Button = btNext
  then WireThickness := WireThickness + 1
  else WireThickness := WireThickness - 1;
end;

procedure TFormDllMain.knob_iolevel_UnFocus(const aSender: TObject);
begin
  DoUnFocus( aSender);
end;

procedure TFormDllMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Log( LC_TERMINATE, 'TFormWren.FormClose');
  FClosing := True;

  try
    if   assigned( FSynthesizer)
    then FSynthesizer.Locked := True;
  except
    on E: Exception
    do KilledException( E);
  end;

  if   FIniLoaded
  then SaveIni( IniFileName, False);

  FIniSaved := True;
end;

procedure TFormDllMain.FormCreate(Sender: TObject);

  function RootKeyName( aRootKey: HKEY): string;
  const
    KeyNames: array[ HKEY_CLASSES_ROOT .. HKEY_DYN_DATA] of string = (
      'HKEY_CLASSES_ROOT', 'HKEY_CURRENT_USER', 'HKEY_LOCAL_MACHINE',
      'HKEY_USERS', 'HKEY_PERFORMANCE_DATA', 'HKEY_CURRENT_CONFIG', 'HKEY_DYN_DATA');
  begin
    if   ( aRootKey >= HKEY_CLASSES_ROOT)
    and  ( aRootKey <= HKEY_DYN_DATA    )
    then Result := KeyNames[ aRootKey]
    else Result := '';
  end;

var
  anErrVal            : Integer;
  i                   : Integer;
  aLooksFolderName    : string;
  aDefaultsFolderName : string;
  aPhraseSets         : TTalkiePhraseSets;
  ESpeakVersion       : string;
  Paths               : TStringList;
  S                   : string;
begin
  FLogToFile      := True;
  GOnLoadCaptions := DoLoadCaptions;               // Set a global caption loader for dynamic count selectors.
  CanLog          := False;                        // Causes logs to be stored on FSavedLogs

  if   FileExists( LogFileName)                    // Delete existing logs
  then DeleteFile( LogFileName);

  LogFmt( LC_GENERAL, 'For logs also see the log file: %s', [ LogFileName]);
  Log   ( LC_STARTUP, 'Creating main form');
  LogFmt( LC_STARTUP, 'Wren version  : %s', [ GetFileVersion   ]);
  LogFmt( LC_STARTUP, 'Patch version : %d', [ KnobsPatchVersion]);

  Log   ( LC_STARTUP, 'Creating folder selection form');
  FormFolderSelector := TFormFolderSelector.Create( Application);
  Log   ( LC_STARTUP, 'Created  folder selection form');

  Log   ( LC_STARTUP, '**  Note: all DLLs are being searched for in the following folders:');
  Paths := Explode( System.SysUtils.GetEnvironmentVariable( 'path'), ';');
  try
    Paths.Insert( 0, ApplicationPath);

    for i := 0 to Paths.Count - 1
    do begin
      S := Trim( Paths[ i]);

      if   S <> ''
      then LogFmt( LC_STARTUP, '    %s', [ S]);
    end;
  finally
    FreeAndNil( Paths);
  end;

  GOnModWarning := DoModuleWarning;
  FRegErrors    := False;
  RegisterModuleTypes;
  FOrigCaption := Caption;

  FFFTWAvailable := False;
  try
    LogFmt( LC_STARTUP, 'probing FFTW DLL presence, looking for "%s"', [ fftw_dllpath]);
    anErrVal := fftw_init_threads;

    if   anErrVal <> 0
    then begin
      fftw_plan_with_nthreads( 1);                         // Set maximum number of threads for FFTW to use
      Log( LC_STARTUP, 'FFTW DLL found (it has no version information)');
      FFFTWAvailable := True;
    end
    else Log( LC_STARTUP, 'FFTW DLL found, but no threads avaliable, some modules may not work');
  except
    on E: Exception
    do begin
      LogFmt( LC_STARTUP, 'FFTW DLL not found, or an error occurred calling into it (%s), some modules will not work', [ E.ToString]);
      KilledException( E);
    end;
  end;

  FESpeakAvailable := False;

  if   ESpeakIsPresent  // Check registry for Espeak installation
  then begin
    Log( LC_STARTUP, 'Looks like an ESpeak installation was found in the registry');
    LogFmt( LC_STARTUP, 'probing eSpeak DLL presence, looking for "%s" (using API revision %d)', [ ESpeakDllName, ESpeakApiVersion]);
    FESpeaker := nil;
    try
      FESpeaker        := TESpeakSpeaker.Create;
      ESpeakVersion    := FESpeaker.ESpeakVersion;
      FESpeakAvailable := True;
    except
      on E: Exception
      do begin
        LogFmt( LC_STARTUP, 'eSpeak DLL not found, or an error occurred calling into it (%s]), some modules may not work', [ E.ToString]);
        KilledException( E);
      end;
    end;
  end
  else LogFmt( LC_STARTUP, 'ESpeak installation not found in registry (%s%s)', [ RootKeyName( ESPEAK_KEY_ROOT), ESPEAK_KEY_NAME]);

  if FESpeakAvailable
  then LogFmt( LC_STARTUP, 'eSpeak DLL found, version "%s', [ ESpeakVersion]);

  Log( LC_STARTUP, 'reading Talkie phrase sets from ''words.lpc''');
  TalkieCreatePhrases( ApplicationPath + 'words.lpc');
  aPhraseSets := TalkiePhraseSets;

  if   not Assigned( aPhraseSets)
  then Log( LC_STARTUP, '''words.lpc'' not found or erroneous, some modules will not work')
  else Log( LC_STARTUP, 'phrase sets loaded');

  Log( LC_STARTUP, 'reading Formant sets sets from ''wren.formants''');
  try
    ReadFormantCollection( ApplicationPath + 'wren.formants', FormantLogger);
  except
    on E: Exception
    do begin
      KilledException( E);
      LogFmt( LC_STARTUP, 'Error reading wren.formants (%s), some modules will not work', [ E.Message]);
    end;
  end;

  Log( LC_STARTUP, 'formants rules were checked, now reading modal sets from ''wren.modal''');
  try
    ReadModalCollection( ApplicationPath + 'wren.modal', FormantLogger);
  except
    on E: Exception
    do begin
      KilledException( E);
      LogFmt( LC_STARTUP, 'Error reading wren.modal (%s), some modules will not work', [ E.Message]);
    end;
  end;

  Log( LC_STARTUP, 'modal rules were checked, some fiddling ...');
  System.Randomize;                                        // Randomize the random generator
  GOnCreateModuleBitmap := DoCreateModuleBitmap;           // Helpers for help system and for module selector
  GOnReadModuleComment  := DoReadModuleComment;
  GOnLookupModuleClass  := LookupModuleGeneralClass;
  GOnPopupEditorShow    := DoPopupEditorShow;
  GOnPopupEditorHide    := DoPopupEditorHide;

  with Application
  do begin
    HintPause     :=    0;
    HintHidePause := 5000;
    OnShowHint    := DoShowHint;
  end;

  Log( LC_STARTUP, 'fiddling done, creating SAPI speech voice names');
  try
    FSapiVoiceNames := CreateSpeechVoiceNames;
    FSapiRateNames  := CreateSpeechRateNames;
    Log( LC_STARTUP, 'created SAPI speech voice names');
  except
    on E: Exception
    do LogFmt( LC_STARTUP, 'Error creating SAPI speech voice names: %s, you may try to install the windows speech API', [ E.Message]);
  end;

  Log( LC_STARTUP, 'Creating designer form');
  FormLogger := DoFormDesignerLog;
  FormStore  := TFormStore.Create( Application);
  Log( LC_STARTUP, 'Created designer form, creating last edits form and folde selector');
  FormLastEdits := TFormLastEdits.Create( Application);

  if   Assigned( FormLastEdits)
  then FormLastEdits.Initialize( PatchEditor);

  if   Assigned( FormStore)
  then FMazeList := FormStore.CreateMazeTypes;

  Log( LC_STARTUP, 'created last edits form, creating randomizer form');
  FormRandomizer := TFormRandomizer.Create( Application);
  Log( LC_STARTUP, 'created randomizer form, creating MIDI CC selector');
  FormSelectMidiCC := TFormSelectMidiCC.Create( Application);
  Log( LC_STARTUP, 'Created MIDI CC selector, creating looks');
  aLooksFolderName    := Format( '%slooks', [ IncludeTrailingPathDelimiter( ApplicationPath)], AppLocale);
  aDefaultsFolderName := Format( '%slooks\default', [ IncludeTrailingPathDelimiter( ApplicationPath)], AppLocale);
  FLooksFolder        := aDefaultsFolderName;

  if   not System.SysUtils.DirectoryExists( aLooksFolderName)
  then begin
    LogFmt( LC_STARTUP, 'looks folder (%s) not found, trying to create it', [ aLooksFolderName]);

    try
      System.SysUtils.ForceDirectories( aLooksFolderName);
      Log( LC_STARTUP, 'looks folder was created');
    except
      on E: Exception
      do LogFmt( LC_STARTUP, 'Could not create looks folder, error: %s', [ E.ToString]);
    end
  end;

  if   System.SysUtils.DirectoryExists( aLooksFolderName)
  then begin
    LogFmt( LC_STARTUP, 'looks folder (%s) found, checking for default theme', [ aLooksFolderName]);

    if   System.SysUtils.DirectoryExists( aDefaultsFolderName)
    then LogFmt( LC_STARTUP, 'default theme folder (%s) found, done', [ aDefaultsFolderName])
    else begin
      LogFmt( LC_STARTUP, 'default theme folder (%s) not found, trying to create it', [ aDefaultsFolderName]);

      try
        System.SysUtils.ForceDirectories( aDefaultsFolderName);
        Log( LC_STARTUP, 'default theme folder was created, done');
      except
        on E: Exception
        do LogFmt( LC_STARTUP, 'Could not create default theme folder, error: %s', [ E.ToString]);
      end
    end;
  end;

  Log( LC_STARTUP, 'Preset folders checked, creating patch reader and writer and signal specs');
  FPatchReader := TPatchReader.Create;                     // For patch reading / writing to/from file and clipboard
  FPatchWriter := TPatchWriter.Create;
  Log( LC_STARTUP, 'Created patch reader and writer and signal specs, configuring Editor');

  with PatchEditor
  do begin
    OnLog                    := DoWirePanelLog;
    PatchReader              := FPatchReader;
    PatchWriter              := FPatchWriter;
    OnHistoryChange          := DoHistoryChanged;
    OnGetFolder              := DoGetFolder;
    OnValueChanged           := DoValueChanged;
    OnTextChanged            := DoTextChanged;
    OnRecompile              := DoRecompile;
    OnShowPoup               := DoShowPopupMenu;
    OnUnFocus                := DoUnFocus;
    OnLoadDataGraph          := DoLoadDataGraph;
    OnSaveDataGraph          := DoSaveDataGraph;
    OnLoadGridControl        := DoLoadGridControl;
    OnSaveGridControl        := DoSaveGridControl;
    OnActiveVariationChanged := DoActiveVariationChanged;
    OnLoadCaptions           := DoLoadCaptions;
    OnEditHistoryChanged     := DoEditHistoryChanged;
    OnModulesRenamed         := DoModulesRenamed;
    OnModulesRenamedFriendly := DoModulesRenamedFriendly;
    OnAudioChanged           := DoAudioChanged;
    OnBeginStateChange       := DoBeginStateChange;
    OnEndStateChange         := DoEndStateChange;
    Title                    := 'no name';
    MaxHistory               := 100;
  end;

  Log( LC_STARTUP, 'Configured Editor, creating RecentSourceFiles');
  FRecentSourceFiles := TRecentStrings.Create( 20);
  with PageControlMain                             // Hide outer level tabs
  do begin
    for i := 0 to PageCount - 1
    do Pages[ i].TabVisible := False;
  end;

  ViewEditor;
  Log( LC_STARTUP, 'Done here, created main form');
end;


procedure TFormDllMain.FormDestroy(Sender: TObject);
begin
  Log( LC_TERMINATE, 'Destroying main form');
  Log( LC_TERMINATE, 'Destroying synth patch');

  if   not FIniSaved
  and  FIniLoaded
  then SaveIni( IniFileName, False);

  FIniSaved := True;

  if   Assigned( FSynthesizer)
  then begin
    FSynthesizer.OnSendOSCString := nil;
  end;

  FreeAndNil( FSynthesizer);

  Log( LC_TERMINATE, 'Destroying audio devices');

  Log( LC_TERMINATE, 'Destroying RecentSourceFiles');
  FreeAndNil( FRecentSourceFiles);
  Log( LC_TERMINATE, 'Cleaning up DLL stuff');

  // Clean up DLL stuff
  //
  try
    if   FFFTWAvailable
    then begin
      fftw_cleanup_threads;
      FFFTWAvailable := False;
    end;
  except
    on E: Exception
    do KilledException( E);
  end;
  //
  try
    if   FESpeakAvailable
    then FreeAndNil( FESpeaker);
  except
    on E: Exception
    do KilledException( E);
  end;
  //
  // end Clean up DLL stuff

  Log( LC_TERMINATE, 'Cleaned up DLL stuff, free the logger, MIDI reciever/transmitter, OSC stuff, Talkie stuff etc.');
  FreeAndNil( FSavedLogs      );
  FreeAndNil( FMazeList       );
  FreeAndNil( FSapiVoiceNames );
  FreeAndNil( FSapiRateNames  );
  TalkieDestroyPhrases;

  Log( LC_TERMINATE, 'Saving last patch');
  try
    SaveLastPatch;
  except
    on E: Exception
    do begin
      LogFmt( LC_TERMINATE, 'Error saving last patch: %s', [ E.Message]);
      KilledException( E);
    end;
  end;

  Log( LC_TERMINATE, 'Main form destroyed, bye ...');
end;

procedure TFormDllMain.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if   Shift = []
  then begin
    if   Key = VK_APPS
    then begin
      if   HandleContextMenu( FindControlUnderMouse)
      then Key := 0;
    end
  end;
end;

procedure TFormDllMain.FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint;
  var Handled: Boolean);
var
  aMsg  : Cardinal;
  aCode : Cardinal;
  i     : Integer;
  n     : Integer;
begin
  MousePos := ScreenToClient( MousePos);

  if   ( MousePos.X > PatchEditor.Left                     )
  and  ( MousePos.Y > PatchEditor.Top                      )
  and  ( MousePos.X < PatchEditor.Left + PatchEditor.Width )
  and  ( MousePos.Y < PatchEditor.Top  + PatchEditor.Height)
  then begin
    if   ssShift in Shift
    then aMsg := WM_HSCROLL
    else aMsg := WM_VSCROLL;

    if   WheelDelta < 0
    then aCode := SB_LINEDOWN
    else aCode := SB_LINEUP;

    n := Mouse.WheelScrollLines;

    for i:= 1 to n
    do  PatchEditor.Perform( aMsg, aCode, 0);

    PatchEditor.Perform( aMsg, SB_ENDSCROLL, 0);
    Handled := True;
  end;
end;

procedure TFormDllMain.FormShow(Sender: TObject);
var
  aColor         : TColor;
begin
  FInFormShow := True;

  try
    Log( LC_STARTUP, 'TFormWren.FormShow started, handle saved logs');

    try
      ClearLog;                                        // Clear log window
      FLogToFile := False;                             // Do not log old logs to the log file, they are in it already

      try
        CanLog := True;                                // This will dump FSavedLogs to to the log view
      finally
        FLogToFile := True;
      end;
    except
      on E: Exception
      do KilledException( E);
    end;

    Log( LC_STARTUP, 'Saved logs written to log view');
    PatchName      := '';
    SourceFileName := '';

    Log( LC_STARTUP, 'Set default values');
    Width                 := 1322;                // Default form width
    Height                :=  700;                // Default form height
    ModuleSelector.Width := 389;                  // By default show everything
    FCurvedLines              := False;        CurvedLines              := True;
    FControlMode              := dmHorizontal; ControlMode              := dmCircular;
    FUseConfigHints           := False;        UseConfigHints           := True;
    FUseModuleTitleHints      := False;        UseModuleTitleHints      := True;
    FUsePatchHints            := False;        UsePatchHints            := True;
    FPolyphony                := 2;            Polyphony                := 1;
    FWireThickness            := 2;            WireThickness            := 1;
    OffsetLeft                := 6;
    OffsetTop                 := 6;
    FUseWheelOnKnobs          := False;        UseWheelOnKnobs          := True;
    FWheelSensitivity         := 1;            WheelSensitivity         := 50;
    FUseConnectorBorders      := False;        UseConnectorBorders      := True;
    FUseTypedConnectorBorders := False;        UseTypedConnectorBorders := True;
    FUseNoteNames             := True;         UseNoteNames             := False;
    ModuleColor               := clNone;       ModuleColor              := clWhite;
    ConnectorBorderColor      := clNone;       ConnectorBorderColor     := clWhite;
    SelectorColor             := clNone;       SelectorColor            := clGray;
    SelectorBorderColor       := clNone;       SelectorBorderColor      := clYellow;
    SelectorBorderColorS      := clNone;       SelectorBorderColorS     := clWhite;
    EditFontColor             := clNone;       EditFontColor            := clWhite;
    DisplayColor              := clNone;       DisplayColor             := clGray;
    DisplayBorderColor        := clNone;       DisplayBorderColor       := clSilver;
    KnobMIDIColor             := clNone;       KnobMIDIColor            := clBlue;
    KnobFocusColor            := clNone;       KnobFocusColor           := clRed;
    KnobMIDIFocusColor        := clNone;       KnobMIDIFocusColor       := clFuchsia;
    ViewerColor               := clNone;       ViewerColor              := clGray;
    ViewerBorderColor         := clNone;       ViewerBorderColor        := clGray;
    ViewerLineColor           := clNone;       ViewerLineColor          := clWhite;
    ViewerFillColor           := clNone;       ViewerFillColor          := $009E9E9E;
    ScopeLineColor            := clNone;       ScopeLineColor           := clLime;
    IndBarPeakColor           := clNone;       IndBarPeakColor          := clWhite;
    IndBarValeyColor          := clNone;       IndBarValeyColor         := clBlack;
    GroupIOColor              := clNone;       GroupIOColor             := clSilver;
    GroupOSCColor             := clNone;       GroupOSCColor            := clRed;
    GroupLFOColor             := clNone;       GroupLFOColor            := clLime;
    GroupEnvColor             := clNone;       GroupEnvColor            := clYellow;
    GroupFilterColor          := clNone;       GroupFilterColor         := clBlue;
    GroupSwitchColor          := clNone;       GroupSwitchColor         := clFuchsia;
    GroupMixColor             := clNone;       GroupMixColor            := clAqua;
    GroupControlColor         := clNone;       GroupControlColor        := clTeal;
    GroupNoteColor            := clNone;       GroupNoteColor           := $000080FF; // Orange
    GroupMathColor            := clNone;       GroupMathColor           := clWhite;
    GroupLogicColor           := clNone;       GroupLogicColor          := clGreen;
    GroupGeneratorColor       := clNone;       GroupGeneratorColor      := $009882FF; // Pink
    GroupSeqColor             := clNone;       GroupSeqColor            := clPurple;
    GroupFXColor              := clNone;       GroupFXColor             := clMaroon;
    GroupDelayColor           := clNone;       GroupDelayColor          := clTeal;
    GroupVoiceColor           := clNone;       GroupVoiceColor          := $00404080; // Brown
    GroupUtilityColor         := clNone;       GroupUtilityColor        := clBlack;
    AudioRateColor            := clNone;       AudioRateColor           := clRed;
    ControlRateColor          := clNone;       ControlRateColor         := clBlue;
    ControlRateLogicColor     := clNone;       ControlRateLogicColor    := clLime;
    LogicColor                := clNone;       LogicColor               := clYellow;
    UnconnectedWireColor      := clNone;       UnconnectedWireColor     := clAqua;
    WireHighlightColor        := clNone;       WireHighlightColor       := clWhite;
    FEditorBackgroundColor    := clNone;       EditorBackgroundColor    := $00313131;
    NewPatchNameTemplate      := '$D $T $N';
    ModuleOpacity             := 195;
    ModuleFlatness            := False;
    ModuleTexture             := True;
    FStandardColors           := False;        StandardColors           := True;
    SynthName                 := '-';          SynthName                := 'wren';
    FLightsActive             := False;        LightsActive             := True;
    FReloadLastPatch          := False;        ReloadLastPatch          := True;
    TextWriterPrefix          := '-';          TextWriterPrefix         := '';
    PrependPrefix             := False;        PrependPrefix            := True;
    UseTextWriter             := True;         UseTextWriter            := False;

    Log( LC_STARTUP, 'Clear some user interface stuff');
    FLightsRate                            :=   0;
    KnobsSmallKnob_LightsRate.KnobPosition :=   0;
    KnobsSmallKnob_LightsRate.KnobPosition := 156;
    LabelProgramVersion.Caption := Format( 'program version: %s %s', [ GetFileVersion, OsString], AppLocale);
    LabelPatchVersion  .Caption := Format( 'patch version: %d'     , [ KnobsPatchVersion       ], AppLocale);

    // Load user prefs when they are available

    if   FileExists( IniFileName)
    then begin
      LogFmt( LC_STARTUP, 'Trying to load user prefs from ''%s''', [ IniFileName]);
      LoadIni( IniFileName, False);
    end
    else if FileExists( WrenIniFileName)
    then begin
      LogFmt( LC_STARTUP, 'trying to load user prefs from ''%s''', [ WrenIniFileName]);
      LoadIni( IniFileName, False)
    end
    else begin
      LogFmt( LC_STARTUP, 'No on disk user prefs found, creating initial prefs in ''%s''', [ IniFileName]);
      AutoRun    := True;
      FIniLoaded := True;
      SaveIni( IniFileName, False);   // Save default settings
      AutoRun    := False;            // To be reloaded from ini file

      if   FileExists( IniFileName)   // and load them
      then begin
        Log( LC_STARTUP, 'Trying to load freshly created user prefs');
        LoadIni( IniFileName, False);
        Log( LC_STARTUP, 'User prefs loaded');
      end
      else Log( LC_STARTUP, 'Could read suppusedly created ini file, probaly could not create it');
    end;

    Log( LC_STARTUP, 'Fixing color maps');
    FixColorMaps;

    with FormStore
    do begin
      FormStore.OnDesignerLog := DoDesignerLog;                     // Set a logger for the module store
      PopulateSelector( ModuleSelector, FLooksFolder);              // Poplulate module selector from the module store
      PopulateMenu( MenuItemInsertmodule, DoEditorMenuAddModule);   // Populate the editor context menu as well
      FixTabColors;                                                 // Obtain tab colors for the module selector
    end;


    // FormStore.RegisterModules;

    LogFmt( LC_STARTUP, 'Designer window fixed, load the theme bitmaps [from %s]', [ FLooksFolder]);
    LoadModuleGlyphs;

    if   Assigned( FormStore)
    then FormStore.LoadControlGlyphs( FLooksFolder);

    if   Assigned( PatchEditor)
    then begin
      Log( LC_STARTUP, 'Theme settings completed, colorize the editor');
      aColor := GetThemedPanelColor;

      if   aColor <> clNone
      then PatchEditor.Color := aColor;

//      Log( LC_STARTUP, 'Colorized the editor, initializing randomizer');
//
//      if   Assigned( FormRandomizer)
//      then begin
//        FormRandomizer.PatchEditor        := PatchEditor;
//        FormRandomizer.Logger             := LogRandomizer;
//        FormRandomizer.OnLiveMorphChanged := DoRandomizerLiveMorphChanged;
//        FormRandomizer.OnSendToEditor     := DoRandomizerSendToEditor;
//        FormRandomizer.OnLoadFromEditor   := DoRandomizerLoadFromEditor;
//        FormRandomizer.OnPatchRandomize   := DoPatchRandomize;
//        FormRandomizer.OnPatchMutate      := DoPatchMutate;
//        FormRandomizer.OnPatchMate        := DoPatchMate;
//        FormRandomizer.OnPatchMorph       := DoPatchMorph;
//        FormRandomizer.OnWormSelected     := DoWormSelected;
//        FormRandomizer.OnWormDropped      := DoWormDropped;
//
//        Log( LC_STARTUP, 'Randomizer initialized, checking help files');
//      end
//      else Log( LC_STARTUP, 'BUG: FormRandomizer is not available, could not initialize it');
    end
    else Log( LC_STARTUP, 'BUG: PatchEditor is NIL - could not colorize editor, could not set up randomizer');

//    if   MustMakeDocs
//    then begin
//      Log( LC_STARTUP, 'Help files are not up to date, generating new ones from designer window');
//      try
//        MakeDocs( DocsPath);
//        Log( LC_STARTUP, 'Generated help files');
//      except
//        on E: Exception
//        do LogFmt( LC_STARTUP, 'Help file generation failed, reason: ''%s''', [ E.ToString]);
//      end;
//    end
//    else Log( LC_STARTUP, 'Help files were up to date, not making new ones');

    Log( LC_STARTUP, 'Help files checked, some minor stuff now ...');
    DoUnfocus( Self);                         // Set keyboard and mouse wheel focus to something harmless .. if possible
    Log( LC_STARTUP, '... start Stats timer');
    TimerStatsUpdate.Enabled := True;
    Log( LC_STARTUP, 'trying to load a patch, either nothing the last past used or one specified on the command line');

    try
      if   ( ParamCount >= 1)
      and  FileExists( ParamStr( 1))
      then Begin
        SourceFileName := ParamStr( 1);
        LogFmt( LC_STARTUP, 'Opening patch passed on command line [%s]', [ SourceFileName]);
        DoLoadPatch( SourceFileName, True);
        PatchEditor.ClearUndoRedo;            // This prevents undo to a blank patch when undo after load is in effect
      end
      else if ReloadLastPatch
      and     FileExists( SourceFileName)
      then begin
        LogFmt( LC_STARTUP, 'Reopening last used patch [%s]', [ SourceFileName]);
        DoLoadPatch( SourceFileName, True);
        PatchEditor.ClearUndoRedo;            // This prevents undo to a blank patch when undo after load is in effect
      end
      else begin
        LogFmt( LC_STARTUP, 'Could not find last used patch [%s] or reload was not specified - starting a new patch', [ SourceFileName]);
        NewPatch;
      end;

      Log( LC_STARTUP, 'patch loading done OK');
    except
      on E: Exception
      do LogFmt( LC_STARTUP, 'patch loading failed for reason: %s', [ E.ToString]);
    end;

    Log( LC_STARTUP, 'Allow audio from here on');
    //FCanCreateAudio := True;

    PatchEditor.FixAllSynthParams( False);  // todo: raaaaar .. this is needed ... non comprendo .. anyway .. otherwise stuff doesnt properly start up on the initial patch ...

    if   FRegErrors
    then begin
      MessageDlg( 'There were module registration errors', mtError, [ mbOK], 0);
      PageControlMain.ActivePage := TabsheetDebug;
      Log( LC_BUG, '... done with TFormWren.FormShow, ****>>>> THERE WERE MODULE REGISTRATION  ERRORS <<<<****');
    end
    else Log( LC_STARTUP, '... done with TFormWren.FormShow, all seems OK');
  finally
    FInFormShow := False;
    FreeAndNil( FormSplash);
  end;

  FormDllMain := Self;
end;

function TFormDllMain.ModuleSelectorGetGlyph(const aSender: TObject; const aBitmapsFolder, aName: string): TBitmap;
begin
  Result := FormStore.FindBitmap( aName);
end;

procedure TFormDllMain.ModuleSelectorModuleButtonClick(const aSender: TObject; aModuleclass: Integer);
begin
  AddModule( aModuleclass);
end;

procedure TFormDllMain.Redwires1Click(Sender: TObject);
begin
  SetWireColor( FPopupSender as TKnobsConnector, clRed);
end;


// /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    procedure   TFormDllMain.rlg( const aMsg: string);                     // short for RegistrationLog ...
    begin
      Log( LC_MODULE_REG, aMsg);
      FRegErrors := True;
    end;

    procedure   TFormDllMain.RegisterModuleTypes;
    begin
      // Couple editor module IDs (design) to synth module types (run-time)
      // Audio in and output need pre-defined module type IDs
      // (all modules without run-time semantics are skipped here (like the panels from the utils tab))

      // Note: module types 1xx are I/O modules, the recorder maybe does not belong in that range ...

      RegisterModuleType( MTYPE_AUDIO_IN , TModInput , rlg); RegisterModuleType( MTYPE_AUDIO_OUT, TModOutput , rlg);

      RegisterModuleType(  110, TModRecorder         , rlg);

      RegisterModuleType(  201, TModXFade            , rlg);  RegisterModuleType(  202, TModPan              , rlg);
      RegisterModuleType(  203, TModMixM4to1         , rlg);  RegisterModuleType(  204, TModMixS2to1         , rlg);
      RegisterModuleType(  205, TModMixM6to6         , rlg);  RegisterModuleType(  206, TModMixT1to1         , rlg);
      RegisterModuleType(  207, TModMixAudio         , rlg);  RegisterModuleType(  208, TModMixQ1to1         , rlg);
      RegisterModuleType(  209, TModMixM16to1        , rlg);  RegisterModuleType(  210, TModMixMM16to1       , rlg);
      RegisterModuleType(  211, TModMixM3to1         , rlg);  RegisterModuleType(  212, TModVolume           , rlg);
      RegisterModuleType(  213, TModMixS5to1         , rlg);  RegisterModuleType(  214, TModMixMS8to1        , rlg);
      RegisterModuleType(  215, TModMixQS2to1        , rlg);  RegisterModuleType(  216, TModMixQS1to1        , rlg);
      RegisterModuleType(  217, TModQuadPan          , rlg);  RegisterModuleType(  218, TModXFadeStereo      , rlg);
      RegisterModuleType(  219, TModStereoPan        , rlg);

      RegisterModuleType(  301, TModConstant         , rlg);  RegisterModuleType(  302, TModInverter         , rlg);
      RegisterModuleType(  303, TModAdder            , rlg);  RegisterModuleType(  304, TModMultiplier       , rlg);
      RegisterModuleType(  305, TModAmplifier        , rlg);  RegisterModuleType(  306, TModScaler           , rlg);
      RegisterModuleType(  307, TModQuantize         , rlg);  RegisterModuleType(  308, TModRatio            , rlg);
      RegisterModuleType(  309, TModRotator          , rlg);  RegisterModuleType(  310, TModRectifier        , rlg);
      RegisterModuleType(  311, TModMinMax           , rlg);  RegisterModuleType(  312, TModMultiMult        , rlg);
      RegisterModuleType(  313, TModNoteToDelay      , rlg);  RegisterModuleType(  314, TModZeroCounter      , rlg);
      RegisterModuleType(  315, TModMedian           , rlg);  RegisterModuleType(  316, TModScala            , rlg);
      RegisterModuleType(  317, TModHardClip         , rlg);  RegisterModuleType(  318, TModTransInv         , rlg);
      RegisterModuleType(  319, TModPad              , rlg);  RegisterModuleType(  320, TModEuclideanScale   , rlg);
      RegisterModuleType(  321, TModSpiral           , rlg);  RegisterModuleType(  322, TModRangeConverter   , rlg);
      RegisterModuleType(  323, TModLogisticMap      , rlg);  RegisterModuleType(  324, TModWaveWiper        , rlg);
      RegisterModuleType(  325, TModWaveWrapper      , rlg);  RegisterModuleType(  326, TModChebGen          , rlg);
      RegisterModuleType(  327, TModPoetry           , rlg);  RegisterModuleType(  328, TModNoteQuantize     , rlg);
      RegisterModuleType(  329, TModLevelConverter   , rlg);  RegisterModuleType(  330, TModReciprocal       , rlg);
      RegisterModuleType(  331, TModDivide           , rlg);  RegisterModuleType(  332, TModMonadic          , rlg);
      RegisterModuleType(  333, TModDyadic           , rlg);  RegisterModuleType(  334, TModMultipleMult     , rlg);
      RegisterModuleType(  335, TModScaleQuantizer   , rlg);  RegisterModuleType(  336, TModModQuantizer     , rlg);
      RegisterModuleType(  337, TModSlew             , rlg);  RegisterModuleType(  338, TModMixingScaler     , rlg);
      RegisterModuleType(  339, TModRMS              , rlg);  RegisterModuleType(  340, TModCubicClip        , rlg);
      RegisterModuleType(  341, TModWavolver         , rlg);  RegisterModuleType(  342, TModSawMill          , rlg);
      RegisterModuleType(  343, TModTypeFlip         , rlg);  RegisterModuleType(  344, TModFastToSlow       , rlg);
      RegisterModuleType(  345, TModPolar            , rlg);  RegisterModuleType(  346, TModMConst           , rlg);
      RegisterModuleType(  347, TModRectangular      , rlg);  RegisterModuleType(  348, TModRandomScale      , rlg);
      RegisterModuleType(  349, TModArpeggiator      , rlg);  RegisterModuleType(  350, TModMarkov           , rlg);
      RegisterModuleType(  351, TModAmMod            , rlg);  RegisterModuleType(  352, TModModControl       , rlg);
      RegisterModuleType(  353, TModTranspose        , rlg);  RegisterModuleType(  354, TModShaper           , rlg);
      RegisterModuleType(  355, TModPropScaler       , rlg);  RegisterModuleType(  356, TModAnalogXor        , rlg);
      RegisterModuleType(  357, TModCuClipStereo     , rlg);  RegisterModuleType(  358, TModValues           , rlg);
      RegisterModuleType(  359, TModMatrixMult       , rlg);  RegisterModuleType(  360, TModChladniControl   , rlg);
      RegisterModuleType(  361, TModSlowToFast       , rlg);  RegisterModuleType(  362, TModRotator2         , rlg);
      RegisterModuleType(  363, TModAutoQuantizer    , rlg);  RegisterModuleType(  364, TModNoteSplit        , rlg);
      RegisterModuleType(  365, TModNoteMerge        , rlg);  RegisterModuleType(  366, TModFindNearest      , rlg);
      RegisterModuleType(  367, TModChangeDetector   , rlg);  RegisterModuleType(  368, TModMinMaxMix        , rlg);
      RegisterModuleType(  369, TModPhaseSplitter    , rlg);  RegisterModuleType(  370, TModSieve            , rlg);
      RegisterModuleType(  371, TModClockedSieve     , rlg);  RegisterModuleType(  372, TModAbsAdder         , rlg);
      RegisterModuleType(  373, TModLimiter          , rlg);  RegisterModuleType(  374, TModMinMaxRange      , rlg);
      RegisterModuleType(  375, TModEqDivQuantizer   , rlg);  RegisterModuleType(  376, TModRateMult         , rlg);
      RegisterModuleType(  377, TModScaledClip       , rlg);  RegisterModuleType(  378, TModNoop             , rlg);
      RegisterModuleType(  379, TModRateConverter    , rlg);

      RegisterModuleType(  401, TModLfo              , rlg);  RegisterModuleType(  402, TModSquareLfo        , rlg);
      RegisterModuleType(  403, TModNoiseLfo         , rlg);  RegisterModuleType(  404, TModRandomWalkLfo    , rlg);
      RegisterModuleType(  405, TModLfoTrig          , rlg);  RegisterModuleType(  406, TModSquareLfoTrig    , rlg);
      RegisterModuleType(  407, TModNoiseLfoTrig     , rlg);  RegisterModuleType(  408, TModRandSig          , rlg);
      RegisterModuleType(  409, TModPulses           , rlg);  RegisterModuleType(  410, TModAttractorLfo     , rlg);
      RegisterModuleType(  411, TModClockGen         , rlg);  RegisterModuleType(  412, TModTod              , rlg);
      RegisterModuleType(  413, TModSquareSineLfo    , rlg);  RegisterModuleType(  414, TModLfoMultiPhase    , rlg);
      RegisterModuleType(  415, TModRandSigs         , rlg);  RegisterModuleType(  416, TModVanDerPolLfo     , rlg);
      RegisterModuleType(  417, TModFastLfo          , rlg);

      RegisterModuleType(  501, TModOsc              , rlg);  RegisterModuleType(  502, TModSquare           , rlg);
      RegisterModuleType(  503, TModNoise            , rlg);  RegisterModuleType(  504, TModOscTrig          , rlg);
      RegisterModuleType(  505, TModSquareTrig       , rlg);  RegisterModuleType(  506, TModNoiseTrig        , rlg);
      RegisterModuleType(  507, TModTerraGen         , rlg);  RegisterModuleType(  508, TModPluck            , rlg);
      RegisterModuleType(  509, TModKS               , rlg);  RegisterModuleType(  510, TModTalkie           , rlg);
      RegisterModuleType(  511, TModAttractor        , rlg);  RegisterModuleType(  512, TModTss              , rlg);
      RegisterModuleType(  513, TModSong             , rlg);  RegisterModuleType(  514, TModVosim            , rlg);
      RegisterModuleType(  515, TModPerlinNoise      , rlg);  RegisterModuleType(  516, TModPerlinNoiseTrig  , rlg);
      RegisterModuleType(  517, TModPerlinTone       , rlg);  RegisterModuleType(  518, TModPhasor           , rlg);
      RegisterModuleType(  519, TModAudioGraph       , rlg);  RegisterModuleType(  520, TModMultiOsc         , rlg);
      RegisterModuleType(  521, TModModOsc           , rlg);  RegisterModuleType(  522, TModMultiSine        , rlg);
      RegisterModuleType(  523, TModPdOsc            , rlg);  RegisterModuleType(  524, TModMasterOsc        , rlg);
      RegisterModuleType(  525, TModMultiPhaseOsc    , rlg);  RegisterModuleType(  526, TModVanDerPol        , rlg);
      RegisterModuleType(  527, TModChladnicOsc      , rlg);  RegisterModuleType(  528, TModSapiVoice        , rlg);
      RegisterModuleType(  529, TModESpeakVoice      , rlg);  RegisterModuleType(  530, TModDTMF             , rlg);
      RegisterModuleType(  531, TModPadSynth         , rlg);

      RegisterModuleType(  601, TModNot              , rlg);  RegisterModuleType(  602, TModGate             , rlg);
      RegisterModuleType(  603, TModDivider          , rlg);  RegisterModuleType(  604, TModDFlipFlop        , rlg);
      RegisterModuleType(  605, TModRSFlipFlop       , rlg);  RegisterModuleType(  606, TModCounter          , rlg);
      RegisterModuleType(  607, TModAdc              , rlg);  RegisterModuleType(  608, TModDac              , rlg);
      RegisterModuleType(  609, TModPhaseDetect      , rlg);  RegisterModuleType(  610, TModCompare          , rlg);
      RegisterModuleType(  611, TModLogicSelector    , rlg);  RegisterModuleType(  612, TModProgDivider      , rlg);
      RegisterModuleType(  613, TModMultiCompare     , rlg);  RegisterModuleType(  614, TModPulseDelay       , rlg);
      RegisterModuleType(  615, TModRungler          , rlg);  RegisterModuleType(  616, TModFixedDiv         , rlg);
      RegisterModuleType(  617, TModPulseSync        , rlg);  RegisterModuleType(  618, TModPrimeDiv         , rlg);
      RegisterModuleType(  619, TModPulseSkip        , rlg);  RegisterModuleType(  620, TModShifter          , rlg);
      RegisterModuleType(  621, TModLFSR             , rlg);  RegisterModuleType(  622, TModStack            , rlg);
      RegisterModuleType(  623, TModMultiGate        , rlg);  RegisterModuleType(  624, TModBranches         , rlg);

      RegisterModuleType(  701, TModDelay            , rlg);  RegisterModuleType(  702, TModWavePlayer       , rlg);
      RegisterModuleType(  703, TModReverb           , rlg);  RegisterModuleType(  704, TModFlangeChorus     , rlg);
      RegisterModuleType(  705, TModPitchChange      , rlg);  RegisterModuleType(  706, TModClockedDelay     , rlg);
      RegisterModuleType(  707, TModGVerb            , rlg);  RegisterModuleType(  708, TModDelay8           , rlg);
      RegisterModuleType(  709, TModRndGranulator    , rlg);  RegisterModuleType(  710, TModGranulator       , rlg);
      RegisterModuleType(  711, TModSimpleGranulator , rlg);  RegisterModuleType(  712, TModClockedDelay16   , rlg);
      RegisterModuleType(  713, TModAllPass          , rlg);  RegisterModuleType(  714, TModLooper           , rlg);
      RegisterModuleType(  715, TModStereoLooper     , rlg);  RegisterModuleType(  716, TModDelayMod         , rlg);
      RegisterModuleType(  717, TModDelayStereo      , rlg);  RegisterModuleType(  718, TModDelayMix         , rlg);
      RegisterModuleType(  719, TModGrainDelay       , rlg);  RegisterModuleType(  720, TModWavePlayTrig     , rlg);
      RegisterModuleType(  721, TModWavePlayTrig2    , rlg); {RegisterModuleType(  722, TModConvolver        , rlg);}
      RegisterModuleType(  723, TModLoopPlayer       , rlg);  RegisterModuleType(  724, TModTapeRecorder     , rlg);

      RegisterModuleType(  801, TModSampleAndHold    , rlg);  RegisterModuleType(  802, TModTrackAndHold     , rlg);
      RegisterModuleType(  803, TModSwitch2to1       , rlg);  RegisterModuleType(  804, TModMultiSAndH       , rlg);
      RegisterModuleType(  805, TModMux              , rlg);  RegisterModuleType(  806, TModDemux            , rlg);
      RegisterModuleType(  807, TModDigiMux          , rlg);  RegisterModuleType(  808, TModDigiDemux        , rlg);
      RegisterModuleType(  809, TModSerPar           , rlg);  RegisterModuleType(  810, TModValueSync        , rlg);

      RegisterModuleType(  901, TModEnvAR            , rlg);  RegisterModuleType(  902, TModEnvARRetrig      , rlg);
      RegisterModuleType(  903, TModEnvAHD           , rlg);  RegisterModuleType(  904, TModEnvADSR          , rlg);
      RegisterModuleType(  905, TModEnvelope         , rlg);  RegisterModuleType(  906, TModCompressor       , rlg);
      RegisterModuleType(  907, TModVocoder          , rlg);  RegisterModuleType(  908, TModEnvControl       , rlg);
      RegisterModuleType(  909, TModConvoder         , rlg);  RegisterModuleType(  910, TModPitchShift       , rlg);
      RegisterModuleType(  911, TModMultiEnvelope    , rlg);  RegisterModuleType(  912, TModStereoWiden      , rlg);
      RegisterModuleType(  913, TModPhaseDistort     , rlg);  RegisterModuleType(  914, TModDetector         , rlg);
      RegisterModuleType(  915, TModSmear            , rlg);  RegisterModuleType(  916, TModStereoNarrow     , rlg);

      RegisterModuleType( 1001, TModAverage          , rlg);  RegisterModuleType( 1002, TModSVF              , rlg);
      RegisterModuleType( 1003, TModIntDif           , rlg);  RegisterModuleType( 1004, TModFormant          , rlg);
      RegisterModuleType( 1005, TModPinkFilter       , rlg);  RegisterModuleType( 1006, TModResonator        , rlg);
      RegisterModuleType( 1007, TModTritoneBank      , rlg);  RegisterModuleType( 1008, TModThirdBank        , rlg);
      RegisterModuleType( 1009, TModTritoneSplitter  , rlg);  RegisterModuleType( 1010, TModThirdSplitter    , rlg);
      RegisterModuleType( 1011, TModModal            , rlg);  RegisterModuleType( 1012, TModFormant2         , rlg);
      RegisterModuleType( 1013, TModFilter6dB        , rlg);  RegisterModuleType( 1014, TModFilter6dBBP      , rlg);
      RegisterModuleType( 1015, TModFilter6dBBPS     , rlg);  RegisterModuleType( 1016, TModTiltFilter       , rlg);
      RegisterModuleType( 1017, TModMoogFilter       , rlg);  RegisterModuleType( 1018, TModDCBlock          , rlg);
      RegisterModuleType( 1019, TModFilter6dBBPSM    , rlg);  RegisterModuleType( 1020, TModTritoneCombiner  , rlg);
      RegisterModuleType( 1021, TModThirdCombiner    , rlg);  RegisterModuleType( 1022, TModTiltFilterStereo , rlg);
      RegisterModuleType( 1023, TModModal2           , rlg);  RegisterModuleType( 1024, TModChladni          , rlg);
      RegisterModuleType( 1025, TModMovingAverage    , rlg);  RegisterModuleType( 1026, TModUnclick          , rlg);
      RegisterModuleType( 1027, TModDif              , rlg);  RegisterModuleType( 1028, TModDualFilter       , rlg);
      RegisterModuleType( 1029, TModStereoDCBlock    , rlg);


      // Note: do NOT register the following module types:
      //
      //  1101 // Panel module - has no compiled equivalent (no runtime semantics)
      //  1102 // Panel module - has no compiled equivalent
      //  1103 // Panel module - has no compiled equivalent
      //  1104 // Panel module - has no compiled equivalent
      //  1105 // Panel module - has no compiled equivalent
      //  1107 // Notes module - has no compiled equivalent

      RegisterModuleType( 1106, TModDisplay          , rlg);  RegisterModuleType( 1108, TModXYScope          , rlg);
      RegisterModuleType( 1109, TModKnobs4           , rlg);  RegisterModuleType( 1110, TModSwitches4        , rlg);
      RegisterModuleType( 1111, TModButtons4         , rlg);  RegisterModuleType( 1112, TModValue            , rlg);
      RegisterModuleType( 1113, TModTapper           , rlg);  RegisterModuleType( 1114, TModCount            , rlg);
      RegisterModuleType( 1115, TModDataGraph        , rlg);  RegisterModuleType( 1116, TModFreqCount        , rlg);
      RegisterModuleType( 1117, TModTextWriter       , rlg);

      RegisterModuleType( 1201, TModSequencer        , rlg);  RegisterModuleType( 1202, TModTextSequencer    , rlg);
      RegisterModuleType( 1203, TModSeqStep          , rlg);  RegisterModuleType( 1204, TModSeqClockStep     , rlg);
      RegisterModuleType( 1205, TModEuclideanRhythm  , rlg);  RegisterModuleType( 1206, TModCsvData          , rlg);
      RegisterModuleType( 1207, TModKleeBit          , rlg);  RegisterModuleType( 1208, TModSeqRandom        , rlg);
      RegisterModuleType( 1209, TModMorse            , rlg);  RegisterModuleType( 1210, TModGateSeq          , rlg);
      RegisterModuleType( 1211, TModSeq16            , rlg);  RegisterModuleType( 1212, TModSeqSeq           , rlg);
      RegisterModuleType( 1213, TModPattern          , rlg);  RegisterModuleType( 1214, TModProbSequencer    , rlg);
      RegisterModuleType( 1215, TModLut              , rlg);  RegisterModuleType( 1216, TModSeqChord         , rlg);
      RegisterModuleType( 1217, TModSeqRandVal       , rlg);  RegisterModuleType( 1218, TModRewriter         , rlg);
      RegisterModuleType( 1219, TModRewriterNote     , rlg);  RegisterModuleType( 1220, TModTuneSmithy       , rlg);
      RegisterModuleType( 1221, TModCellAut          , rlg);  RegisterModuleType( 1222, TModLifeSeq          , rlg);
      RegisterModuleType( 1223, TModSeqPattern       , rlg);  RegisterModuleType( 1224, TModSeqValues        , rlg);
      RegisterModuleType( 1225, TModAmuse            , rlg);  RegisterModuleType( 1226, TModAnalogStack      , rlg);
      RegisterModuleType( 1227, TModSwanSong         , rlg);  RegisterModuleType( 1228, TModCellAut2         , rlg);
      RegisterModuleType( 1229, TModSeq32            , rlg);  RegisterModuleType( 1230, TModLeakyGateSeq     , rlg);
      RegisterModuleType( 1231, TModNeuralPulser     , rlg);  RegisterModuleType( 1232, TModSmallLifeSeq     , rlg);
      RegisterModuleType( 1233, TModVcps             , rlg);  RegisterModuleType( 1234, TModQueue            , rlg);
      RegisterModuleType( 1235, TModVitaPHoBium      , rlg);  RegisterModuleType( 1236, TModEconomy          , rlg);
      RegisterModuleType( 1237, TModModGateSeq       , rlg);  RegisterModuleType( 1238, TModDPendulum        , rlg);
      RegisterModuleType( 1239, TModGenePlayer       , rlg);  RegisterModuleType( 1240, TModSeq16Rep         , rlg);


      RegisterModuleType( 1301, TModMidiNoteIn       , rlg);  RegisterModuleType( 1302, TModMidiCCIn         , rlg);
      RegisterModuleType( 1303, TModMidiNoteOut      , rlg);  RegisterModuleType( 1304, TModMidiCCOut        , rlg);
      RegisterModuleType( 1305, TModMidiPlayer       , rlg);  RegisterModuleType( 1306, TModSync             , rlg);
      RegisterModuleType( 1307, TModOscMessage       , rlg);  RegisterModuleType( 1308, TModMidiMultiNoteIn  , rlg);
      RegisterModuleType( 1309, TModLiveMorph        , rlg);  RegisterModuleType( 1310, TModAuto             , rlg);
      RegisterModuleType( 1311, TModMidiPCOut        , rlg);  RegisterModuleType( 1312, TModMidiPNOut        , rlg);
      RegisterModuleType( 1313, TModMidiClockIn      , rlg);  RegisterModuleType( 1314, TModMorphControl     , rlg);
      RegisterModuleType( 1315, TModMidiSysexOut     , rlg);  RegisterModuleType( 1316, TModMidi8NoteIn      , rlg);

      RegisterModuleType( 1401, TModFreqShifter      , rlg);

      RegisterModuleType( 4001, TModHrastOsc         , rlg);  RegisterModuleType( 4002, TModHrastSVF         , rlg);
    end;




end.
